@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+
 var CalendarDeleteCommandEnabled = false;
 var CalendarNewItemsCommandEnabled = false;
 
@@ -177,7 +179,9 @@ var calendarController = {
                                 this.todo_items_selected &&
                                 this.todo_items_writable;
                 } else {
-                    attendSel = this.item_selected && this.selected_events_invitation;
+                    attendSel = this.item_selected &&
+                                this.selected_events_invitation &&
+                                this.selected_items_writable;
                 }
 
                 // Small hack, we want to hide instead of disable.
@@ -522,7 +526,7 @@ var calendarController = {
      * mode.
      */
     get offline() {
-        return getIOService().offline;
+        return Services.io.offline;
     },
 
     /**
@@ -588,8 +592,8 @@ var calendarController = {
     },
 
     /**
-     * Returns a boolean indicating if the items selected in the current view
-     * all have writable calendars.
+     * Returns a boolean indicating that at least one of the items selected
+     * in the current view has a writable calendar.
      */
     get selected_items_writable() {
         return this.writable &&
