@@ -10,126 +10,34 @@ Components.utils.import("resource://calendar/modules/calProviderUtils.jsm");
 const REGISTRY_BRANCH = "calendar.registry.";
 const DB_SCHEMA_VERSION = 10;
 
-function getPrefBranchFor(id) {
-    return (REGISTRY_BRANCH + id + ".");
-}
-
-/**
- * Helper function to flush the preferences file. If the application crashes
- * after a calendar has been created using the prefs registry, then the calendar
- * won't show up. Writing the prefs helps counteract.
- */
-function flushPrefs() {
-    Components.classes["@mozilla.org/preferences-service;1"]
-              .getService(Components.interfaces.nsIPrefService)
-              .savePrefFile(null);
-}
-
-
-/**
- * Callback object for the refresh timer. Should be called as an object, i.e
- * let foo = new timerCallback(calendar);
- *
- * @param aCalendar     The calendar to refresh on notification
- */
-function timerCallback(aCalendar) {
-    this.notify = function refreshNotify(aTimer) {
-        if (!aCalendar.getProperty("disabled") && aCalendar.canRefresh) {
-            aCalendar.refresh();
-        }
-    }
-}
-
-var gCalendarManagerAddonListener = {
-    onDisabling: function(aAddon, aNeedsRestart) {
-        if (!this.queryUninstallProvider(aAddon)) {
-            // If the addon should not be disabled, then re-enable it.
-            aAddon.userDisabled = false;
-        }
-    },
-
-    onUninstalling: function(aAddon, aNeedsRestart) {
-        if (!this.queryUninstallProvider(aAddon)) {
-            // If the addon should not be uninstalled, then cancel the uninstall.
-            aAddon.cancelUninstall();
-        }
-    },
-
-    queryUninstallProvider: function(aAddon) {
-        const uri = "chrome://calendar/content/calendar-providerUninstall-dialog.xul";
-        const features = "chrome,titlebar,resizable,modal";
-        let calMgr = cal.getCalendarManager();
-        let affectedCalendars =
-            [ calendar for each (calendar in calMgr.getCalendars({}))
-              if (calendar.providerID == aAddon.id) ];
-        if (!affectedCalendars.length) {
-            // If no calendars are affected, then everything is fine.
-            return true;
-        }
-
-        let args = { shouldUninstall: false, extension: aAddon };
-
-        // Now find a window. The best choice would be the most recent
-        // addons window, otherwise the most recent calendar window, or we
-        // create a new toplevel window.
-        let win = Services.wm.getMostRecentWindow("Extension:Manager") ||
-                  cal.getCalendarWindow();
-        if (win) {
-            win.openDialog(uri, "CalendarProviderUninstallDialog", features, args);
-        } else {
-            // Use the window watcher to open a parentless window.
-            Services.ww.openWindow(null, uri, "CalendarProviderUninstallWindow", features, args);
-        }
-
-        // Now that we are done, check if the dialog was accepted or canceled.
-        return args.shouldUninstall;
-    }
-};
-
 function calCalendarManager() {
     this.wrappedJSObject = this;
     this.mObservers = new calListenerBag(Components.interfaces.calICalendarManagerObserver);
+    this.mCalendarObservers = new calListenerBag(Components.interfaces.calIObserver);
 }
 
+const calCalendarManagerClassID = Components.ID("{f42585e7-e736-4600-985d-9624c1c51992}");
+const calCalendarManagerInterfaces = [
+    Components.interfaces.calICalendarManager,
+    Components.interfaces.calIStartupService,
+    Components.interfaces.nsIObserver,
+];
 calCalendarManager.prototype = {
-    contractID: "@mozilla.org/calendar/manager;1",
-    classDescription: "Calendar Manager",
-    classID: Components.ID("{f42585e7-e736-4600-985d-9624c1c51992}"),
+    classID: calCalendarManagerClassID,
+    QueryInterface: XPCOMUtils.generateQI(calCalendarManagerInterfaces),
+    classInfo: XPCOMUtils.generateCI({
+        classID: calCalendarManagerClassID,
+        contractID: "@mozilla.org/calendar/manager;1",
+        classDescription: "Calendar Manager",
+        interfaces: calCalendarManagerInterfaces,
+        flags: Components.interfaces.nsIClassInfo.SINGLETON
+    }),
 
-    getInterfaces: function (count) {
-        let ifaces = [
-            Components.interfaces.nsISupports,
-            Components.interfaces.calICalendarManager,
-            Components.interfaces.calIStartupService,
-            Components.interfaces.nsIObserver,
-            Components.interfaces.nsIClassInfo
-        ];
-        count.value = ifaces.length;
-        return ifaces;
-    },
-
-    getHelperForLanguage: function (language) {
-        return null;
-    },
-
-    implementationLanguage: Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT,
-    flags: Components.interfaces.nsIClassInfo.SINGLETON,
-    QueryInterface: function (aIID) {
-        return cal.doQueryInterface(this, calCalendarManager.prototype, aIID, null, this);
-    },
-
-    get networkCalendarCount() {
-        return this.mNetworkCalendarCount;
-    },
-
-    get readOnlyCalendarCount() {
-        return this.mReadonlyCalendarCount;
-    },
-
-    get calendarCount() {
-        return this.mCalendarCount;
-    },
+    get networkCalendarCount() this.mNetworkCalendarCount,
+    get readOnlyCalendarCount() this.mReadonlyCalendarCount,
+    get calendarCount() this.mCalendarCount,
 
+    // calIStartupService:
     startup: function ccm_startup(aCompleteListener) {
         AddonManager.addAddonListener(gCalendarManagerAddonListener);
         this.checkAndMigrateDB();
@@ -162,8 +70,6 @@ calCalendarManager.prototype = {
 
         this.cleanupOfflineObservers();
 
-        Services.obs.removeObserver(this, "profile-after-change");
-        Services.obs.removeObserver(this, "profile-before-change");
         Services.obs.removeObserver(this, "http-on-modify-request");
 
         AddonManager.removeAddonListener(gCalendarManagerAddonListener);
@@ -207,12 +113,6 @@ calCalendarManager.prototype = {
 
     observe: function ccm_observe(aSubject, aTopic, aData) {
         switch (aTopic) {
-            case "profile-after-change":
-                this.startup();
-                break;
-            case "profile-before-change":
-                this.shutdown();
-                break;
             case "timer-callback":
                 // Refresh all the calendars that can be refreshed.
                 var cals = this.getCalendars({});
@@ -243,7 +143,9 @@ calCalendarManager.prototype = {
                             // The provider may choose to explicitly disable the
                             // rewriting, for example if all calendars on a
                             // domain have the same credentials
-                            authHeader = authHeader.replace(/realm="(.*)"/, 'realm="$1 (' + calendar.name + ')"');
+                            let escapedName = calendar.name.replace('\\', '\\\\', 'g')
+                                                           .replace('"','\\"', 'g');
+                            authHeader = appendToRealm(authHeader, "(" + escapedName + ")");
                             channel.setResponseHeader("WWW-Authenticate", authHeader, false);
                         }
                     }
@@ -432,7 +334,7 @@ calCalendarManager.prototype = {
     checkAndMigrateDB: function calmgr_checkAndMigrateDB() {
         let storageSdb = Services.dirsvc.get("ProfD", Components.interfaces.nsILocalFile);
         storageSdb.append("storage.sdb");
-        db = Services.storage.openDatabase(storageSdb);
+        let db = Services.storage.openDatabase(storageSdb);
 
         db.beginTransactionAs(Components.interfaces.mozIStorageConnection.TRANSACTION_EXCLUSIVE);
         try {
@@ -530,8 +432,7 @@ calCalendarManager.prototype = {
             errorBoxButtonLabel = calGetString("calendar", "tooNewSchemaButtonRestart", [hostAppName]);
         }
 
-        var promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                                  .getService(Components.interfaces.nsIPromptService);
+        var promptSvc = Services.prompt;
 
         var errorBoxButtonFlags = (promptSvc.BUTTON_POS_0 *
                                    promptSvc.BUTTON_TITLE_IS_STRING +
@@ -547,24 +448,18 @@ calCalendarManager.prototype = {
                                          null, // No checkbox
                                          { value: false }); // Unnecessary checkbox state
 
-        var startup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
-                                .getService(Components.interfaces.nsIAppStartup);
         if (isSunbird()) {
-            startup.quit(Components.interfaces.nsIAppStartup.eForceQuit);
+            Services.startup.quit(Components.interfaces.nsIAppStartup.eForceQuit);
         } else {
             // Disable Lightning
             AddonManager.getAddonByID("{e2fda1a4-762b-4020-b5ad-a41df1933103}", function getLightningExt(aAddon) {
                 aAddon.userDisabled = true;
-                startup.quit(Components.interfaces.nsIAppStartup.eRestart |
+                Services.startup.quit(Components.interfaces.nsIAppStartup.eRestart |
                     Components.interfaces.nsIAppStartup.eForceQuit);
             });
         }
     },
 
-    notifyObservers: function(functionName, args) {
-        this.mObservers.notify(functionName, args);
-    },
-
     /**
      * calICalendarManager interface
      */
@@ -590,6 +485,7 @@ calCalendarManager.prototype = {
                 case Components.interfaces.calIErrors.STORAGE_UNKNOWN_SCHEMA_ERROR:
                     // For now we alert and quit on schema errors like we've done before:
                     this.alertAndQuit();
+                    return;
                 case Components.interfaces.calIErrors.STORAGE_UNKNOWN_TIMEZONES_ERROR:
                     uiMessage = calGetString("calendar", "unknownTimezonesError", [uri.spec]);
                     break;
@@ -607,13 +503,11 @@ calCalendarManager.prototype = {
             paramBlock.SetString(0, uiMessage);
             paramBlock.SetString(1, "0x" + rc.toString(0x10));
             paramBlock.SetString(2, ex);
-            let wWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
-                                     .getService(Components.interfaces.nsIWindowWatcher);
-            wWatcher.openWindow(null,
-                                "chrome://calendar/content/calendar-error-prompt.xul",
-                                "_blank",
-                                "chrome,dialog=yes,alwaysRaised=yes",
-                                paramBlock);
+            Services.ww.openWindow(null,
+                                   "chrome://calendar/content/calendar-error-prompt.xul",
+                                   "_blank",
+                                   "chrome,dialog=yes,alwaysRaised=yes",
+                                   paramBlock);
             return null;
         }
     },
@@ -634,7 +528,8 @@ calCalendarManager.prototype = {
         cal.setPref(getPrefBranchFor(calendar.id) + "uri", calendar.uri.spec);
 
         if ((calendar.getProperty("cache.supported") !== false) &&
-            calendar.getProperty("cache.enabled")) {
+            (calendar.getProperty("cache.enabled") ||
+             calendar.getProperty("cache.always"))) {
             calendar = new calCachedCalendar(calendar);
         }
 
@@ -742,10 +637,11 @@ calCalendarManager.prototype = {
         // providers (storage and memory). Otherwise we may nuke someone's
         // calendar stored on a server when all they really wanted to do was
         // unsubscribe.
-        if (cal.calInstanceOf(calendar, Components.interfaces.calICalendarProvider) &&
-            (calendar.type == "storage" || calendar.type == "memory")) {
+        let wrappedCalendar = cal.wrapInstance(calendar, Components.interfaces.calICalendarProvider);
+        if (wrappedCalendar &&
+            (wrappedCalendar.type == "storage" || wrappedCalendar.type == "memory")) {
             try {
-                calendar.deleteCalendar(calendar, null);
+                wrappedCalendar.deleteCalendar(calendar, null);
             } catch (e) {
                 Components.utils.reportError("error purging calendar: " + e);
             }
@@ -775,10 +671,8 @@ calCalendarManager.prototype = {
             this.mCache = {};
             this.mCalObservers = {};
 
-            let prefService = Components.classes["@mozilla.org/preferences-service;1"]
-                                        .getService(Components.interfaces.nsIPrefBranch);
             let allCals = {};
-            for each (let key in prefService.getChildList(REGISTRY_BRANCH)) { // merge down all keys
+            for each (let key in Services.prefs.getChildList(REGISTRY_BRANCH)) { // merge down all keys
                 allCals[key.substring(0, key.indexOf(".", REGISTRY_BRANCH.length))] = true;
             }
 
@@ -789,7 +683,7 @@ calCalendarManager.prototype = {
 
                 try {
                     if (!ctype || !curi) { // sanity check
-                        prefService.deleteBranch(calBranch + ".");
+                        Services.prefs.deleteBranch(calBranch + ".");
                         continue;
                     }
 
@@ -803,7 +697,8 @@ calCalendarManager.prototype = {
                         }
 
                         if ((calendar.getProperty("cache.supported") !== false) &&
-                            calendar.getProperty("cache.enabled")) {
+                            (calendar.getProperty("cache.enabled") ||
+                             calendar.getProperty("cache.always"))) {
                             calendar = new calCachedCalendar(calendar);
                         }
                     } else { // create dummy calendar that stays disabled for this run:
@@ -852,11 +747,8 @@ calCalendarManager.prototype = {
         cal.ASSERT(name && name.length > 0, "Pref Name must be non-empty!");
 
         let branch = (getPrefBranchFor(calendar.id) + name);
-
-        let prefService = Components.classes["@mozilla.org/preferences-service;1"]
-                                    .getService(Components.interfaces.nsIPrefBranch);
-        // delete before to allow pref-type changes:
-        prefService.deleteBranch(branch);
+        // Delete before to allow pref-type changes:
+        Services.prefs.deleteBranch(branch);
 
         if ( name === "name" ) {
             cal.setLocalizedPref(branch, value);
@@ -869,20 +761,18 @@ calCalendarManager.prototype = {
         cal.ASSERT(calendar, "Invalid Calendar!");
         cal.ASSERT(calendar.id !== null, "Calendar id needs to be set!");
         cal.ASSERT(name && name.length > 0, "Pref Name must be non-empty!");
-
-        let prefService = Components.classes["@mozilla.org/preferences-service;1"]
-                                    .getService(Components.interfaces.nsIPrefBranch);
-        prefService.deleteBranch(getPrefBranchFor(calendar.id) + name);
+        Services.prefs.deleteBranch(getPrefBranchFor(calendar.id) + name);
     },
 
     mObservers: null,
-    addObserver: function(aObserver) {
-        this.mObservers.add(aObserver);
-    },
+    addObserver: function(aObserver) this.mObservers.add(aObserver),
+    removeObserver: function(aObserver) this.mObservers.remove(aObserver),
+    notifyObservers: function(functionName, args) this.mObservers.notify(functionName, args),
 
-    removeObserver: function(aObserver) {
-        this.mObservers.remove(aObserver);
-    }
+    mCalendarObservers: null,
+    addCalendarObserver: function(aObserver) this.mCalendarObservers.add(aObserver),
+    removeCalendarObserver: function(aObserver) this.mCalendarObservers.remove(aObserver),
+    notifyCalendarObservers: function(functionName, args) this.mCalendarObservers.notify(functionName, args)
 };
 
 function equalMessage(msg1, msg2) {
@@ -907,24 +797,25 @@ calMgrCalendarObserver.prototype = {
     storedReadOnly: null,
     calMgr: null,
 
-    QueryInterface: function mBL_QueryInterface(aIID) {
-        return doQueryInterface(this, calMgrCalendarObserver.prototype, aIID,
-                                [Components.interfaces.nsIWindowMediatorListener,
-                                 Components.interfaces.calIObserver]);
-    },
+    QueryInterface: XPCOMUtils.generateQI([
+        Components.interfaces.nsIWindowMediatorListener,
+        Components.interfaces.calIObserver
+    ]),
 
     // calIObserver:
-    onStartBatch: function() {},
-    onEndBatch: function() {},
-    onLoad: function(calendar) {},
-    onAddItem: function(aItem) {},
-    onModifyItem: function(aNewItem, aOldItem) {},
-    onDeleteItem: function(aDeletedItem) {},
+    onStartBatch: function() this.calMgr.notifyCalendarObservers("onStartBatch", arguments),
+    onEndBatch: function() this.calMgr.notifyCalendarObservers("onEndBatch", arguments),
+    onLoad: function(calendar) this.calMgr.notifyCalendarObservers("onLoad", arguments),
+    onAddItem: function(aItem) this.calMgr.notifyCalendarObservers("onAddItem", arguments),
+    onModifyItem: function(aNewItem, aOldItem) this.calMgr.notifyCalendarObservers("onModifyItem", arguments),
+    onDeleteItem: function(aDeletedItem) this.calMgr.notifyCalendarObservers("onDeleteItem", arguments),
     onError: function(aCalendar, aErrNo, aMessage) {
+        this.calMgr.notifyCalendarObservers("onError", arguments);
         this.announceError(aCalendar, aErrNo, aMessage);
     },
 
     onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {
+        this.calMgr.notifyCalendarObservers("onPropertyChanged", arguments);
         switch (aName) {
             case "requiresNetwork":
                 this.calMgr.mNetworkCalendarCount += (aValue ? 1 : -1);
@@ -936,59 +827,7 @@ calMgrCalendarObserver.prototype = {
                 this.calMgr.setupRefreshTimer(aCalendar);
                 break;
             case "cache.enabled":
-                aOldValue = aOldValue || false;
-                aValue = aValue || false;
-
-                if (aOldValue != aValue) {
-
-                    // Try to find the current sort order
-                    let sortOrderPref = cal.getPrefSafe("calendar.list.sortOrder", "").split(" ");
-                    let initialSortOrderPos = null;
-                    for (let i = 0; i < sortOrderPref.length; ++i) {
-                        if (sortOrderPref[i] == aCalendar.id) {
-                            initialSortOrderPos = i;
-                        }
-                    }
-                    // Enabling or disabling cache on a calendar re-creates
-                    // it so the registerCalendar call can wrap/unwrap the
-                    // calCachedCalendar facade saving the user the need to
-                    // restart Thunderbird and making sure a new Id is used.
-                    this.calMgr.unregisterCalendar(aCalendar);
-                    this.calMgr.deleteCalendar(aCalendar);
-                    var newCal = this.calMgr.createCalendar(aCalendar.type,aCalendar.uri);
-                    newCal.name = aCalendar.name;
-
-                    // TODO: if properties get added this list will need to be adjusted,
-                    // ideally we should add a "getProperties" method to calICalendar.idl
-                    // to retrieve all non-transient properties for a calendar.
-                    let propsToCopy = [ "color",
-                                        "disabled",
-                                        "auto-enabled",
-                                        "cache.enabled",
-                                        "refreshInterval",
-                                        "suppressAlarms",
-                                        "calendar-main-in-composite",
-                                        "calendar-main-default",
-                                        "readOnly",
-                                        "imip.identity.key"];
-                    for each (let prop in propsToCopy ) {
-                      newCal.setProperty(prop,
-                                         aCalendar.getProperty(prop));
-                    }
-
-                    if (initialSortOrderPos != null) {
-                        newCal.setProperty("initialSortOrderPos",
-                                           initialSortOrderPos);
-                    }
-                    this.calMgr.registerCalendar(newCal);
-                }
-                else {
-                    if (aCalendar.wrappedJSObject instanceof calCachedCalendar) {
-                        // any attempt to switch this flag will reset the cached calendar;
-                        // could be useful for users in case the cache may be corrupted.
-                        aCalendar.wrappedJSObject.setupCachedCalendar();
-                    }
-                }
+                this.changeCalendarCache.apply(this, arguments);
                 break;
             case "disabled":
                 if (!aValue && aCalendar.canRefresh) {
@@ -998,19 +837,70 @@ calMgrCalendarObserver.prototype = {
         }
     },
 
+    changeCalendarCache: function(aCalendar, aName, aValue, aOldValue) {
+        aOldValue = aOldValue || false;
+        aValue = aValue || false;
+
+        if (aOldValue != aValue) {
+            // Try to find the current sort order
+            let sortOrderPref = cal.getPrefSafe("calendar.list.sortOrder", "").split(" ");
+            let initialSortOrderPos = null;
+            for (let i = 0; i < sortOrderPref.length; ++i) {
+                if (sortOrderPref[i] == aCalendar.id) {
+                    initialSortOrderPos = i;
+                }
+            }
+            // Enabling or disabling cache on a calendar re-creates
+            // it so the registerCalendar call can wrap/unwrap the
+            // calCachedCalendar facade saving the user the need to
+            // restart Thunderbird and making sure a new Id is used.
+            this.calMgr.unregisterCalendar(aCalendar);
+            this.calMgr.deleteCalendar(aCalendar);
+            var newCal = this.calMgr.createCalendar(aCalendar.type,aCalendar.uri);
+            newCal.name = aCalendar.name;
+
+            // TODO: if properties get added this list will need to be adjusted,
+            // ideally we should add a "getProperties" method to calICalendar.idl
+            // to retrieve all non-transient properties for a calendar.
+            let propsToCopy = [ "color",
+                                "disabled",
+                                "auto-enabled",
+                                "cache.enabled",
+                                "refreshInterval",
+                                "suppressAlarms",
+                                "calendar-main-in-composite",
+                                "calendar-main-default",
+                                "readOnly",
+                                "imip.identity.key"];
+            for each (let prop in propsToCopy ) {
+              newCal.setProperty(prop,
+                                 aCalendar.getProperty(prop));
+            }
+
+            if (initialSortOrderPos != null) {
+                newCal.setProperty("initialSortOrderPos",
+                                   initialSortOrderPos);
+            }
+            this.calMgr.registerCalendar(newCal);
+        } else {
+            if (aCalendar.wrappedJSObject instanceof calCachedCalendar) {
+                // any attempt to switch this flag will reset the cached calendar;
+                // could be useful for users in case the cache may be corrupted.
+                aCalendar.wrappedJSObject.setupCachedCalendar();
+            }
+        }
+    },
+
     onPropertyDeleting: function(aCalendar, aName) {
         this.onPropertyChanged(aCalendar, aName, false, true);
     },
 
     // Error announcer specific functions
-
     announceError: function(aCalendar, aErrNo, aMessage) {
 
         var paramBlock = Components.classes["@mozilla.org/embedcomp/dialogparam;1"]
                                    .createInstance(Components.interfaces.nsIDialogParamBlock);
-        var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
-                            .getService(Components.interfaces.nsIStringBundleService);
-        var props = sbs.createBundle("chrome://calendar/locale/calendar.properties");
+        var props = Services.strings.createBundle("chrome://calendar/locale/calendar.properties");
         var errMsg;
         paramBlock.SetNumberStrings(3);
         if (!this.storedReadOnly && this.calendar.readOnly) {
@@ -1084,10 +974,8 @@ calMgrCalendarObserver.prototype = {
             this.announcedMessages.push(paramBlock);
 
             // Display in prompt window.
-            var wWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
-                                  .getService(Components.interfaces.nsIWindowWatcher);
             var promptWindow =
-                wWatcher.openWindow
+                Services.ww.openWindow
                     (null, "chrome://calendar/content/calendar-error-prompt.xul",
                      "_blank", "chrome,dialog=yes,alwaysRaised=yes",
                      paramBlock);
@@ -1141,3 +1029,97 @@ calDummyCalendar.prototype = {
         }
     }
 };
+
+function getPrefBranchFor(id) {
+    return (REGISTRY_BRANCH + id + ".");
+}
+
+/**
+ * Helper function to flush the preferences file. If the application crashes
+ * after a calendar has been created using the prefs registry, then the calendar
+ * won't show up. Writing the prefs helps counteract.
+ */
+function flushPrefs() {
+    Services.prefs.savePrefFile(null);
+}
+
+/**
+ * Callback object for the refresh timer. Should be called as an object, i.e
+ * let foo = new timerCallback(calendar);
+ *
+ * @param aCalendar     The calendar to refresh on notification
+ */
+function timerCallback(aCalendar) {
+    this.notify = function refreshNotify(aTimer) {
+        if (!aCalendar.getProperty("disabled") && aCalendar.canRefresh) {
+            aCalendar.refresh();
+        }
+    }
+}
+
+var gCalendarManagerAddonListener = {
+    onDisabling: function(aAddon, aNeedsRestart) {
+        if (!this.queryUninstallProvider(aAddon)) {
+            // If the addon should not be disabled, then re-enable it.
+            aAddon.userDisabled = false;
+        }
+    },
+
+    onUninstalling: function(aAddon, aNeedsRestart) {
+        if (!this.queryUninstallProvider(aAddon)) {
+            // If the addon should not be uninstalled, then cancel the uninstall.
+            aAddon.cancelUninstall();
+        }
+    },
+
+    queryUninstallProvider: function(aAddon) {
+        const uri = "chrome://calendar/content/calendar-providerUninstall-dialog.xul";
+        const features = "chrome,titlebar,resizable,modal";
+        let calMgr = cal.getCalendarManager();
+        let affectedCalendars =
+            [ calendar for each (calendar in calMgr.getCalendars({}))
+              if (calendar.providerID == aAddon.id) ];
+        if (!affectedCalendars.length) {
+            // If no calendars are affected, then everything is fine.
+            return true;
+        }
+
+        let args = { shouldUninstall: false, extension: aAddon };
+
+        // Now find a window. The best choice would be the most recent
+        // addons window, otherwise the most recent calendar window, or we
+        // create a new toplevel window.
+        let win = Services.wm.getMostRecentWindow("Extension:Manager") ||
+                  cal.getCalendarWindow();
+        if (win) {
+            win.openDialog(uri, "CalendarProviderUninstallDialog", features, args);
+        } else {
+            // Use the window watcher to open a parentless window.
+            Services.ww.openWindow(null, uri, "CalendarProviderUninstallWindow", features, args);
+        }
+
+        // Now that we are done, check if the dialog was accepted or canceled.
+        return args.shouldUninstall;
+    }
+};
+
+function appendToRealm(authHeader, appendStr) {
+    let isEscaped = false;
+    let idx = authHeader.search(/realm="(.*?)(\\*)"/);
+    if (idx > -1) {
+        let remain = authHeader.substr(idx + 7); idx += 7;
+        while (remain.length && !isEscaped) {
+            let m = remain.match(/(.*?)(\\*)"/);
+            idx += m[0].length;
+
+            isEscaped = ((m[2].length % 2) == 0);
+            if (!isEscaped) {
+                remain = remain.substr(m[0].length);
+            }
+        }
+        return authHeader.substr(0, idx - 1) + " " +
+                appendStr + authHeader.substr(idx - 1);
+    } else {
+        return authHeader;
+    }
+}
