@@ -17,25 +17,25 @@ DEFAULTS = {
 
   # LDAPSDKS
   'LDAPSDKS_REPO': 'http://hg.mozilla.org/projects/ldap-sdks/',
-  'LDAPSDKS_REV': 'LDAPCSDK_6_0_7D_RTM',
+  'LDAPSDKS_REV': 'LDAPCSDK_6_0_7F_RTM',
 
   # URL of the default hg repository to clone for ChatZilla.
   'CHATZILLA_REPO': 'http://hg.mozilla.org/chatzilla/',
-  'CHATZILLA_REV':  'CHATZILLA_0_9_88_2_RELEASE',
-
+  # The stable revision to use
+  'CHATZILLA_REV':  'SEA2_21_RELBRANCH',
 
   # URL of the default hg repository to clone for DOM Inspector.
   'INSPECTOR_REPO': 'http://hg.mozilla.org/dom-inspector/',
-  'INSPECTOR_REV':  'DOMI_2_0_11_RELEASE',
-
+  # The stable revision to use
+  'INSPECTOR_REV':  'SEA2_21_RELBRANCH',
 
   # URL of the default hg repository to clone for Venkman.
   'VENKMAN_REPO': 'http://hg.mozilla.org/venkman/',
-  'VENKMAN_REV':  'VENKMAN_RELEASE_0_9_89',
-
+  # The stable revision to use
+  'VENKMAN_REV':  'SEA2_21_RELBRANCH',
 
   # URL of the default hg repository to clone for Mozilla.
-  'MOZILLA_REPO': 'http://hg.mozilla.org/releases/mozilla-release/',
+  'MOZILLA_REPO': 'http://hg.mozilla.org/releases/mozilla-esr24/',
 }
 
 REPO_SHORT_NAMES = {
