@@ -41,7 +41,6 @@ function TabItem(tab, options) {
 
   this.tabCanvas = new TabCanvas(this.tab, this.$canvas[0]);
 
-  this.defaultSize = new Point(TabItems.tabWidth, TabItems.tabHeight);
   this._hidden = false;
   this.isATabItem = true;
   this.keepProportional = true;
@@ -278,7 +277,7 @@ TabItem.prototype = Utils.extend(new Item(), new Subscribable(), {
 
         // if it matches the selected tab or no active tab and the browser
         // tab is hidden, the active group item would be set.
-        if (this.tab == gBrowser.selectedTab ||
+        if (this.tab.selected ||
             (!GroupItems.getActiveGroupItem() && !this.tab.hidden))
           UI.setActive(this.parent);
       }
@@ -535,16 +534,16 @@ TabItem.prototype = Utils.extend(new Item(), new Subscribable(), {
     let tab = this.tab;
 
     function onZoomDone() {
-      $canvas.css({ '-moz-transform': null });
+      $canvas.css({ 'transform': null });
       $tabEl.removeClass("front");
 
       UI.goToTab(tab);
 
-      // tab might not be selected because hideTabView() is invoked after 
+      // tab might not be selected because hideTabView() is invoked after
       // UI.goToTab() so we need to setup everything for the gBrowser.selectedTab
-      if (tab != gBrowser.selectedTab) {
+      if (!tab.selected) {
         UI.onTabSelect(gBrowser.selectedTab);
-      } else { 
+      } else {
         if (isNewBlankTab)
           gWindow.gURLBar.focus();
       }
@@ -563,8 +562,8 @@ TabItem.prototype = Utils.extend(new Item(), new Subscribable(), {
         $tabEl.removeClass("stack-trayed");
       $tabEl.addClass("front");
       $canvas
-        .css({ '-moz-transform-origin': transform.transformOrigin })
-        .animate({ '-moz-transform': transform.transform }, {
+        .css({ 'transform-origin': transform.transformOrigin })
+        .animate({ 'transform': transform.transform }, {
           duration: 230,
           easing: 'fast',
           complete: function() {
@@ -593,7 +592,7 @@ TabItem.prototype = Utils.extend(new Item(), new Subscribable(), {
     
     let onZoomDone = function onZoomDone() {
       $tab.removeClass("front");
-      $canvas.css("-moz-transform", null);
+      $canvas.css("transform", null);
 
       if (typeof complete == "function")
         complete();
@@ -612,11 +611,11 @@ TabItem.prototype = Utils.extend(new Item(), new Subscribable(), {
       TabItems.pausePainting();
 
       $canvas.css({
-        '-moz-transform': transform.transform,
-        '-moz-transform-origin': transform.transformOrigin
+        'transform': transform.transform,
+        'transform-origin': transform.transformOrigin
       });
 
-      $canvas.animate({ "-moz-transform": "scale(1.0)" }, {
+      $canvas.animate({ "transform": "scale(1.0)" }, {
         duration: 300,
         easing: 'cubic-bezier', // note that this is legal easing, even without parameters
         complete: function() {
@@ -716,7 +715,6 @@ let TabItems = {
   _lastUpdateTime: Date.now(),
   _eventListeners: [],
   _pauseUpdateForTest: false,
-  tempCanvas: null,
   _reconnectingPaused: false,
   tabItemPadding: {},
   _mozAfterPaintHandler: null,
@@ -745,11 +743,6 @@ let TabItems = {
       .attr('moz-opaque', '');
     $canvas.appendTo(iQ("body"));
     $canvas.hide();
-    this.tempCanvas = $canvas[0];
-    // 150 pixels is an empirical size, below which FF's drawWindow()
-    // algorithm breaks down
-    this.tempCanvas.width = 150;
-    this.tempCanvas.height = 112;
 
     let mm = gWindow.messageManager;
     this._mozAfterPaintHandler = this.onMozAfterPaint.bind(this);
@@ -1387,94 +1380,12 @@ TabCanvas.prototype = Utils.extend(new Subscribable(), {
       return;
     }
 
-    let ctx = this.canvas.getContext("2d");
-    let tempCanvas = TabItems.tempCanvas;
-    let bgColor = '#fff';
-
-    if (w < tempCanvas.width) {
-      // Small draw case where nearest-neighbor algorithm breaks down in Windows
-      // First draw to a larger canvas (150px wide), and then draw that image
-      // to the destination canvas.
-      let tempCtx = tempCanvas.getContext("2d");
-      this._drawWindow(tempCtx, tempCanvas.width, tempCanvas.height, bgColor);
-
-      // Now copy to tabitem canvas.
-      try {
-        this._fillCanvasBackground(ctx, w, h, bgColor);
-        ctx.drawImage(tempCanvas, 0, 0, w, h);
-      } catch (e) {
-        Utils.error('paint', e);
-      }
-    } else {
-      // General case where nearest neighbor algorithm looks good
-      // Draw directly to the destination canvas
-      this._drawWindow(ctx, w, h, bgColor);
-    }
+    let win = this.tab.linkedBrowser.contentWindow;
+    gPageThumbnails.captureToCanvas(win, this.canvas);
 
     this._sendToSubscribers("painted");
   },
 
-  // ----------
-  // Function: _fillCanvasBackground
-  // Draws a rectangle of <width>x<height> with color <bgColor> to the given
-  // canvas context.
-  _fillCanvasBackground: function TabCanvas__fillCanvasBackground(ctx, width, height, bgColor) {
-    ctx.fillStyle = bgColor;
-    ctx.fillRect(0, 0, width, height);
-  },
-
-  // ----------
-  // Function: _drawWindow
-  // Draws contents of the tabs' browser window to the given canvas context.
-  _drawWindow: function TabCanvas__drawWindow(ctx, width, height, bgColor) {
-    this._fillCanvasBackground(ctx, width, height, bgColor);
-
-    let rect = this._calculateClippingRect(width, height);
-    let scaler = width / rect.width;
-
-    ctx.save();
-    ctx.scale(scaler, scaler);
-
-    try {
-      let win = this.tab.linkedBrowser.contentWindow;
-      ctx.drawWindow(win, rect.left, rect.top, rect.width, rect.height,
-                     bgColor, ctx.DRAWWINDOW_DO_NOT_FLUSH);
-    } catch (e) {
-      Utils.error('paint', e);
-    }
-
-    ctx.restore();
-  },
-
-  // ----------
-  // Function: _calculateClippingRect
-  // Calculate the clipping rect that will be projected to the tab's
-  // thumbnail canvas.
-  _calculateClippingRect: function TabCanvas__calculateClippingRect(origWidth, origHeight) {
-    let win = this.tab.linkedBrowser.contentWindow;
-
-    // TODO BUG 631593: retrieve actual scrollbar width
-    // 25px is supposed to be width of the vertical scrollbar
-    let maxWidth = Math.max(1, win.innerWidth - 25);
-    let maxHeight = win.innerHeight;
-
-    let height = Math.min(maxHeight, Math.floor(origHeight * maxWidth / origWidth));
-    let width = Math.floor(origWidth * height / origHeight);
-
-    // very short pages in combination with a very wide browser window force us
-    // to extend the clipping rect and add some empty space around the thumb
-    let factor = 0.7;
-    if (width < maxWidth * factor) {
-      width = maxWidth * factor;
-      height = Math.floor(origHeight * width / origWidth);
-    }
-
-    let left = win.scrollX + Math.max(0, Math.round((maxWidth - width) / 2));
-    let top = win.scrollY;
-
-    return new Rect(left, top, width, height);
-  },
-
   // ----------
   // Function: toImageData
   toImageData: function TabCanvas_toImageData() {
