@@ -22,16 +22,19 @@
 #include "nsIScriptRuntime.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIPrincipal.h"
+#include "nsJSPrincipals.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIHttpChannel.h"
+#include "nsIHttpChannelInternal.h"
 #include "nsIScriptElement.h"
 #include "nsIDOMHTMLScriptElement.h"
 #include "nsIDocShell.h"
 #include "nsContentUtils.h"
+#include "nsCxPusher.h"
 #include "nsUnicharUtils.h"
 #include "nsAutoPtr.h"
 #include "nsIXPConnect.h"
-#include "nsContentErrors.h"
+#include "nsError.h"
 #include "nsThreadUtils.h"
 #include "nsDocShellCID.h"
 #include "nsIContentSecurityPolicy.h"
@@ -40,10 +43,10 @@
 #include "nsChannelPolicy.h"
 #include "nsCRT.h"
 #include "nsContentCreatorFunctions.h"
-#include "nsGenericElement.h"
+#include "mozilla/dom/Element.h"
 #include "nsCrossSiteListenerProxy.h"
+#include "nsSandboxFlags.h"
 
-#include "mozilla/FunctionTimer.h"
 #include "mozilla/CORSMode.h"
 #include "mozilla/Attributes.h"
 
@@ -61,7 +64,7 @@ using namespace mozilla::dom;
 class nsScriptLoadRequest MOZ_FINAL : public nsISupports {
 public:
   nsScriptLoadRequest(nsIScriptElement* aElement,
-                      PRUint32 aVersion,
+                      uint32_t aVersion,
                       CORSMode aCORSMode)
     : mElement(aElement),
       mLoading(true),
@@ -85,17 +88,17 @@ public:
 
   bool IsPreload()
   {
-    return mElement == nsnull;
+    return mElement == nullptr;
   }
 
   nsCOMPtr<nsIScriptElement> mElement;
   bool mLoading;             // Are we still waiting for a load to complete?
   bool mIsInline;            // Is the script inline or loaded?
   nsString mScriptText;              // Holds script for loaded scripts
-  PRUint32 mJSVersion;
+  uint32_t mJSVersion;
   nsCOMPtr<nsIURI> mURI;
   nsCOMPtr<nsIPrincipal> mOriginPrincipal;
-  PRInt32 mLineNo;
+  int32_t mLineNo;
   const CORSMode mCORSMode;
 };
 
@@ -130,25 +133,25 @@ nsScriptLoader::~nsScriptLoader()
     mParserBlockingRequest->FireScriptAvailable(NS_ERROR_ABORT);
   }
 
-  for (PRUint32 i = 0; i < mXSLTRequests.Length(); i++) {
+  for (uint32_t i = 0; i < mXSLTRequests.Length(); i++) {
     mXSLTRequests[i]->FireScriptAvailable(NS_ERROR_ABORT);
   }
 
-  for (PRUint32 i = 0; i < mDeferRequests.Length(); i++) {
+  for (uint32_t i = 0; i < mDeferRequests.Length(); i++) {
     mDeferRequests[i]->FireScriptAvailable(NS_ERROR_ABORT);
   }
 
-  for (PRUint32 i = 0; i < mAsyncRequests.Length(); i++) {
+  for (uint32_t i = 0; i < mAsyncRequests.Length(); i++) {
     mAsyncRequests[i]->FireScriptAvailable(NS_ERROR_ABORT);
   }
 
-  for (PRUint32 i = 0; i < mNonAsyncExternalScriptInsertedRequests.Length(); i++) {
+  for (uint32_t i = 0; i < mNonAsyncExternalScriptInsertedRequests.Length(); i++) {
     mNonAsyncExternalScriptInsertedRequests[i]->FireScriptAvailable(NS_ERROR_ABORT);
   }
 
   // Unblock the kids, in case any of them moved to a different document
   // subtree in the meantime and therefore aren't actually going away.
-  for (PRUint32 j = 0; j < mPendingChildLoaders.Length(); ++j) {
+  for (uint32_t j = 0; j < mPendingChildLoaders.Length(); ++j) {
     mPendingChildLoaders[j]->RemoveExecuteBlocker();
   }  
 }
@@ -214,13 +217,13 @@ nsScriptLoader::CheckContentPolicy(nsIDocument* aDocument,
                                    nsIURI *aURI,
                                    const nsAString &aType)
 {
-  PRInt16 shouldLoad = nsIContentPolicy::ACCEPT;
+  int16_t shouldLoad = nsIContentPolicy::ACCEPT;
   nsresult rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_SCRIPT,
                                           aURI,
                                           aDocument->NodePrincipal(),
                                           aContext,
                                           NS_LossyConvertUTF16toASCII(aType),
-                                          nsnull,    //extra
+                                          nullptr,    //extra
                                           &shouldLoad,
                                           nsContentUtils::GetContentPolicy(),
                                           nsContentUtils::GetSecurityManager());
@@ -257,7 +260,8 @@ nsScriptLoader::ShouldLoadScript(nsIDocument* aDocument,
 }
 
 nsresult
-nsScriptLoader::StartLoad(nsScriptLoadRequest *aRequest, const nsAString &aType)
+nsScriptLoader::StartLoad(nsScriptLoadRequest *aRequest, const nsAString &aType,
+                          bool aScriptFromHead)
 {
   nsISupports *context = aRequest->mElement.get()
                          ? static_cast<nsISupports *>(aRequest->mElement.get())
@@ -269,7 +273,7 @@ nsScriptLoader::StartLoad(nsScriptLoadRequest *aRequest, const nsAString &aType)
 
   nsCOMPtr<nsILoadGroup> loadGroup = mDocument->GetDocumentLoadGroup();
 
-  nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(mDocument->GetScriptGlobalObject()));
+  nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(mDocument->GetWindow()));
   if (!window) {
     return NS_ERROR_NULL_POINTER;
   }
@@ -278,6 +282,11 @@ nsScriptLoader::StartLoad(nsScriptLoadRequest *aRequest, const nsAString &aType)
 
   nsCOMPtr<nsIInterfaceRequestor> prompter(do_QueryInterface(docshell));
 
+  // If this document is sandboxed without 'allow-scripts', abort.
+  if (mDocument->GetSandboxFlags() & SANDBOXED_SCRIPTS) {
+    return NS_OK;
+  }
+
   // check for a Content Security Policy to pass down to the channel
   // that will be created to load the script
   nsCOMPtr<nsIChannelPolicy> channelPolicy;
@@ -292,11 +301,20 @@ nsScriptLoader::StartLoad(nsScriptLoadRequest *aRequest, const nsAString &aType)
 
   nsCOMPtr<nsIChannel> channel;
   rv = NS_NewChannel(getter_AddRefs(channel),
-                     aRequest->mURI, nsnull, loadGroup, prompter,
+                     aRequest->mURI, nullptr, loadGroup, prompter,
                      nsIRequest::LOAD_NORMAL | nsIChannel::LOAD_CLASSIFY_URI,
                      channelPolicy);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsIScriptElement *script = aRequest->mElement;
+  if (aScriptFromHead &&
+      !(script && (script->GetScriptAsync() || script->GetScriptDeferred()))) {
+    nsCOMPtr<nsIHttpChannelInternal>
+      internalHttpChannel(do_QueryInterface(channel));
+    if (internalHttpChannel)
+      internalHttpChannel->SetLoadAsBlocking(true);
+  }
+
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
   if (httpChannel) {
     // HTTP content negotation has little value in this context.
@@ -314,10 +332,12 @@ nsScriptLoader::StartLoad(nsScriptLoadRequest *aRequest, const nsAString &aType)
 
   if (aRequest->mCORSMode != CORS_NONE) {
     bool withCredentials = (aRequest->mCORSMode == CORS_USE_CREDENTIALS);
-    listener =
-      new nsCORSListenerProxy(listener, mDocument->NodePrincipal(), channel,
-                              withCredentials, &rv);
+    nsRefPtr<nsCORSListenerProxy> corsListener =
+      new nsCORSListenerProxy(listener, mDocument->NodePrincipal(),
+                              withCredentials);
+    rv = corsListener->Init(channel);
     NS_ENSURE_SUCCESS(rv, rv);
+    listener = corsListener;
   }
 
   rv = channel->AsyncOpen(listener, aRequest);
@@ -352,6 +372,36 @@ public:
   }
 };
 
+static inline bool
+ParseTypeAttribute(const nsAString& aType, JSVersion* aVersion)
+{
+  MOZ_ASSERT(!aType.IsEmpty());
+  MOZ_ASSERT(aVersion);
+  MOZ_ASSERT(*aVersion == JSVERSION_DEFAULT);
+
+  nsContentTypeParser parser(aType);
+
+  nsAutoString mimeType;
+  nsresult rv = parser.GetType(mimeType);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  if (!nsContentUtils::IsJavascriptMIMEType(mimeType)) {
+    return false;
+  }
+
+  // Get the version string, and ensure the language supports it.
+  nsAutoString versionName;
+  rv = parser.GetParameter("version", versionName);
+
+  if (NS_SUCCEEDED(rv)) {
+    *aVersion = nsContentUtils::ParseJavascriptVersion(versionName);
+  } else if (rv != NS_ERROR_INVALID_ARG) {
+    return false;
+  }
+
+  return true;
+}
+
 bool
 nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
 {
@@ -380,7 +430,8 @@ nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
   // For now though, if JS is disabled we assume every language is
   // disabled.
   // XXX is this different from the mDocument->IsScriptEnabled() call?
-  nsIScriptGlobalObject *globalObject = mDocument->GetScriptGlobalObject();
+  nsCOMPtr<nsIScriptGlobalObject> globalObject =
+    do_QueryInterface(mDocument->GetWindow());
   if (!globalObject) {
     return false;
   }
@@ -393,131 +444,40 @@ nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
     return false;
   }
 
-  PRUint32 typeID = nsIProgrammingLanguage::JAVASCRIPT;
-  PRUint32 version = 0;
-  nsAutoString language, type, src;
-  nsresult rv = NS_OK;
+  JSVersion version = JSVERSION_DEFAULT;
 
   // Check the type attribute to determine language and version.
   // If type exists, it trumps the deprecated 'language='
+  nsAutoString type;
   aElement->GetScriptType(type);
   if (!type.IsEmpty()) {
-    nsContentTypeParser parser(type);
-
-    nsAutoString mimeType;
-    rv = parser.GetType(mimeType);
-    NS_ENSURE_SUCCESS(rv, false);
-
-    // Javascript keeps the fast path, optimized for most-likely type
-    // Table ordered from most to least likely JS MIME types.
-    // See bug 62485, feel free to add <script type="..."> survey data to it,
-    // or to a new bug once 62485 is closed.
-    static const char *jsTypes[] = {
-      "text/javascript",
-      "text/ecmascript",
-      "application/javascript",
-      "application/ecmascript",
-      "application/x-javascript",
-      nsnull
-    };
-
-    bool isJavaScript = false;
-    for (PRInt32 i = 0; jsTypes[i]; i++) {
-      if (mimeType.LowerCaseEqualsASCII(jsTypes[i])) {
-        isJavaScript = true;
-        break;
-      }
-    }
-
-    if (!isJavaScript) {
-      typeID = nsIProgrammingLanguage::UNKNOWN;
-    }
-
-    if (typeID != nsIProgrammingLanguage::UNKNOWN) {
-      // Get the version string, and ensure the language supports it.
-      nsAutoString versionName;
-      rv = parser.GetParameter("version", versionName);
-      if (NS_FAILED(rv)) {
-        // no version attribute - version remains 0.
-        if (rv != NS_ERROR_INVALID_ARG)
-          return false;
-      } else {
-        nsCOMPtr<nsIScriptRuntime> runtime;
-        rv = NS_GetJSRuntime(getter_AddRefs(runtime));
-        if (NS_FAILED(rv)) {
-          NS_ERROR("Failed to locate the language with this ID");
-          return false;
-        }
-        rv = runtime->ParseVersion(versionName, &version);
-        if (NS_FAILED(rv)) {
-          NS_WARNING("This script language version is not supported - ignored");
-          typeID = nsIProgrammingLanguage::UNKNOWN;
-        }
-      }
-    }
-
-    // Some js specifics yet to be abstracted.
-    if (typeID == nsIProgrammingLanguage::JAVASCRIPT) {
-      nsAutoString value;
-      rv = parser.GetParameter("e4x", value);
-      if (NS_FAILED(rv)) {
-        if (rv != NS_ERROR_INVALID_ARG)
-          return false;
-      } else {
-        if (value.Length() == 1 && value[0] == '1')
-          // This happens in about 2 web pages. Enable E4X no matter what JS
-          // version number was selected.  We do this by turning on the "moar
-          // XML" version bit.  This is OK even if version has
-          // JSVERSION_UNKNOWN (-1).
-          version = js::VersionSetMoarXML(JSVersion(version), true);
-      }
-    }
+    NS_ENSURE_TRUE(ParseTypeAttribute(type, &version), false);
   } else {
     // no 'type=' element
     // "language" is a deprecated attribute of HTML, so we check it only for
     // HTML script elements.
     if (scriptContent->IsHTML()) {
+      nsAutoString language;
       scriptContent->GetAttr(kNameSpaceID_None, nsGkAtoms::language, language);
       if (!language.IsEmpty()) {
-        if (nsContentUtils::IsJavaScriptLanguage(language, &version))
-          typeID = nsIProgrammingLanguage::JAVASCRIPT;
-        else
-          typeID = nsIProgrammingLanguage::UNKNOWN;
-        // IE, Opera, etc. do not respect language version, so neither should
-        // we at this late date in the browser wars saga.  Note that this change
-        // affects HTML but not XUL or SVG (but note also that XUL has its own
-        // code to check nsContentUtils::IsJavaScriptLanguage -- that's probably
-        // a separate bug, one we may not be able to fix short of XUL2).  See
-        // bug 255895 (https://bugzilla.mozilla.org/show_bug.cgi?id=255895).
-        NS_ASSERTION(JSVERSION_DEFAULT == 0,
-                     "We rely on all languages having 0 as a version default");
-        version = 0;
+        if (!nsContentUtils::IsJavaScriptLanguage(language)) {
+          return false;
+        }
       }
     }
   }
 
-  // If we don't know the language, we don't know how to evaluate
-  if (typeID == nsIProgrammingLanguage::UNKNOWN) {
-    return false;
-  }
-  // If not from a chrome document (which is always trusted), we need some way 
-  // of checking the language is "safe".  Currently the only other language 
-  // impl is Python, and that is *not* safe in untrusted code - so fixing 
-  // this isn't a priority.!
-  // See also similar code in nsXULContentSink.cpp
-  if (typeID != nsIProgrammingLanguage::JAVASCRIPT &&
-      !nsContentUtils::IsChromeDoc(mDocument)) {
-    NS_WARNING("Untrusted language called from non-chrome - ignored");
-    return false;
-  }
-
   // Step 14. in the HTML5 spec
-
+  nsresult rv = NS_OK;
   nsRefPtr<nsScriptLoadRequest> request;
   if (aElement->GetScriptExternal()) {
     // external script
     nsCOMPtr<nsIURI> scriptURI = aElement->GetScriptURI();
     if (!scriptURI) {
+      // Asynchronously report the failure to create a URI object
+      NS_DispatchToCurrentThread(
+        NS_NewRunnableMethod(aElement,
+                             &nsIScriptElement::FireErrorEvent));
       return false;
     }
     CORSMode ourCORSMode = aElement->GetCORSMode();
@@ -541,7 +501,7 @@ nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
         NS_ENSURE_SUCCESS(rv, false);
       } else {
         // Drop the preload
-        request = nsnull;
+        request = nullptr;
       }
     }
 
@@ -551,8 +511,17 @@ nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
       request->mURI = scriptURI;
       request->mIsInline = false;
       request->mLoading = true;
-      rv = StartLoad(request, type);
-      NS_ENSURE_SUCCESS(rv, false);
+
+      // set aScriptFromHead to false so we don't treat non preloaded scripts as
+      // blockers for full page load. See bug 792438.
+      rv = StartLoad(request, type, false);
+      if (NS_FAILED(rv)) {
+        // Asynchronously report the load failure
+        NS_DispatchToCurrentThread(
+          NS_NewRunnableMethod(aElement,
+                               &nsIScriptElement::FireErrorEvent));
+        return false;
+      }
     }
 
     request->mJSVersion = version;
@@ -635,21 +604,26 @@ nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
   }
 
   // inline script
+  // Is this document sandboxed without 'allow-scripts'?
+  if (mDocument->GetSandboxFlags() & SANDBOXED_SCRIPTS) {
+    return false;
+  }
+
   nsCOMPtr<nsIContentSecurityPolicy> csp;
   rv = mDocument->NodePrincipal()->GetCsp(getter_AddRefs(csp));
   NS_ENSURE_SUCCESS(rv, false);
 
   if (csp) {
     PR_LOG(gCspPRLog, PR_LOG_DEBUG, ("New ScriptLoader i ****with CSP****"));
-    bool inlineOK;
-    rv = csp->GetAllowsInlineScript(&inlineOK);
+    bool inlineOK = true;
+    bool reportViolations = false;
+    rv = csp->GetAllowsInlineScript(&reportViolations, &inlineOK);
     NS_ENSURE_SUCCESS(rv, false);
 
-    if (!inlineOK) {
-      PR_LOG(gCspPRLog, PR_LOG_DEBUG, ("CSP blocked inline scripts (2)"));
+    if (reportViolations) {
       // gather information to log with violation report
       nsIURI* uri = mDocument->GetDocumentURI();
-      nsCAutoString asciiSpec;
+      nsAutoCString asciiSpec;
       uri->GetAsciiSpec(asciiSpec);
       nsAutoString scriptText;
       aElement->GetScriptText(scriptText);
@@ -664,6 +638,10 @@ nsScriptLoader::ProcessScriptElement(nsIScriptElement *aElement)
                                NS_ConvertUTF8toUTF16(asciiSpec),
                                scriptText,
                                aElement->GetScriptLineNumber());
+    }
+
+    if (!inlineOK) {
+      PR_LOG(gCspPRLog, PR_LOG_DEBUG, ("CSP blocked inline scripts (2)"));
       return false;
     }
   }
@@ -723,8 +701,6 @@ nsScriptLoader::ProcessRequest(nsScriptLoadRequest* aRequest)
   nsAFlatString* script;
   nsAutoString textData;
 
-  NS_TIME_FUNCTION;
-
   nsCOMPtr<nsIDocument> doc;
 
   nsCOMPtr<nsINode> scriptElem = do_QueryInterface(aRequest->mElement);
@@ -744,7 +720,7 @@ nsScriptLoader::ProcessRequest(nsScriptLoadRequest* aRequest)
   }
 
   nsCOMPtr<nsIScriptElement> oldParserInsertedScript;
-  PRUint32 parserCreated = aRequest->mElement->GetParserCreated();
+  uint32_t parserCreated = aRequest->mElement->GetParserCreated();
   if (parserCreated) {
     oldParserInsertedScript = mCurrentParserInsertedScript;
     mCurrentParserInsertedScript = aRequest->mElement;
@@ -789,7 +765,7 @@ void
 nsScriptLoader::FireScriptAvailable(nsresult aResult,
                                     nsScriptLoadRequest* aRequest)
 {
-  for (PRInt32 i = 0; i < mObservers.Count(); i++) {
+  for (int32_t i = 0; i < mObservers.Count(); i++) {
     nsCOMPtr<nsIScriptLoaderObserver> obs = mObservers[i];
     obs->ScriptAvailable(aResult, aRequest->mElement,
                          aRequest->mIsInline, aRequest->mURI,
@@ -803,7 +779,7 @@ void
 nsScriptLoader::FireScriptEvaluated(nsresult aResult,
                                     nsScriptLoadRequest* aRequest)
 {
-  for (PRInt32 i = 0; i < mObservers.Count(); i++) {
+  for (int32_t i = 0; i < mObservers.Count(); i++) {
     nsCOMPtr<nsIScriptLoaderObserver> obs = mObservers[i];
     obs->ScriptEvaluated(aResult, aRequest->mElement,
                          aRequest->mIsInline);
@@ -831,7 +807,7 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   }
 
   nsPIDOMWindow *pwin = mDocument->GetInnerWindow();
-  if (!pwin || !pwin->IsInnerWindow()) {
+  if (!pwin) {
     return NS_ERROR_FAILURE;
   }
   nsCOMPtr<nsIScriptGlobalObject> globalObject = do_QueryInterface(pwin);
@@ -852,6 +828,7 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   if (!context) {
     return NS_ERROR_FAILURE;
   }
+  AutoPushJSContext cx(context->GetNativeContext());
 
   bool oldProcessingScriptTag = context->GetProcessingScriptTag();
   context->SetProcessingScriptTag(true);
@@ -862,23 +839,25 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
 
   // It's very important to use aRequest->mURI, not the final URI of the channel
   // aRequest ended up getting script data from, as the script filename.
-  nsCAutoString url;
+  nsAutoCString url;
   nsContentUtils::GetWrapperSafeScriptFilename(mDocument, aRequest->mURI, url);
 
-  bool isUndefined;
-  rv = context->EvaluateString(aScript, globalObject->GetGlobalJSObject(),
-                               mDocument->NodePrincipal(),
-                               aRequest->mOriginPrincipal,
-                               url.get(), aRequest->mLineNo,
-                               JSVersion(aRequest->mJSVersion), nsnull,
-                               &isUndefined);
+  JSVersion version = JSVersion(aRequest->mJSVersion);
+  if (version != JSVERSION_UNKNOWN) {
+    JS::CompileOptions options(cx);
+    options.setFileAndLine(url.get(), aRequest->mLineNo)
+           .setVersion(JSVersion(aRequest->mJSVersion));
+    if (aRequest->mOriginPrincipal) {
+      options.setOriginPrincipals(nsJSPrincipals::get(aRequest->mOriginPrincipal));
+    }
+    JS::Rooted<JSObject*> global(cx, globalObject->GetGlobalJSObject());
+    rv = context->EvaluateString(aScript, global,
+                                 options, /* aCoerceToString = */ false, nullptr);
+  }
 
   // Put the old script back in case it wants to do anything else.
   mCurrentScript = oldCurrent;
 
-  JSContext *cx = nsnull; // Initialize this to keep GCC happy.
-  cx = context->GetNativeContext();
-  JSAutoRequest ar(cx);
   context->SetProcessingScriptTag(oldProcessingScriptTag);
   return rv;
 }
@@ -915,7 +894,7 @@ nsScriptLoader::ProcessPendingRequests()
     ProcessRequest(request);
   }
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   while (mEnabled && i < mAsyncRequests.Length()) {
     if (!mAsyncRequests[i]->mLoading) {
       request.swap(mAsyncRequests[i]);
@@ -987,7 +966,7 @@ nsScriptLoader::ReadyToExecuteScripts()
 
 // This function was copied from nsParser.cpp. It was simplified a bit.
 static bool
-DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oCharset)
+DetectByteOrderMark(const unsigned char* aBytes, int32_t aLen, nsCString& oCharset)
 {
   if (aLen < 2)
     return false;
@@ -1004,14 +983,14 @@ DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oChars
     if (0xFF == aBytes[1]) {
       // FE FF
       // UTF-16, big-endian
-      oCharset.Assign("UTF-16");
+      oCharset.Assign("UTF-16BE");
     }
     break;
   case 0xFF:
     if (0xFE == aBytes[1]) {
       // FF FE
       // UTF-16, little-endian
-      oCharset.Assign("UTF-16");
+      oCharset.Assign("UTF-16LE");
     }
     break;
   }
@@ -1019,8 +998,8 @@ DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oChars
 }
 
 /* static */ nsresult
-nsScriptLoader::ConvertToUTF16(nsIChannel* aChannel, const PRUint8* aData,
-                               PRUint32 aLength, const nsAString& aHintCharset,
+nsScriptLoader::ConvertToUTF16(nsIChannel* aChannel, const uint8_t* aData,
+                               uint32_t aLength, const nsAString& aHintCharset,
                                nsIDocument* aDocument, nsString& aString)
 {
   if (!aLength) {
@@ -1028,84 +1007,74 @@ nsScriptLoader::ConvertToUTF16(nsIChannel* aChannel, const PRUint8* aData,
     return NS_OK;
   }
 
-  nsCAutoString characterSet;
+  // The encoding info precedence is as follows from high to low:
+  // The BOM
+  // HTTP Content-Type (if name recognized)
+  // charset attribute (if name recognized)
+  // The encoding of the document
 
-  nsresult rv = NS_OK;
-  if (aChannel) {
-    rv = aChannel->GetContentCharset(characterSet);
-  }
-
-  if (!aHintCharset.IsEmpty() && (NS_FAILED(rv) || characterSet.IsEmpty())) {
-    // charset name is always ASCII.
-    LossyCopyUTF16toASCII(aHintCharset, characterSet);
-  }
-
-  if (NS_FAILED(rv) || characterSet.IsEmpty()) {
-    DetectByteOrderMark(aData, aLength, characterSet);
-  }
-
-  if (characterSet.IsEmpty() && aDocument) {
-    // charset from document default
-    characterSet = aDocument->GetDocumentCharacterSet();
-  }
-
-  if (characterSet.IsEmpty()) {
-    // fall back to ISO-8859-1, see bug 118404
-    characterSet.AssignLiteral("ISO-8859-1");
-  }
+  nsAutoCString charset;
 
   nsCOMPtr<nsICharsetConverterManager> charsetConv =
-    do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
+    do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID);
 
   nsCOMPtr<nsIUnicodeDecoder> unicodeDecoder;
 
-  if (NS_SUCCEEDED(rv) && charsetConv) {
-    rv = charsetConv->GetUnicodeDecoder(characterSet.get(),
-                                        getter_AddRefs(unicodeDecoder));
-    if (NS_FAILED(rv)) {
-      // fall back to ISO-8859-1 if charset is not supported. (bug 230104)
-      rv = charsetConv->GetUnicodeDecoderRaw("ISO-8859-1",
-                                             getter_AddRefs(unicodeDecoder));
-    }
+  if (DetectByteOrderMark(aData, aLength, charset)) {
+    // charset is now "UTF-8" or "UTF-16". The UTF-16 decoder will re-sniff
+    // the BOM for endianness. Both the UTF-16 and the UTF-8 decoder will
+    // take care of swallowing the BOM.
+    charsetConv->GetUnicodeDecoderRaw(charset.get(),
+                                      getter_AddRefs(unicodeDecoder));
   }
 
-  // converts from the charset to unicode
-  if (NS_SUCCEEDED(rv)) {
-    PRInt32 unicodeLength = 0;
-
-    rv = unicodeDecoder->GetMaxLength(reinterpret_cast<const char*>(aData),
-                                      aLength, &unicodeLength);
-    if (NS_SUCCEEDED(rv)) {
-      if (!EnsureStringLength(aString, unicodeLength))
-        return NS_ERROR_OUT_OF_MEMORY;
-
-      PRUnichar *ustr = aString.BeginWriting();
-
-      PRInt32 consumedLength = 0;
-      PRInt32 originalLength = aLength;
-      PRInt32 convertedLength = 0;
-      PRInt32 bufferLength = unicodeLength;
-      do {
-        rv = unicodeDecoder->Convert(reinterpret_cast<const char*>(aData),
-                                     (PRInt32 *) &aLength, ustr,
-                                     &unicodeLength);
-        if (NS_FAILED(rv)) {
-          // if we failed, we consume one byte, replace it with U+FFFD
-          // and try the conversion again.
-          ustr[unicodeLength++] = (PRUnichar)0xFFFD;
-          ustr += unicodeLength;
-
-          unicodeDecoder->Reset();
-        }
-        aData += ++aLength;
-        consumedLength += aLength;
-        aLength = originalLength - consumedLength;
-        convertedLength += unicodeLength;
-        unicodeLength = bufferLength - convertedLength;
-      } while (NS_FAILED(rv) && (originalLength > consumedLength) && (bufferLength > convertedLength));
-      aString.SetLength(convertedLength);
-    }
+  if (!unicodeDecoder &&
+      aChannel &&
+      NS_SUCCEEDED(aChannel->GetContentCharset(charset)) &&
+      !charset.IsEmpty()) {
+    charsetConv->GetUnicodeDecoder(charset.get(),
+                                   getter_AddRefs(unicodeDecoder));
   }
+
+  if (!unicodeDecoder && !aHintCharset.IsEmpty()) {
+    CopyUTF16toUTF8(aHintCharset, charset);
+    charsetConv->GetUnicodeDecoder(charset.get(),
+                                   getter_AddRefs(unicodeDecoder));
+  }
+
+  if (!unicodeDecoder && aDocument) {
+    charset = aDocument->GetDocumentCharacterSet();
+    charsetConv->GetUnicodeDecoderRaw(charset.get(),
+                                      getter_AddRefs(unicodeDecoder));
+  }
+
+  if (!unicodeDecoder) {
+    // Curiously, there are various callers that don't pass aDocument. The
+    // fallback in the old code was ISO-8859-1, which behaved like
+    // windows-1252. Saying windows-1252 for clarity and for compliance
+    // with the Encoding Standard.
+    charsetConv->GetUnicodeDecoderRaw("windows-1252",
+                                      getter_AddRefs(unicodeDecoder));
+  }
+
+  int32_t unicodeLength = 0;
+
+  nsresult rv =
+    unicodeDecoder->GetMaxLength(reinterpret_cast<const char*>(aData),
+                                 aLength, &unicodeLength);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!aString.SetLength(unicodeLength, fallible_t())) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  PRUnichar *ustr = aString.BeginWriting();
+
+  rv = unicodeDecoder->Convert(reinterpret_cast<const char*>(aData),
+                               (int32_t *) &aLength, ustr,
+                               &unicodeLength);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  aString.SetLength(unicodeLength);
   return rv;
 }
 
@@ -1113,8 +1082,8 @@ NS_IMETHODIMP
 nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
                                  nsISupports* aContext,
                                  nsresult aStatus,
-                                 PRUint32 aStringLen,
-                                 const PRUint8* aString)
+                                 uint32_t aStringLen,
+                                 const uint8_t* aString)
 {
   nsScriptLoadRequest* request = static_cast<nsScriptLoadRequest*>(aContext);
   NS_ASSERTION(request, "null request in stream complete handler");
@@ -1129,7 +1098,7 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
         mXSLTRequests.RemoveElement(request)) {
       FireScriptAvailable(rv, request);
     } else if (mParserBlockingRequest == request) {
-      mParserBlockingRequest = nsnull;
+      mParserBlockingRequest = nullptr;
       UnblockParser(request);
       FireScriptAvailable(rv, request);
       ContinueParserAsync(request);
@@ -1160,8 +1129,8 @@ nsresult
 nsScriptLoader::PrepareLoadedRequest(nsScriptLoadRequest* aRequest,
                                      nsIStreamLoader* aLoader,
                                      nsresult aStatus,
-                                     PRUint32 aStringLen,
-                                     const PRUint8* aString)
+                                     uint32_t aStringLen,
+                                     const uint8_t* aString)
 {
   if (NS_FAILED(aStatus)) {
     return aStatus;
@@ -1213,10 +1182,6 @@ nsScriptLoader::PrepareLoadedRequest(nsScriptLoadRequest* aRequest,
                         aRequest->mScriptText);
 
     NS_ENSURE_SUCCESS(rv, rv);
-
-    if (!ShouldExecuteScript(mDocument, channel)) {
-      return NS_ERROR_NOT_AVAILABLE;
-    }
   }
 
   // This assertion could fire errorously if we ran out of memory when
@@ -1237,36 +1202,6 @@ nsScriptLoader::PrepareLoadedRequest(nsScriptLoadRequest* aRequest,
   return NS_OK;
 }
 
-/* static */
-bool
-nsScriptLoader::ShouldExecuteScript(nsIDocument* aDocument,
-                                    nsIChannel* aChannel)
-{
-  if (!aChannel) {
-    return false;
-  }
-
-  bool hasCert;
-  nsIPrincipal* docPrincipal = aDocument->NodePrincipal();
-  docPrincipal->GetHasCertificate(&hasCert);
-  if (!hasCert) {
-    return true;
-  }
-
-  nsCOMPtr<nsIPrincipal> channelPrincipal;
-  nsresult rv = nsContentUtils::GetSecurityManager()->
-    GetChannelPrincipal(aChannel, getter_AddRefs(channelPrincipal));
-  NS_ENSURE_SUCCESS(rv, false);
-
-  NS_ASSERTION(channelPrincipal, "Gotta have a principal here!");
-
-  // If the channel principal isn't at least as powerful as the
-  // document principal, then we don't execute the script.
-  bool subsumes;
-  rv = channelPrincipal->Subsumes(docPrincipal, &subsumes);
-  return NS_SUCCEEDED(rv) && subsumes;
-}
-
 void
 nsScriptLoader::ParsingComplete(bool aTerminated)
 {
@@ -1281,7 +1216,7 @@ nsScriptLoader::ParsingComplete(bool aTerminated)
     mAsyncRequests.Clear();
     mNonAsyncExternalScriptInsertedRequests.Clear();
     mXSLTRequests.Clear();
-    mParserBlockingRequest = nsnull;
+    mParserBlockingRequest = nullptr;
   }
 
   // Have to call this even if aTerminated so we'll correctly unblock
@@ -1292,7 +1227,8 @@ nsScriptLoader::ParsingComplete(bool aTerminated)
 void
 nsScriptLoader::PreloadURI(nsIURI *aURI, const nsAString &aCharset,
                            const nsAString &aType,
-                           const nsAString &aCrossOrigin)
+                           const nsAString &aCrossOrigin,
+                           bool aScriptFromHead)
 {
   // Check to see if scripts has been turned off.
   if (!mEnabled || !mDocument->IsScriptEnabled()) {
@@ -1300,12 +1236,12 @@ nsScriptLoader::PreloadURI(nsIURI *aURI, const nsAString &aCharset,
   }
 
   nsRefPtr<nsScriptLoadRequest> request =
-    new nsScriptLoadRequest(nsnull, 0,
-                            nsGenericElement::StringToCORSMode(aCrossOrigin));
+    new nsScriptLoadRequest(nullptr, 0,
+                            Element::StringToCORSMode(aCrossOrigin));
   request->mURI = aURI;
   request->mIsInline = false;
   request->mLoading = true;
-  nsresult rv = StartLoad(request, aType);
+  nsresult rv = StartLoad(request, aType, aScriptFromHead);
   if (NS_FAILED(rv)) {
     return;
   }
