@@ -4,23 +4,22 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "SVGPathData.h"
-#include "SVGPathSegUtils.h"
-#include "nsSVGElement.h"
-#include "nsDOMError.h"
+#include "gfxPlatform.h"
+#include "nsError.h"
 #include "nsString.h"
-#include "nsSVGUtils.h"
-#include "string.h"
 #include "nsSVGPathDataParser.h"
 #include "nsSVGPathGeometryElement.h" // for nsSVGMark
-#include "gfxPlatform.h"
 #include <stdarg.h>
+#include "SVGContentUtils.h"
+#include "SVGPathSegUtils.h"
+#include <algorithm>
 
 using namespace mozilla;
 
-static bool IsMoveto(PRUint16 aSegType)
+static bool IsMoveto(uint16_t aSegType)
 {
-  return aSegType == nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS ||
-         aSegType == nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL;
+  return aSegType == PATHSEG_MOVETO_ABS ||
+         aSegType == PATHSEG_MOVETO_REL;
 }
 
 nsresult
@@ -42,7 +41,7 @@ SVGPathData::GetValueAsString(nsAString& aValue) const
   if (!Length()) {
     return;
   }
-  PRUint32 i = 0;
+  uint32_t i = 0;
   for (;;) {
     nsAutoString segAsString;
     SVGPathSegUtils::GetValueAsString(&mData[i], segAsString);
@@ -69,17 +68,17 @@ SVGPathData::SetValueFromString(const nsAString& aValue)
 }
 
 nsresult
-SVGPathData::AppendSeg(PRUint32 aType, ...)
+SVGPathData::AppendSeg(uint32_t aType, ...)
 {
-  PRUint32 oldLength = mData.Length();
-  PRUint32 newLength = oldLength + 1 + SVGPathSegUtils::ArgCountForType(aType);
+  uint32_t oldLength = mData.Length();
+  uint32_t newLength = oldLength + 1 + SVGPathSegUtils::ArgCountForType(aType);
   if (!mData.SetLength(newLength)) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
   mData[oldLength] = SVGPathSegUtils::EncodeType(aType);
   va_list args;
   va_start(args, aType);
-  for (PRUint32 i = oldLength + 1; i < newLength; ++i) {
+  for (uint32_t i = oldLength + 1; i < newLength; ++i) {
     // NOTE! 'float' is promoted to 'double' when passed through '...'!
     mData[i] = float(va_arg(args, double));
   }
@@ -92,7 +91,7 @@ SVGPathData::GetPathLength() const
 {
   SVGPathTraversalState state;
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   while (i < mData.Length()) {
     SVGPathSegUtils::TraversePathSegment(&mData[i], state);
     i += 1 + SVGPathSegUtils::ArgCountForType(mData[i]);
@@ -104,10 +103,10 @@ SVGPathData::GetPathLength() const
 }
 
 #ifdef DEBUG
-PRUint32
+uint32_t
 SVGPathData::CountItems() const
 {
-  PRUint32 i = 0, count = 0;
+  uint32_t i = 0, count = 0;
 
   while (i < mData.Length()) {
     i += 1 + SVGPathSegUtils::ArgCountForType(mData[i]);
@@ -126,7 +125,7 @@ SVGPathData::GetSegmentLengths(nsTArray<double> *aLengths) const
   aLengths->Clear();
   SVGPathTraversalState state;
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   while (i < mData.Length()) {
     state.length = 0.0;
     SVGPathSegUtils::TraversePathSegment(&mData[i], state);
@@ -149,9 +148,9 @@ SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOu
 
   aOutput->Clear();
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   while (i < mData.Length()) {
-    PRUint32 segType = SVGPathSegUtils::DecodeType(mData[i]);
+    uint32_t segType = SVGPathSegUtils::DecodeType(mData[i]);
     SVGPathSegUtils::TraversePathSegment(&mData[i], state);
 
     // We skip all moveto commands except an initial moveto. See the text 'A
@@ -164,8 +163,8 @@ SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOu
     // this case an equal amount of time is spent on each path segment,
     // except on moveto segments which are jumped over immediately.
 
-    if (i == 0 || (segType != nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS &&
-                   segType != nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL)) {
+    if (i == 0 || (segType != PATHSEG_MOVETO_ABS &&
+                   segType != PATHSEG_MOVETO_REL)) {
       if (!aOutput->AppendElement(state.length)) {
         return false;
       }
@@ -178,7 +177,7 @@ SVGPathData::GetDistancesFromOriginToEndsOfVisibleSegments(nsTArray<double> *aOu
   return true;
 }
 
-PRUint32
+uint32_t
 SVGPathData::GetPathSegAtLength(float aDistance) const
 {
   // TODO [SVGWG issue] get specified what happen if 'aDistance' < 0, or
@@ -186,7 +185,7 @@ SVGPathData::GetPathSegAtLength(float aDistance) const
   // Return -1? Throwing would better help authors avoid tricky bugs (DOM
   // could do that if we return -1).
 
-  PRUint32 i = 0, segIndex = 0;
+  uint32_t i = 0, segIndex = 0;
   SVGPathTraversalState state;
 
   while (i < mData.Length()) {
@@ -200,7 +199,7 @@ SVGPathData::GetPathSegAtLength(float aDistance) const
 
   NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
 
-  return NS_MAX(0U, segIndex - 1); // -1 because while loop takes us 1 too far
+  return std::max(0U, segIndex - 1); // -1 because while loop takes us 1 too far
 }
 
 /**
@@ -242,7 +241,7 @@ ApproximateZeroLengthSubpathSquareCaps(const gfxPoint &aPoint, gfxContext *aCtx)
     if (capsAreSquare && !subpathHasLength && subpathContainsNonArc &&        \
         SVGPathSegUtils::IsValidType(prevSegType) &&                          \
         (!IsMoveto(prevSegType) ||                                            \
-         segType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH)) {                   \
+         segType == PATHSEG_CLOSEPATH)) {                                     \
       ApproximateZeroLengthSubpathSquareCaps(segStart, aCtx);                 \
     }                                                                         \
   } while(0)
@@ -250,7 +249,7 @@ ApproximateZeroLengthSubpathSquareCaps(const gfxPoint &aPoint, gfxContext *aCtx)
 void
 SVGPathData::ConstructPath(gfxContext *aCtx) const
 {
-  if (!mData.Length() || !IsMoveto(SVGPathSegUtils::DecodeType(mData[0]))) {
+  if (mData.IsEmpty() || !IsMoveto(SVGPathSegUtils::DecodeType(mData[0]))) {
     return; // paths without an initial moveto are invalid
   }
 
@@ -258,7 +257,8 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
   bool subpathHasLength = false;  // visual length
   bool subpathContainsNonArc = false;
 
-  PRUint32 segType, prevSegType = nsIDOMSVGPathSeg::PATHSEG_UNKNOWN;
+  uint32_t segType     = PATHSEG_UNKNOWN;
+  uint32_t prevSegType = PATHSEG_UNKNOWN;
   gfxPoint pathStart(0.0, 0.0); // start point of [sub]path
   gfxPoint segStart(0.0, 0.0);
   gfxPoint segEnd;
@@ -269,14 +269,14 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
   // then cp2 is its second control point. If the previous segment was a
   // quadratic curve, then cp1 is its (only) control point.
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   while (i < mData.Length()) {
     segType = SVGPathSegUtils::DecodeType(mData[i++]);
-    PRUint32 argCount = SVGPathSegUtils::ArgCountForType(segType);
+    uint32_t argCount = SVGPathSegUtils::ArgCountForType(segType);
 
     switch (segType)
     {
-    case nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH:
+    case PATHSEG_CLOSEPATH:
       // set this early to allow drawing of square caps for "M{x},{y} Z":
       subpathContainsNonArc = true;
       MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
@@ -284,7 +284,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       aCtx->ClosePath();
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS:
+    case PATHSEG_MOVETO_ABS:
       MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
       pathStart = segEnd = gfxPoint(mData[i], mData[i+1]);
       aCtx->MoveTo(segEnd);
@@ -292,7 +292,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = false;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL:
+    case PATHSEG_MOVETO_REL:
       MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
       pathStart = segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
       aCtx->MoveTo(segEnd);
@@ -300,7 +300,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = false;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_ABS:
+    case PATHSEG_LINETO_ABS:
       segEnd = gfxPoint(mData[i], mData[i+1]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -309,7 +309,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_REL:
+    case PATHSEG_LINETO_REL:
       segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -318,7 +318,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_ABS:
+    case PATHSEG_CURVETO_CUBIC_ABS:
       cp1 = gfxPoint(mData[i], mData[i+1]);
       cp2 = gfxPoint(mData[i+2], mData[i+3]);
       segEnd = gfxPoint(mData[i+4], mData[i+5]);
@@ -329,7 +329,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_REL:
+    case PATHSEG_CURVETO_CUBIC_REL:
       cp1 = segStart + gfxPoint(mData[i], mData[i+1]);
       cp2 = segStart + gfxPoint(mData[i+2], mData[i+3]);
       segEnd = segStart + gfxPoint(mData[i+4], mData[i+5]);
@@ -340,7 +340,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_ABS:
       cp1 = gfxPoint(mData[i], mData[i+1]);
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -353,7 +353,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_REL:
+    case PATHSEG_CURVETO_QUADRATIC_REL:
       cp1 = segStart + gfxPoint(mData[i], mData[i+1]);
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -366,12 +366,12 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_REL:
+    case PATHSEG_ARC_ABS:
+    case PATHSEG_ARC_REL:
     {
       gfxPoint radii(mData[i], mData[i+1]);
       segEnd = gfxPoint(mData[i+5], mData[i+6]);
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_ARC_REL) {
+      if (segType == PATHSEG_ARC_REL) {
         segEnd += segStart;
       }
       if (segEnd != segStart) {
@@ -391,7 +391,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_ABS:
+    case PATHSEG_LINETO_HORIZONTAL_ABS:
       segEnd = gfxPoint(mData[i], segStart.y);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -400,7 +400,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_REL:
+    case PATHSEG_LINETO_HORIZONTAL_REL:
       segEnd = segStart + gfxPoint(mData[i], 0.0f);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -409,7 +409,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_ABS:
+    case PATHSEG_LINETO_VERTICAL_ABS:
       segEnd = gfxPoint(segStart.x, mData[i]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -418,7 +418,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_REL:
+    case PATHSEG_LINETO_VERTICAL_REL:
       segEnd = segStart + gfxPoint(0.0f, mData[i]);
       aCtx->LineTo(segEnd);
       if (!subpathHasLength) {
@@ -427,7 +427,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
       cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ? segStart * 2 - cp2 : segStart;
       cp2 = gfxPoint(mData[i],   mData[i+1]);
       segEnd = gfxPoint(mData[i+2], mData[i+3]);
@@ -438,7 +438,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
       cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ? segStart * 2 - cp2 : segStart;
       cp2 = segStart + gfxPoint(mData[i], mData[i+1]);
       segEnd = segStart + gfxPoint(mData[i+2], mData[i+3]);
@@ -449,7 +449,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
       cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ? segStart * 2 - cp1 : segStart;
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -462,7 +462,7 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
       subpathContainsNonArc = true;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
       cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ? segStart * 2 - cp1 : segStart;
       // Convert quadratic curve to cubic curve:
       tcp1 = segStart + (cp1 - segStart) * 2 / 3;
@@ -485,6 +485,8 @@ SVGPathData::ConstructPath(gfxContext *aCtx) const
   }
 
   NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
+  NS_ABORT_IF_FALSE(prevSegType == segType,
+                    "prevSegType should be left at the final segType");
 
   MAYBE_APPROXIMATE_ZERO_LENGTH_SUBPATH_SQUARE_CAPS;
 }
@@ -532,16 +534,16 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
   float pathStartAngle = 0.0f;
 
   // info on previous segment:
-  PRUint16 prevSegType = nsIDOMSVGPathSeg::PATHSEG_UNKNOWN;
+  uint16_t prevSegType = PATHSEG_UNKNOWN;
   gfxPoint prevSegEnd(0.0, 0.0);
   float prevSegEndAngle = 0.0f;
   gfxPoint prevCP; // if prev seg was a bezier, this was its last control point
 
-  PRUint32 i = 0;
+  uint32_t i = 0;
   while (i < mData.Length()) {
 
     // info on current segment:
-    PRUint16 segType =
+    uint16_t segType =
       SVGPathSegUtils::DecodeType(mData[i++]); // advances i to args
     gfxPoint &segStart = prevSegEnd;
     gfxPoint segEnd;
@@ -549,14 +551,14 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
 
     switch (segType) // to find segStartAngle, segEnd and segEndAngle
     {
-    case nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH:
+    case PATHSEG_CLOSEPATH:
       segEnd = pathStart;
       segStartAngle = segEndAngle = AngleOfVectorF(segEnd - segStart);
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_MOVETO_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_MOVETO_ABS) {
+    case PATHSEG_MOVETO_ABS:
+    case PATHSEG_MOVETO_REL:
+      if (segType == PATHSEG_MOVETO_ABS) {
         segEnd = gfxPoint(mData[i], mData[i+1]);
       } else {
         segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
@@ -568,9 +570,9 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       i += 2;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_LINETO_ABS) {
+    case PATHSEG_LINETO_ABS:
+    case PATHSEG_LINETO_REL:
+      if (segType == PATHSEG_LINETO_ABS) {
         segEnd = gfxPoint(mData[i], mData[i+1]);
       } else {
         segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
@@ -579,11 +581,11 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       i += 2;
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_REL:
+    case PATHSEG_CURVETO_CUBIC_ABS:
+    case PATHSEG_CURVETO_CUBIC_REL:
     {
       gfxPoint cp1, cp2; // control points
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_ABS) {
+      if (segType == PATHSEG_CURVETO_CUBIC_ABS) {
         cp1 = gfxPoint(mData[i],   mData[i+1]);
         cp2 = gfxPoint(mData[i+2], mData[i+3]);
         segEnd = gfxPoint(mData[i+4], mData[i+5]);
@@ -605,11 +607,11 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_REL:
+    case PATHSEG_CURVETO_QUADRATIC_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_REL:
     {
       gfxPoint cp1, cp2; // control points
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_ABS) {
+      if (segType == PATHSEG_CURVETO_QUADRATIC_ABS) {
         cp1 = gfxPoint(mData[i],   mData[i+1]);
         segEnd = gfxPoint(mData[i+2], mData[i+3]);
       } else {
@@ -623,15 +625,15 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_ARC_REL:
+    case PATHSEG_ARC_ABS:
+    case PATHSEG_ARC_REL:
     {
       double rx = mData[i];
       double ry = mData[i+1];
       double angle = mData[i+2];
       bool largeArcFlag = mData[i+3] != 0.0f;
       bool sweepFlag = mData[i+4] != 0.0f;
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_ARC_ABS) {
+      if (segType == PATHSEG_ARC_ABS) {
         segEnd = gfxPoint(mData[i+5], mData[i+6]);
       } else {
         segEnd = segStart + gfxPoint(mData[i+5], mData[i+6]);
@@ -726,9 +728,9 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_LINETO_HORIZONTAL_ABS) {
+    case PATHSEG_LINETO_HORIZONTAL_ABS:
+    case PATHSEG_LINETO_HORIZONTAL_REL:
+      if (segType == PATHSEG_LINETO_HORIZONTAL_ABS) {
         segEnd = gfxPoint(mData[i++], segStart.y);
       } else {
         segEnd = segStart + gfxPoint(mData[i++], 0.0f);
@@ -736,9 +738,9 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       segStartAngle = segEndAngle = AngleOfVectorF(segEnd - segStart);
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_REL:
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_LINETO_VERTICAL_ABS) {
+    case PATHSEG_LINETO_VERTICAL_ABS:
+    case PATHSEG_LINETO_VERTICAL_REL:
+      if (segType == PATHSEG_LINETO_VERTICAL_ABS) {
         segEnd = gfxPoint(segStart.x, mData[i++]);
       } else {
         segEnd = segStart + gfxPoint(0.0f, mData[i++]);
@@ -746,13 +748,13 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       segStartAngle = segEndAngle = AngleOfVectorF(segEnd - segStart);
       break;
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_CUBIC_SMOOTH_REL:
     {
       gfxPoint cp1 = SVGPathSegUtils::IsCubicType(prevSegType) ?
                        segStart * 2 - prevCP : segStart;
       gfxPoint cp2;
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_CUBIC_SMOOTH_ABS) {
+      if (segType == PATHSEG_CURVETO_CUBIC_SMOOTH_ABS) {
         cp2 = gfxPoint(mData[i], mData[i+1]);
         segEnd = gfxPoint(mData[i+2], mData[i+3]);
       } else {
@@ -772,13 +774,13 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       break;
     }
 
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
-    case nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS:
+    case PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL:
     {
       gfxPoint cp1 = SVGPathSegUtils::IsQuadraticType(prevSegType) ?
                        segStart * 2 - prevCP : segStart;
       gfxPoint cp2;
-      if (segType == nsIDOMSVGPathSeg::PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS) {
+      if (segType == PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS) {
         segEnd = gfxPoint(mData[i], mData[i+1]);
       } else {
         segEnd = segStart + gfxPoint(mData[i], mData[i+1]);
@@ -805,12 +807,12 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
         pathStartAngle = mark.angle = segStartAngle;
       } else if (IsMoveto(segType) && !IsMoveto(prevSegType)) {
         // end of a subpath
-        if (prevSegType != nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH)
+        if (prevSegType != PATHSEG_CLOSEPATH)
           mark.angle = prevSegEndAngle;
       } else {
-        if (!(segType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH &&
-              prevSegType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH))
-          mark.angle = nsSVGUtils::AngleBisect(prevSegEndAngle, segStartAngle);
+        if (!(segType == PATHSEG_CLOSEPATH &&
+              prevSegType == PATHSEG_CLOSEPATH))
+          mark.angle = SVGContentUtils::AngleBisect(prevSegEndAngle, segStartAngle);
       }
     }
 
@@ -821,11 +823,11 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
       return;
     }
 
-    if (segType == nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH &&
-        prevSegType != nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH) {
+    if (segType == PATHSEG_CLOSEPATH &&
+        prevSegType != PATHSEG_CLOSEPATH) {
       aMarks->ElementAt(aMarks->Length() - 1).angle =
         //aMarks->ElementAt(pathStartIndex).angle =
-        nsSVGUtils::AngleBisect(segEndAngle, pathStartAngle);
+        SVGContentUtils::AngleBisect(segEndAngle, pathStartAngle);
     }
 
     prevSegType = segType;
@@ -836,7 +838,7 @@ SVGPathData::GetMarkerPositioningData(nsTArray<nsSVGMark> *aMarks) const
   NS_ABORT_IF_FALSE(i == mData.Length(), "Very, very bad - mData corrupt");
 
   if (aMarks->Length() &&
-      prevSegType != nsIDOMSVGPathSeg::PATHSEG_CLOSEPATH)
+      prevSegType != PATHSEG_CLOSEPATH)
     aMarks->ElementAt(aMarks->Length() - 1).angle = prevSegEndAngle;
 }
 
