@@ -157,11 +157,7 @@ AccessibleEventsViewer.prototype =
     var object = this.mView.getObject(idx);
 
     // Set subject for right panel view.
-    var subject = object.node || object.accessnode;
-    if (object.node) {
-      subject[" accessible "] = object.accessnode;
-    }
-    this.mSelection = subject;
+    this.mSelection = object.accessnode;
 
     // Set parameters for right panel view.
     if (this.pane.panelset.panelCount > 1) {
@@ -187,6 +183,13 @@ AccessibleEventsViewer.prototype =
     this.mWatchView.updateHandlerState(aState);
   },
 
+  onWatchViewKeyPressed: function onWatchViewKeyPressed(aEvent)
+  {
+    // SPACE key was pressed. Toggle the row's Watched column tick.
+    if (aEvent.charCode == KeyEvent.DOM_VK_SPACE)
+      this.mWatchView.toggleEventWatched();
+  },
+
   /**
    * Clear the list of handled events.
    */
@@ -274,9 +277,10 @@ function AccessibleEventsView(aObject, aWatchView)
 AccessibleEventsView.prototype = new inBaseTreeView();
 
 /**
- * Global variable used to store user's event handler output from helper
- * functions.
+ * Global variables used to store event object and user's event handler output
+ * from helper functions.
  */
+var gEvent = null;
 var gEventHandlerOutput = [ ];
 
 AccessibleEventsView.prototype.observe =
@@ -296,33 +300,26 @@ function observe(aSubject, aTopic, aData)
   }
 
   // Ignore events having target not in subtree of currently inspected
-  // accessible.
+  // document accessible.
   if (!this.canSkipTreeTraversal) {
-    var parentAccessible = accessible;
-    while (parentAccessible) {
-      // The target accessible is inspected accessible or its child.
-      if (parentAccessible == this.mAccessible) {
+    var parentDocAccessible = accessible.document;
+    while (true) {
+      // The target accessible is inspected document accessible or its child.
+      if (parentDocAccessible == this.mAccessible) {
         break;
       }
 
       // Ignore events on this DOM inspector to avoid a mess.
-      if (parentAccessible == this.mDOMIRootDocumentAccessible) {
+      if (parentDocAccessible == this.mDOMIRootDocumentAccessible) {
         return;
       }
 
-      try {
-        parentAccessible = parentAccessible.parent;
-
-        // Ignore events that aren't in subtree of inspected accessible.
-        if (!parentAccessible) {
-          return;
-        }
-      } catch (e) {
-        // Accessibles on hide events are unattached from tree and parent throws
-        // an exception, we should dump all of them not depending on document they
-        // used to be until we have better approach.
-        break;
+      // Ignore events that aren't in subtree of inspected accessible.
+      if (!parentDocAccessible.parent || !parentDocAccessible.parent.document) {
+        return;
       }
+
+      parentDocAccessible = parentDocAccessible.parent.document;
     }
   }
 
@@ -332,6 +329,7 @@ function observe(aSubject, aTopic, aData)
     return;
 
   // Execute user handlers.
+  gEvent = event;
   gEventHandlerOutput = [ ];
   var expr = this.mWatchView.getHandlerExpr(type);
   if (expr) {
@@ -618,6 +616,16 @@ function WatchAccessibleEventsListView()
     return "";
   }
 
+  this.toggleEventWatched = function watchview_toggleEventWatched()
+  {
+    var idx = this.selection.currentIndex;
+    var colWatched = this.mTree.columns.welIsWatched;
+    var newValue = !this.getCellValue(idx, colWatched);
+
+    // setCellValue() needs the new value to be passed as a string.
+    this.setCellValue(idx, colWatched, newValue.toString());
+  }
+
   /**
    * Updates state and value of handler source editor.
    */
@@ -1083,28 +1091,28 @@ inAccTreeView.prototype.getRowProperties =
 {
   var data = this.getDataAt(aRowIdx);
   if (data && data.properties) {
+    if (!aProperties)
+      return data.properties.join(" ");
+
     for (let i = 0; i < data.properties.length; i++) {
       var atom = this.createAtom(data.properties[i]);
       aProperties.AppendElement(atom);
     }
   }
+
+  return "";
 };
 
 inAccTreeView.prototype.getCellProperties =
   function inAccTreeView_getCellProperties(aRowIdx, aCol, aProperties)
 {
-  var data = this.getDataAt(aRowIdx);
-  if (data && data.properties) {
-    for (let i = 0; i < data.properties.length; i++) {
-      var atom = this.createAtom(data.properties[i]);
-      aProperties.AppendElement(atom);
-    }
-  }
+  return this.getRowProperties(aRowIdx, aProperties);
 };
 
 // Initialization
 inAccTreeView.prototype.generateChildren =
-  function inAccTreeView_generateChildren(aAccessible, aHighlightList, aParent)
+  function inAccTreeView_generateChildren(aAccessible, aHighlightList, aParent,
+                                          aIsUnattached)
 {
   var data = {
     properties: []
@@ -1112,13 +1120,19 @@ inAccTreeView.prototype.generateChildren =
 
   var accessible = QIAccessNode(aAccessible);
 
-  // Add row and cells.
+  // Highlight the row for accessible from the list.
   var isHighlighted =
     aHighlightList && aHighlightList.indexOf(aAccessible) != -1;
   if (isHighlighted) {
     data.properties.push("highlight");
   }
 
+  // Gray out the row for accessible unattached from the tree.
+  if (aIsUnattached) {
+    data.properties.push("grayout");
+  }
+
+  // Add cells data.
   data["outputtreeRole"] = accRetrieval.getStringRole(aAccessible.role);
   data["outputtreeName"] = aAccessible.name;
   data["outputtreeNodename"] = accessible.DOMNode.nodeName;
@@ -1128,11 +1142,55 @@ inAccTreeView.prototype.generateChildren =
   var parent = this.appendChild(aParent, data);
   var nodesToExpand = null;
 
+  // Insert highlighted row for target of handled hide event if it's specified
+  // in the list (works for Gecko versions higher 13).
+  var containsUnattached = false;
+  var accBeforeUnattached = null;
+  if ("nsIAccessibleHideEvent" in Ci &&
+      gEvent instanceof Ci.nsIAccessibleHideEvent &&
+      gEvent.targetParent == aAccessible) {
+    containsUnattached = true;
+    try {
+      if (gEvent.targetNextSibling.parent == aAccessible) {
+        accBeforeUnattached = gEvent.targetNextSibling;
+      }
+    } catch (e) {
+    }
+    try {
+      if (!accBeforeUnattached &&
+          gEvent.targetPrevSibling.parent == aAccessible) {
+        accBeforeUnattached = gEvent.targetPrevSibling.nextSibling;
+      }
+    } catch (e) {
+    }
+  }
+
   // Add children.
   var childCount = aAccessible.childCount;
   for (let i = 0; i < childCount; i++) {
+    var child = aAccessible.getChildAt(i);
+
+    // Add unattached child before current child.
+    if (accBeforeUnattached == child) {
+      var list =
+        this.generateChildren(gEvent.accessible, aHighlightList, parent, true);
+      if (list) {
+        nodesToExpand = list.concat(nodesToExpand || []);
+      }
+    }
+
     var list =
-      this.generateChildren(aAccessible.getChildAt(i), aHighlightList, parent);
+      this.generateChildren(child, aHighlightList, parent);
+    if (list) {
+      nodesToExpand = list.concat(nodesToExpand || []);
+    }
+  }
+
+  // Put unattached child as last child of the parent, we don't have good guess
+  // about its hierarchy position.
+  if (containsUnattached && !accBeforeUnattached) {
+    var list =
+      this.generateChildren(gEvent.accessible, aHighlightList, parent, true);
     if (list) {
       nodesToExpand = list.concat(nodesToExpand || []);
     }
