@@ -13,18 +13,27 @@
 
 enum FallbackScripts {
     kArabic_FallbackScript,
+    kArmenian_FallbackScript,
+    kBengali_FallbackScript,
+    kDevanagari_FallbackScript,
     kEthiopic_FallbackScript,
+    kGeorgian_FallbackScript,
     kHebrewRegular_FallbackScript,
     kHebrewBold_FallbackScript,
+    kKannada_FallbackScript,
+    kMalayalam_FallbackScript,
+    kTamilRegular_FallbackScript,
+    kTamilBold_FallbackScript,
     kThai_FallbackScript,
-    kArmenian_FallbackScript,
-    kGeorgian_FallbackScript,
-    kDevanagari_FallbackScript,
-    kBengali_FallbackScript,
-    kTamil_FallbackScript,
+    kTelugu_FallbackScript,
     kFallbackScriptNumber
 };
 
+// This particular mapping will be removed after WebKit is updated to use the
+// new mappings. No new caller should use the kTamil_FallbackScript but rather
+// the more specific Tamil scripts in the standard enum.
+#define kTamil_FallbackScript kTamilRegular_FallbackScript
+
 #define SkTypeface_ValidScript(s) (s >= 0 && s < kFallbackScriptNumber)
 
 /**
@@ -49,4 +58,11 @@ SK_API const char* SkGetFallbackScriptID(FallbackScripts script);
  */
 SK_API FallbackScripts SkGetFallbackScriptFromID(const char* id);
 
+/**
+ *  For test only.
+ *  Load font config from given xml files, instead of those from Android system.
+ */
+SK_API void SkUseTestFontConfigFile(const char* mainconf, const char* fallbackconf,
+                                    const char* fontsdir);
+
 #endif
