@@ -5,22 +5,22 @@
 
 
 /**
-   File Name:          15.9.5.19.js
-   ECMA Section:       15.9.5.19
-   Description:        Date.prototype.getUTCSeconds
+   File Name:          15.9.5.17.js
+   ECMA Section:       15.9.5.17
+   Description:        Date.prototype.getUTCMinutes
 
    1.  Let t be this time value.
    2.  If t is NaN, return NaN.
-   3.  Return SecFromTime(t).
+   3.  Return MinFromTime(t).
 
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
 
-var SECTION = "15.9.5.19";
+var SECTION = "15.9.5.17";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Date.prototype.getUTCSeconds()";
+var TITLE   = "Date.prototype.getUTCMinutes()";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
@@ -33,22 +33,22 @@ addTestCase( UTC_FEB_29_2000 );
 addTestCase( UTC_JAN_1_2005 );
 
 new TestCase( SECTION,
-	      "(new Date(NaN)).getUTCSeconds()",
+	      "(new Date(NaN)).getUTCMinutes()",
 	      NaN,
-	      (new Date(NaN)).getUTCSeconds() );
+	      (new Date(NaN)).getUTCMinutes() );
 
 new TestCase( SECTION,
-	      "Date.prototype.getUTCSeconds.length",
+	      "Date.prototype.getUTCMinutes.length",
 	      0,
-	      Date.prototype.getUTCSeconds.length );
+	      Date.prototype.getUTCMinutes.length );
 test();
 
 function addTestCase( t ) {
   for ( m = 0; m <= 60; m+=10 ) {
-    t += 1000;
+    t += msPerMinute;
     new TestCase( SECTION,
-		  "(new Date("+t+")).getUTCSeconds()",
-		  SecFromTime(t),
-		  (new Date(t)).getUTCSeconds() );
+		  "(new Date("+t+")).getUTCMinutes()",
+		  MinFromTime(t),
+		  (new Date(t)).getUTCMinutes() );
   }
 }
