@@ -4,11 +4,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 387951;
-var summary = 'Do not assert: cg->stackDepth >= 0';
+var BUGNUMBER = 360969;
+var summary = '2^17: local const';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
+var global = this;
 
 //-----------------------------------------------------------------------------
 test();
@@ -20,9 +21,21 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  switch(delete[null?0:{}]){default:}
+  var start = new Date();
+  var p;
+  var i;
+  var limit = 2 << 16;
+
+  for (var i = 0; i < limit; i++)
+  {
+    eval('const pv' + i + ';');
+  }
 
   reportCompare(expect, actual, summary);
 
+  var stop = new Date();
+
+  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
+
   exitFunc ('test');
 }
