@@ -1,3 +1,4 @@
+// |reftest| skip-if(Android)
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,13 +21,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #63 From Brendan Eich
+// ------- Comment #135 From Gary Kwong [:nth10sd]
 
-  function f(that) {
-    for (ix in this)
-      print(ix);
-    for (let ix in that)
-      ;
+  if (typeof timeout == 'function')
+  {
+    expectExitCode(6);
+    timeout(3);
+    eval("do ([]); while(y for each (x in []))");
   }
 
   reportCompare(expect, actual, summary);
