@@ -4,20 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.4/11.4.5/11.4.5-2-2-s.js
- * @description Strict Mode - SyntaxError is thrown for --arguments
+ * @path ch11/11.13/11.13.2/11.13.2-7-s.js
+ * @description Strict Mode - ReferenceError is thrown if the LeftHandSideExpression of a Compound Assignment operator(>>=) evaluates to an unresolvable reference
  * @onlyStrict
  */
 
 
 function testcase() {
         "use strict";
-        var blah = arguments;
         try {
-            eval("--arguments;");
+            eval("_11_13_2_7 >>= 1;");
             return false;
         } catch (e) {
-            return e instanceof SyntaxError && blah === arguments;
+            return e instanceof ReferenceError;
         }
     }
 runTestCase(testcase);
