@@ -8,7 +8,7 @@
 
 #include "xpcprivate.h"
 
-#include "jshash.h"
+#include "js/HashTable.h"
 
 /***************************************************************************/
 // static shared...
@@ -16,31 +16,31 @@
 // Note this is returning the bit pattern of the first part of the nsID, not
 // the pointer to the nsID.
 
-static JSDHashNumber
-HashIIDPtrKey(JSDHashTable *table, const void *key)
+static PLDHashNumber
+HashIIDPtrKey(PLDHashTable *table, const void *key)
 {
-    return *((JSHashNumber*)key);
+    return *((js::HashNumber*)key);
 }
 
-static JSBool
-MatchIIDPtrKey(JSDHashTable *table,
-               const JSDHashEntryHdr *entry,
+static bool
+MatchIIDPtrKey(PLDHashTable *table,
+               const PLDHashEntryHdr *entry,
                const void *key)
 {
     return ((const nsID*)key)->
-                Equals(*((const nsID*)((JSDHashEntryStub*)entry)->key));
+                Equals(*((const nsID*)((PLDHashEntryStub*)entry)->key));
 }
 
-static JSDHashNumber
-HashNativeKey(JSDHashTable *table, const void *key)
+static PLDHashNumber
+HashNativeKey(PLDHashTable *table, const void *key)
 {
     XPCNativeSetKey* Key = (XPCNativeSetKey*) key;
 
-    JSDHashNumber h = 0;
+    PLDHashNumber h = 0;
 
     XPCNativeSet*       Set;
     XPCNativeInterface* Addition;
-    PRUint16            Position;
+    uint16_t            Position;
 
     if (Key->IsAKey()) {
         Set      = Key->GetBaseSet();
@@ -48,7 +48,7 @@ HashNativeKey(JSDHashTable *table, const void *key)
         Position = Key->GetPosition();
     } else {
         Set      = (XPCNativeSet*) Key;
-        Addition = nsnull;
+        Addition = nullptr;
         Position = 0;
     }
 
@@ -56,21 +56,21 @@ HashNativeKey(JSDHashTable *table, const void *key)
         NS_ASSERTION(Addition, "bad key");
         // This would be an XOR like below.
         // But "0 ^ x == x". So it does not matter.
-        h = (JSHashNumber) NS_PTR_TO_INT32(Addition) >> 2;
+        h = (js::HashNumber) NS_PTR_TO_INT32(Addition) >> 2;
     } else {
         XPCNativeInterface** Current = Set->GetInterfaceArray();
-        PRUint16 count = Set->GetInterfaceCount();
+        uint16_t count = Set->GetInterfaceCount();
         if (Addition) {
             count++;
-            for (PRUint16 i = 0; i < count; i++) {
+            for (uint16_t i = 0; i < count; i++) {
                 if (i == Position)
-                    h ^= (JSHashNumber) NS_PTR_TO_INT32(Addition) >> 2;
+                    h ^= (js::HashNumber) NS_PTR_TO_INT32(Addition) >> 2;
                 else
-                    h ^= (JSHashNumber) NS_PTR_TO_INT32(*(Current++)) >> 2;
+                    h ^= (js::HashNumber) NS_PTR_TO_INT32(*(Current++)) >> 2;
             }
         } else {
-            for (PRUint16 i = 0; i < count; i++)
-                h ^= (JSHashNumber) NS_PTR_TO_INT32(*(Current++)) >> 2;
+            for (uint16_t i = 0; i < count; i++)
+                h ^= (js::HashNumber) NS_PTR_TO_INT32(*(Current++)) >> 2;
         }
     }
 
@@ -80,43 +80,31 @@ HashNativeKey(JSDHashTable *table, const void *key)
 /***************************************************************************/
 // implement JSObject2WrappedJSMap...
 
-// static
-JSObject2WrappedJSMap*
-JSObject2WrappedJSMap::newMap(int size)
+void
+JSObject2WrappedJSMap::FindDyingJSObjects(nsTArray<nsXPCWrappedJS*>* dying)
 {
-    JSObject2WrappedJSMap* map = new JSObject2WrappedJSMap(size);
-    if (map && map->mTable)
-        return map;
-    delete map;
-    return nsnull;
+    for (Map::Range r = mTable.all(); !r.empty(); r.popFront()) {
+        nsXPCWrappedJS* wrapper = r.front().value;
+        NS_ASSERTION(wrapper, "found a null JS wrapper!");
+
+        // walk the wrapper chain and find any whose JSObject is to be finalized
+        while (wrapper) {
+            if (wrapper->IsSubjectToFinalization() && wrapper->IsObjectAboutToBeFinalized())
+                dying->AppendElement(wrapper);
+            wrapper = wrapper->GetNextWrapper();
+        }
+    }
 }
 
-JSObject2WrappedJSMap::JSObject2WrappedJSMap(int size)
+void
+JSObject2WrappedJSMap::ShutdownMarker(JSRuntime* rt)
 {
-    mTable = JS_NewDHashTable(JS_DHashGetStubOps(), nsnull,
-                              sizeof(Entry), size);
-}
-
-JSObject2WrappedJSMap::~JSObject2WrappedJSMap()
-{
-    if (mTable)
-        JS_DHashTableDestroy(mTable);
-}
-
-size_t
-JSObject2WrappedJSMap::SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf)
-{
-    size_t n = 0;
-    n += mallocSizeOf(this);
-    n += mTable ? JS_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
-    return n;
-}
-
-/* static */ size_t
-JSObject2WrappedJSMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr,
-                                                JSMallocSizeOfFun mallocSizeOf, void *)
-{
-    return mallocSizeOf(((JSObject2WrappedJSMap::Entry*)hdr)->value);
+    for (Map::Range r = mTable.all(); !r.empty(); r.popFront()) {
+        nsXPCWrappedJS* wrapper = r.front().value;
+        NS_ASSERTION(wrapper, "found a null JS wrapper!");
+        NS_ASSERTION(wrapper->IsValid(), "found an invalid JS wrapper!");
+        wrapper->SystemIsBeingShutDown(rt);
+    }
 }
 
 /***************************************************************************/
@@ -134,19 +122,19 @@ Native2WrappedNativeMap::newMap(int size)
     // use the map, so we abort immediately to provide a more useful
     // crash stack.
     NS_RUNTIMEABORT("Ran out of memory.");
-    return nsnull;
+    return nullptr;
 }
 
 Native2WrappedNativeMap::Native2WrappedNativeMap(int size)
 {
-    mTable = JS_NewDHashTable(JS_DHashGetStubOps(), nsnull,
+    mTable = PL_NewDHashTable(PL_DHashGetStubOps(), nullptr,
                               sizeof(Entry), size);
 }
 
 Native2WrappedNativeMap::~Native2WrappedNativeMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 size_t
@@ -154,12 +142,12 @@ Native2WrappedNativeMap::SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf)
 {
     size_t n = 0;
     n += mallocSizeOf(this);
-    n += mTable ? JS_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
+    n += mTable ? PL_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
     return n;
 }
 
 /* static */ size_t
-Native2WrappedNativeMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr,
+Native2WrappedNativeMap::SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr,
                                                   JSMallocSizeOfFun mallocSizeOf, void *)
 {
     return mallocSizeOf(((Native2WrappedNativeMap::Entry*)hdr)->value);
@@ -168,15 +156,15 @@ Native2WrappedNativeMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr,
 /***************************************************************************/
 // implement IID2WrappedJSClassMap...
 
-struct JSDHashTableOps IID2WrappedJSClassMap::Entry::sOps =
+struct PLDHashTableOps IID2WrappedJSClassMap::Entry::sOps =
 {
-    JS_DHashAllocTable,
-    JS_DHashFreeTable,
+    PL_DHashAllocTable,
+    PL_DHashFreeTable,
     HashIIDPtrKey,
     MatchIIDPtrKey,
-    JS_DHashMoveEntryStub,
-    JS_DHashClearEntryStub,
-    JS_DHashFinalizeStub
+    PL_DHashMoveEntryStub,
+    PL_DHashClearEntryStub,
+    PL_DHashFinalizeStub
 };
 
 // static
@@ -187,33 +175,33 @@ IID2WrappedJSClassMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 IID2WrappedJSClassMap::IID2WrappedJSClassMap(int size)
 {
-    mTable = JS_NewDHashTable(&Entry::sOps, nsnull, sizeof(Entry), size);
+    mTable = PL_NewDHashTable(&Entry::sOps, nullptr, sizeof(Entry), size);
 }
 
 IID2WrappedJSClassMap::~IID2WrappedJSClassMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 
 /***************************************************************************/
 // implement IID2NativeInterfaceMap...
 
-struct JSDHashTableOps IID2NativeInterfaceMap::Entry::sOps =
+struct PLDHashTableOps IID2NativeInterfaceMap::Entry::sOps =
 {
-    JS_DHashAllocTable,
-    JS_DHashFreeTable,
+    PL_DHashAllocTable,
+    PL_DHashFreeTable,
     HashIIDPtrKey,
     MatchIIDPtrKey,
-    JS_DHashMoveEntryStub,
-    JS_DHashClearEntryStub,
-    JS_DHashFinalizeStub
+    PL_DHashMoveEntryStub,
+    PL_DHashClearEntryStub,
+    PL_DHashFinalizeStub
 };
 
 // static
@@ -224,18 +212,18 @@ IID2NativeInterfaceMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 IID2NativeInterfaceMap::IID2NativeInterfaceMap(int size)
 {
-    mTable = JS_NewDHashTable(&Entry::sOps, nsnull, sizeof(Entry), size);
+    mTable = PL_NewDHashTable(&Entry::sOps, nullptr, sizeof(Entry), size);
 }
 
 IID2NativeInterfaceMap::~IID2NativeInterfaceMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 size_t
@@ -243,12 +231,12 @@ IID2NativeInterfaceMap::SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf)
 {
     size_t n = 0;
     n += mallocSizeOf(this);
-    n += mTable ? JS_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
+    n += mTable ? PL_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
     return n;
 }
 
 /* static */ size_t
-IID2NativeInterfaceMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr,
+IID2NativeInterfaceMap::SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr,
                                                  JSMallocSizeOfFun mallocSizeOf, void *)
 {
     XPCNativeInterface *iface = ((IID2NativeInterfaceMap::Entry*)hdr)->value;
@@ -266,19 +254,19 @@ ClassInfo2NativeSetMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 ClassInfo2NativeSetMap::ClassInfo2NativeSetMap(int size)
 {
-    mTable = JS_NewDHashTable(JS_DHashGetStubOps(), nsnull,
+    mTable = PL_NewDHashTable(PL_DHashGetStubOps(), nullptr,
                               sizeof(Entry), size);
 }
 
 ClassInfo2NativeSetMap::~ClassInfo2NativeSetMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 size_t
@@ -286,8 +274,8 @@ ClassInfo2NativeSetMap::ShallowSizeOfIncludingThis(nsMallocSizeOfFun mallocSizeO
 {
     size_t n = 0;
     n += mallocSizeOf(this);
-    // The second arg is NULL because this is a "shallow" measurement of the map.
-    n += mTable ? JS_DHashTableSizeOfIncludingThis(mTable, NULL, mallocSizeOf) : 0;
+    // The second arg is nullptr because this is a "shallow" measurement of the map.
+    n += mTable ? PL_DHashTableSizeOfIncludingThis(mTable, nullptr, mallocSizeOf) : 0;
     return n;
 }
 
@@ -306,19 +294,19 @@ ClassInfo2WrappedNativeProtoMap::newMap(int size)
     // use the map, so we abort immediately to provide a more useful
     // crash stack.
     NS_RUNTIMEABORT("Ran out of memory.");
-    return nsnull;
+    return nullptr;
 }
 
 ClassInfo2WrappedNativeProtoMap::ClassInfo2WrappedNativeProtoMap(int size)
 {
-    mTable = JS_NewDHashTable(JS_DHashGetStubOps(), nsnull,
+    mTable = PL_NewDHashTable(PL_DHashGetStubOps(), nullptr,
                               sizeof(Entry), size);
 }
 
 ClassInfo2WrappedNativeProtoMap::~ClassInfo2WrappedNativeProtoMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 size_t
@@ -326,12 +314,12 @@ ClassInfo2WrappedNativeProtoMap::SizeOfIncludingThis(nsMallocSizeOfFun mallocSiz
 {
     size_t n = 0;
     n += mallocSizeOf(this);
-    n += mTable ? JS_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
+    n += mTable ? PL_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
     return n;
 }
 
 /* static */ size_t
-ClassInfo2WrappedNativeProtoMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr,
+ClassInfo2WrappedNativeProtoMap::SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr,
                                                           JSMallocSizeOfFun mallocSizeOf, void *)
 {
     return mallocSizeOf(((ClassInfo2WrappedNativeProtoMap::Entry*)hdr)->value);
@@ -340,9 +328,9 @@ ClassInfo2WrappedNativeProtoMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr,
 /***************************************************************************/
 // implement NativeSetMap...
 
-JSBool
-NativeSetMap::Entry::Match(JSDHashTable *table,
-                           const JSDHashEntryHdr *entry,
+bool
+NativeSetMap::Entry::Match(PLDHashTable *table,
+                           const PLDHashEntryHdr *entry,
                            const void *key)
 {
     XPCNativeSetKey* Key = (XPCNativeSetKey*) key;
@@ -355,13 +343,13 @@ NativeSetMap::Entry::Match(JSDHashTable *table,
         if (Set1 == Set2)
             return true;
 
-        PRUint16 count = Set1->GetInterfaceCount();
+        uint16_t count = Set1->GetInterfaceCount();
         if (count != Set2->GetInterfaceCount())
             return false;
 
         XPCNativeInterface** Current1 = Set1->GetInterfaceArray();
         XPCNativeInterface** Current2 = Set2->GetInterfaceArray();
-        for (PRUint16 i = 0; i < count; i++) {
+        for (uint16_t i = 0; i < count; i++) {
             if (*(Current1++) != *(Current2++))
                 return false;
         }
@@ -392,14 +380,14 @@ NativeSetMap::Entry::Match(JSDHashTable *table,
     if (!Addition && Set == SetInTable)
         return true;
 
-    PRUint16 count = Set->GetInterfaceCount() + (Addition ? 1 : 0);
+    uint16_t count = Set->GetInterfaceCount() + (Addition ? 1 : 0);
     if (count != SetInTable->GetInterfaceCount())
         return false;
 
-    PRUint16 Position = Key->GetPosition();
+    uint16_t Position = Key->GetPosition();
     XPCNativeInterface** CurrentInTable = SetInTable->GetInterfaceArray();
     XPCNativeInterface** Current = Set->GetInterfaceArray();
-    for (PRUint16 i = 0; i < count; i++) {
+    for (uint16_t i = 0; i < count; i++) {
         if (Addition && i == Position) {
             if (Addition != *(CurrentInTable++))
                 return false;
@@ -412,15 +400,15 @@ NativeSetMap::Entry::Match(JSDHashTable *table,
     return true;
 }
 
-struct JSDHashTableOps NativeSetMap::Entry::sOps =
+struct PLDHashTableOps NativeSetMap::Entry::sOps =
 {
-    JS_DHashAllocTable,
-    JS_DHashFreeTable,
+    PL_DHashAllocTable,
+    PL_DHashFreeTable,
     HashNativeKey,
     Match,
-    JS_DHashMoveEntryStub,
-    JS_DHashClearEntryStub,
-    JS_DHashFinalizeStub
+    PL_DHashMoveEntryStub,
+    PL_DHashClearEntryStub,
+    PL_DHashFinalizeStub
 };
 
 // static
@@ -431,18 +419,18 @@ NativeSetMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 NativeSetMap::NativeSetMap(int size)
 {
-    mTable = JS_NewDHashTable(&Entry::sOps, nsnull, sizeof(Entry), size);
+    mTable = PL_NewDHashTable(&Entry::sOps, nullptr, sizeof(Entry), size);
 }
 
 NativeSetMap::~NativeSetMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 size_t
@@ -450,12 +438,12 @@ NativeSetMap::SizeOfIncludingThis(nsMallocSizeOfFun mallocSizeOf)
 {
     size_t n = 0;
     n += mallocSizeOf(this);
-    n += mTable ? JS_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
+    n += mTable ? PL_DHashTableSizeOfIncludingThis(mTable, SizeOfEntryExcludingThis, mallocSizeOf) : 0;
     return n;
 }
 
 /* static */ size_t
-NativeSetMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *)
+NativeSetMap::SizeOfEntryExcludingThis(PLDHashEntryHdr *hdr, JSMallocSizeOfFun mallocSizeOf, void *)
 {
     XPCNativeSet *set = ((NativeSetMap::Entry*)hdr)->key_value;
     return set->SizeOfIncludingThis(mallocSizeOf);
@@ -464,30 +452,30 @@ NativeSetMap::SizeOfEntryExcludingThis(JSDHashEntryHdr *hdr, JSMallocSizeOfFun m
 /***************************************************************************/
 // implement IID2ThisTranslatorMap...
 
-JSBool
-IID2ThisTranslatorMap::Entry::Match(JSDHashTable *table,
-                                    const JSDHashEntryHdr *entry,
+bool
+IID2ThisTranslatorMap::Entry::Match(PLDHashTable *table,
+                                    const PLDHashEntryHdr *entry,
                                     const void *key)
 {
     return ((const nsID*)key)->Equals(((Entry*)entry)->key);
 }
 
 void
-IID2ThisTranslatorMap::Entry::Clear(JSDHashTable *table, JSDHashEntryHdr *entry)
+IID2ThisTranslatorMap::Entry::Clear(PLDHashTable *table, PLDHashEntryHdr *entry)
 {
     NS_IF_RELEASE(((Entry*)entry)->value);
     memset(entry, 0, table->entrySize);
 }
 
-struct JSDHashTableOps IID2ThisTranslatorMap::Entry::sOps =
+struct PLDHashTableOps IID2ThisTranslatorMap::Entry::sOps =
 {
-    JS_DHashAllocTable,
-    JS_DHashFreeTable,
+    PL_DHashAllocTable,
+    PL_DHashFreeTable,
     HashIIDPtrKey,
     Match,
-    JS_DHashMoveEntryStub,
+    PL_DHashMoveEntryStub,
     Clear,
-    JS_DHashFinalizeStub
+    PL_DHashFinalizeStub
 };
 
 // static
@@ -498,26 +486,26 @@ IID2ThisTranslatorMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 IID2ThisTranslatorMap::IID2ThisTranslatorMap(int size)
 {
-    mTable = JS_NewDHashTable(&Entry::sOps, nsnull, sizeof(Entry), size);
+    mTable = PL_NewDHashTable(&Entry::sOps, nullptr, sizeof(Entry), size);
 }
 
 IID2ThisTranslatorMap::~IID2ThisTranslatorMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 /***************************************************************************/
 
-JSDHashNumber
-XPCNativeScriptableSharedMap::Entry::Hash(JSDHashTable *table, const void *key)
+PLDHashNumber
+XPCNativeScriptableSharedMap::Entry::Hash(PLDHashTable *table, const void *key)
 {
-    JSDHashNumber h;
+    PLDHashNumber h;
     const unsigned char *s;
 
     XPCNativeScriptableShared* obj =
@@ -527,15 +515,15 @@ XPCNativeScriptableSharedMap::Entry::Hash(JSDHashTable *table, const void *key)
     // bitmap since it's very rare that it's different when flags and classname
     // are the same.
 
-    h = (JSDHashNumber) obj->GetFlags();
+    h = (PLDHashNumber) obj->GetFlags();
     for (s = (const unsigned char*) obj->GetJSClass()->name; *s != '\0'; s++)
         h = JS_ROTATE_LEFT32(h, 4) ^ *s;
     return h;
 }
 
-JSBool
-XPCNativeScriptableSharedMap::Entry::Match(JSDHashTable *table,
-                                           const JSDHashEntryHdr *entry,
+bool
+XPCNativeScriptableSharedMap::Entry::Match(PLDHashTable *table,
+                                           const PLDHashEntryHdr *entry,
                                            const void *key)
 {
     XPCNativeScriptableShared* obj1 =
@@ -559,15 +547,15 @@ XPCNativeScriptableSharedMap::Entry::Match(JSDHashTable *table,
     return 0 == strcmp(name1, name2);
 }
 
-struct JSDHashTableOps XPCNativeScriptableSharedMap::Entry::sOps =
+struct PLDHashTableOps XPCNativeScriptableSharedMap::Entry::sOps =
 {
-    JS_DHashAllocTable,
-    JS_DHashFreeTable,
+    PL_DHashAllocTable,
+    PL_DHashFreeTable,
     Hash,
     Match,
-    JS_DHashMoveEntryStub,
-    JS_DHashClearEntryStub,
-    JS_DHashFinalizeStub
+    PL_DHashMoveEntryStub,
+    PL_DHashClearEntryStub,
+    PL_DHashFinalizeStub
 };
 
 // static
@@ -579,24 +567,24 @@ XPCNativeScriptableSharedMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 XPCNativeScriptableSharedMap::XPCNativeScriptableSharedMap(int size)
 {
-    mTable = JS_NewDHashTable(&Entry::sOps, nsnull, sizeof(Entry), size);
+    mTable = PL_NewDHashTable(&Entry::sOps, nullptr, sizeof(Entry), size);
 }
 
 XPCNativeScriptableSharedMap::~XPCNativeScriptableSharedMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 JSBool
 XPCNativeScriptableSharedMap::GetNewOrUsed(uint32_t flags,
                                            char* name,
-                                           PRUint32 interfacesBitmap,
+                                           uint32_t interfacesBitmap,
                                            XPCNativeScriptableInfo* si)
 {
     NS_PRECONDITION(name,"bad param");
@@ -604,7 +592,7 @@ XPCNativeScriptableSharedMap::GetNewOrUsed(uint32_t flags,
 
     XPCNativeScriptableShared key(flags, name, interfacesBitmap);
     Entry* entry = (Entry*)
-        JS_DHashTableOperate(mTable, &key, JS_DHASH_ADD);
+        PL_DHashTableOperate(mTable, &key, PL_DHASH_ADD);
     if (!entry)
         return false;
 
@@ -633,19 +621,19 @@ XPCWrappedNativeProtoMap::newMap(int size)
     if (map && map->mTable)
         return map;
     delete map;
-    return nsnull;
+    return nullptr;
 }
 
 XPCWrappedNativeProtoMap::XPCWrappedNativeProtoMap(int size)
 {
-    mTable = JS_NewDHashTable(JS_DHashGetStubOps(), nsnull,
-                              sizeof(JSDHashEntryStub), size);
+    mTable = PL_NewDHashTable(PL_DHashGetStubOps(), nullptr,
+                              sizeof(PLDHashEntryStub), size);
 }
 
 XPCWrappedNativeProtoMap::~XPCWrappedNativeProtoMap()
 {
     if (mTable)
-        JS_DHashTableDestroy(mTable);
+        PL_DHashTableDestroy(mTable);
 }
 
 /***************************************************************************/
