@@ -3,8 +3,12 @@
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-#include "nsNSSComponent.h"
+
 #include "nsNSSCallbacks.h"
+
+#include "mozilla/Telemetry.h"
+#include "mozilla/TimeStamp.h"
+#include "nsNSSComponent.h"
 #include "nsNSSIOLayer.h"
 #include "nsIWebProgressListener.h"
 #include "nsProtectedAuthThread.h"
@@ -15,11 +19,15 @@
 #include "nsIPrompt.h"
 #include "nsProxyRelease.h"
 #include "PSMRunnable.h"
+#include "ScopedNSSTypes.h"
 #include "nsIConsoleService.h"
 #include "nsIHttpChannelInternal.h"
 #include "nsCRT.h"
+#include "nsNetUtil.h"
+#include "SharedSSLState.h"
 
 #include "ssl.h"
+#include "sslproto.h"
 #include "ocsp.h"
 #include "nssb64.h"
 
@@ -43,6 +51,7 @@ public:
   
   nsCOMPtr<nsHTTPListener> mListener;
   bool mResponsibleForDoneSignal;
+  TimeStamp mStartTime;
 };
 
 nsHTTPDownloadEvent::nsHTTPDownloadEvent()
@@ -70,12 +79,10 @@ nsHTTPDownloadEvent::Run()
   NS_ENSURE_STATE(ios);
 
   nsCOMPtr<nsIChannel> chan;
-  ios->NewChannel(mRequestSession->mURL, nsnull, nsnull, getter_AddRefs(chan));
+  ios->NewChannel(mRequestSession->mURL, nullptr, nullptr, getter_AddRefs(chan));
   NS_ENSURE_STATE(chan);
 
-  // Disabled because it breaks authentication with a proxy, when such proxy
-  // had been setup, and brings blue UI for EV certs.
-  // chan->SetLoadFlags(nsIRequest::LOAD_ANONYMOUS);
+  chan->SetLoadFlags(nsIRequest::LOAD_ANONYMOUS);
 
   // Create a loadgroup for this new channel.  This way if the channel
   // is redirected, we'll have a way to cancel the resulting channel.
@@ -87,8 +94,7 @@ nsHTTPDownloadEvent::Run()
     nsCOMPtr<nsIInputStream> uploadStream;
     rv = NS_NewPostDataStream(getter_AddRefs(uploadStream),
                               false,
-                              mRequestSession->mPostData,
-                              0, ios);
+                              mRequestSession->mPostData);
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIUploadChannel> uploadChannel(do_QueryInterface(chan));
@@ -126,16 +132,18 @@ nsHTTPDownloadEvent::Run()
   rv = NS_NewStreamLoader(getter_AddRefs(mListener->mLoader), 
                           mListener);
 
-  if (NS_SUCCEEDED(rv))
-    rv = hchan->AsyncOpen(mListener->mLoader, nsnull);
+  if (NS_SUCCEEDED(rv)) {
+    mStartTime = TimeStamp::Now();
+    rv = hchan->AsyncOpen(mListener->mLoader, nullptr);
+  }
 
   if (NS_FAILED(rv)) {
     mListener->mResponsibleForDoneSignal = false;
     mResponsibleForDoneSignal = true;
 
     NS_RELEASE(mListener->mLoadGroup);
-    mListener->mLoadGroup = nsnull;
-    mListener->mLoadGroupOwnerThread = nsnull;
+    mListener->mLoadGroup = nullptr;
+    mListener->mLoadGroupOwnerThread = nullptr;
   }
 
   return NS_OK;
@@ -146,13 +154,13 @@ struct nsCancelHTTPDownloadEvent : nsRunnable {
 
   NS_IMETHOD Run() {
     mListener->FreeLoadGroup(true);
-    mListener = nsnull;
+    mListener = nullptr;
     return NS_OK;
   }
 };
 
 SECStatus nsNSSHttpServerSession::createSessionFcn(const char *host,
-                                                   PRUint16 portnum,
+                                                   uint16_t portnum,
                                                    SEC_HTTP_SERVER_SESSION *pSession)
 {
   if (!host || !pSession)
@@ -192,7 +200,7 @@ SECStatus nsNSSHttpRequestSession::createFcn(SEC_HTTP_SERVER_SESSION session,
 
   // Use a maximum timeout value of 10 seconds because of bug 404059.
   // FIXME: Use a better approach once 406120 is ready.
-  PRUint32 maxBug404059Timeout = PR_TicksPerSecond() * 10;
+  uint32_t maxBug404059Timeout = PR_TicksPerSecond() * 10;
   if (timeout > maxBug404059Timeout) {
     rs->mTimeoutInterval = maxBug404059Timeout;
   }
@@ -211,7 +219,7 @@ SECStatus nsNSSHttpRequestSession::createFcn(SEC_HTTP_SERVER_SESSION session,
 }
 
 SECStatus nsNSSHttpRequestSession::setPostDataFcn(const char *http_data, 
-                                                  const PRUint32 http_data_len,
+                                                  const uint32_t http_data_len,
                                                   const char *http_content_type)
 {
   mHasPostData = true;
@@ -238,11 +246,11 @@ SECStatus nsNSSHttpRequestSession::addHeaderFcn(const char *http_header_name,
 }
 
 SECStatus nsNSSHttpRequestSession::trySendAndReceiveFcn(PRPollDesc **pPollDesc,
-                                                        PRUint16 *http_response_code, 
+                                                        uint16_t *http_response_code, 
                                                         const char **http_response_content_type, 
                                                         const char **http_response_headers, 
                                                         const char **http_response_data, 
-                                                        PRUint32 *http_response_data_len)
+                                                        uint32_t *http_response_data_len)
 {
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
          ("nsNSSHttpRequestSession::trySendAndReceiveFcn to %s\n", mURL.get()));
@@ -302,7 +310,7 @@ nsNSSHttpRequestSession::AddRef()
 void
 nsNSSHttpRequestSession::Release()
 {
-  PRInt32 newRefCount = NS_AtomicDecrementRefcnt(mRefCount);
+  int32_t newRefCount = NS_AtomicDecrementRefcnt(mRefCount);
   if (!newRefCount) {
     delete this;
   }
@@ -311,19 +319,19 @@ nsNSSHttpRequestSession::Release()
 SECStatus
 nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
                                                        PRPollDesc **pPollDesc,
-                                                       PRUint16 *http_response_code,
+                                                       uint16_t *http_response_code,
                                                        const char **http_response_content_type,
                                                        const char **http_response_headers,
                                                        const char **http_response_data,
-                                                       PRUint32 *http_response_data_len)
+                                                       uint32_t *http_response_data_len)
 {
-  if (pPollDesc) *pPollDesc = nsnull;
+  if (pPollDesc) *pPollDesc = nullptr;
   if (http_response_code) *http_response_code = 0;
   if (http_response_content_type) *http_response_content_type = 0;
   if (http_response_headers) *http_response_headers = 0;
   if (http_response_data) *http_response_data = 0;
 
-  PRUint32 acceptableResultSize = 0;
+  uint32_t acceptableResultSize = 0;
 
   if (http_response_data_len)
   {
@@ -339,7 +347,7 @@ nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
   volatile bool &waitFlag = mListener->mWaitFlag;
   waitFlag = true;
 
-  nsRefPtr<nsHTTPDownloadEvent> event = new nsHTTPDownloadEvent;
+  RefPtr<nsHTTPDownloadEvent> event(new nsHTTPDownloadEvent);
   if (!event)
     return SECFailure;
 
@@ -365,6 +373,13 @@ nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
     bool running_on_main_thread = NS_IsMainThread();
     if (running_on_main_thread)
     {
+      // The result of running this on the main thread
+      // is a series of small timeouts mixed with spinning the
+      // event loop - this is always dangerous as there is so much main
+      // thread code that does not expect to be called re-entrantly. Your
+      // app really shouldn't do that.
+      NS_WARNING("Security network blocking I/O on Main Thread");
+
       // let's process events quickly
       wait_interval = PR_MicrosecondsToInterval(50);
     }
@@ -386,7 +401,7 @@ nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
         // made me aware of this possibility. (kaie)
 
         MutexAutoUnlock unlock(waitLock);
-        NS_ProcessNextEvent(nsnull);
+        NS_ProcessNextEvent(nullptr);
       }
 
       waitCondition.Wait(wait_interval);
@@ -403,7 +418,8 @@ nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
         {
           request_canceled = true;
 
-          nsRefPtr<nsCancelHTTPDownloadEvent> cancelevent = new nsCancelHTTPDownloadEvent;
+          RefPtr<nsCancelHTTPDownloadEvent> cancelevent(
+            new nsCancelHTTPDownloadEvent);
           cancelevent->mListener = mListener;
           rv = NS_DispatchToMainThread(cancelevent);
           if (NS_FAILED(rv)) {
@@ -415,6 +431,31 @@ nsNSSHttpRequestSession::internal_send_receive_attempt(bool &retryable_error,
     }
   }
 
+  if (!event->mStartTime.IsNull()) {
+    if (request_canceled) {
+      Telemetry::Accumulate(Telemetry::CERT_VALIDATION_HTTP_REQUEST_RESULT, 0);
+      Telemetry::AccumulateTimeDelta(
+        Telemetry::CERT_VALIDATION_HTTP_REQUEST_CANCELED_TIME,
+        event->mStartTime, TimeStamp::Now());
+    }
+    else if (NS_SUCCEEDED(mListener->mResultCode) &&
+             mListener->mHttpResponseCode == 200) {
+      Telemetry::Accumulate(Telemetry::CERT_VALIDATION_HTTP_REQUEST_RESULT, 1);
+      Telemetry::AccumulateTimeDelta(
+        Telemetry::CERT_VALIDATION_HTTP_REQUEST_SUCCEEDED_TIME,
+        event->mStartTime, TimeStamp::Now());
+    }
+    else {
+      Telemetry::Accumulate(Telemetry::CERT_VALIDATION_HTTP_REQUEST_RESULT, 2);
+      Telemetry::AccumulateTimeDelta(
+        Telemetry::CERT_VALIDATION_HTTP_REQUEST_FAILED_TIME,
+        event->mStartTime, TimeStamp::Now());
+    }
+  }
+  else {
+    Telemetry::Accumulate(Telemetry::CERT_VALIDATION_HTTP_REQUEST_RESULT, 3);
+  }
+
   if (request_canceled)
     return SECFailure;
 
@@ -508,18 +549,18 @@ void nsNSSHttpInterface::registerHttpClient()
 
 void nsNSSHttpInterface::unregisterHttpClient()
 {
-  SEC_RegisterDefaultHttpClient(nsnull);
+  SEC_RegisterDefaultHttpClient(nullptr);
 }
 
 nsHTTPListener::nsHTTPListener()
-: mResultData(nsnull),
+: mResultData(nullptr),
   mResultLen(0),
   mLock("nsHTTPListener.mLock"),
   mCondition(mLock, "nsHTTPListener.mCondition"),
   mWaitFlag(true),
   mResponsibleForDoneSignal(false),
-  mLoadGroup(nsnull),
-  mLoadGroupOwnerThread(nsnull)
+  mLoadGroup(nullptr),
+  mLoadGroupOwnerThread(nullptr)
 {
 }
 
@@ -539,7 +580,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsHTTPListener, nsIStreamLoaderObserver)
 void
 nsHTTPListener::FreeLoadGroup(bool aCancelLoad)
 {
-  nsILoadGroup *lg = nsnull;
+  nsILoadGroup *lg = nullptr;
 
   MutexAutoLock locker(mLock);
 
@@ -550,7 +591,7 @@ nsHTTPListener::FreeLoadGroup(bool aCancelLoad)
     }
     else {
       lg = mLoadGroup;
-      mLoadGroup = nsnull;
+      mLoadGroup = nullptr;
     }
   }
 
@@ -566,8 +607,8 @@ NS_IMETHODIMP
 nsHTTPListener::OnStreamComplete(nsIStreamLoader* aLoader,
                                  nsISupports* aContext,
                                  nsresult aStatus,
-                                 PRUint32 stringLen,
-                                 const PRUint8* string)
+                                 uint32_t stringLen,
+                                 const uint8_t* string)
 {
   mResultCode = aStatus;
 
@@ -631,10 +672,10 @@ ShowProtectedAuthPrompt(PK11SlotInfo* slot, nsIInterfaceRequestor *ir)
 {
   if (!NS_IsMainThread()) {
     NS_ERROR("ShowProtectedAuthPrompt called off the main thread");
-    return nsnull;
+    return nullptr;
   }
 
-  char* protAuthRetVal = nsnull;
+  char* protAuthRetVal = nullptr;
 
   // Get protected auth dialogs
   nsITokenDialogs* dialogs = 0;
@@ -671,7 +712,7 @@ ShowProtectedAuthPrompt(PK11SlotInfo* slot, nsIInterfaceRequestor *ir)
                   protAuthRetVal = ToNewCString(nsDependentCString(PK11_PW_RETRY));
                   break;
               default:
-                  protAuthRetVal = nsnull;
+                  protAuthRetVal = nullptr;
                   break;
               
           }
@@ -692,7 +733,7 @@ class PK11PasswordPromptRunnable : public SyncRunnableBase
 public:
   PK11PasswordPromptRunnable(PK11SlotInfo* slot, 
                              nsIInterfaceRequestor* ir)
-    : mResult(nsnull),
+    : mResult(nullptr),
       mSlot(slot),
       mIR(ir)
   {
@@ -708,14 +749,14 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
 {
   nsNSSShutDownPreventionLock locker;
   nsresult rv = NS_OK;
-  PRUnichar *password = nsnull;
+  PRUnichar *password = nullptr;
   bool value = false;
   nsCOMPtr<nsIPrompt> prompt;
 
   /* TODO: Retry should generate a different dialog message */
 /*
   if (retry)
-    return nsnull;
+    return nullptr;
 */
 
   if (!mIR)
@@ -725,7 +766,7 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
   else
   {
     prompt = do_GetInterface(mIR);
-    NS_ASSERTION(prompt != nsnull, "callbacks does not implement nsIPrompt");
+    NS_ASSERTION(prompt, "callbacks does not implement nsIPrompt");
   }
 
   if (!prompt)
@@ -762,8 +803,8 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
       // Although the exact value is ignored, we must not pass invalid
       // bool values through XPConnect.
       bool checkState = false;
-      rv = prompt->PromptPassword(nsnull, promptString.get(),
-                                  &password, nsnull, &checkState, &value);
+      rv = prompt->PromptPassword(nullptr, promptString.get(),
+                                  &password, nullptr, &checkState, &value);
     }
   }
   
@@ -773,56 +814,54 @@ void PK11PasswordPromptRunnable::RunOnTargetThread()
   }
 }
 
-char* PR_CALLBACK
+char*
 PK11PasswordPrompt(PK11SlotInfo* slot, PRBool retry, void* arg)
 {
-  nsRefPtr<PK11PasswordPromptRunnable> runnable = 
+  RefPtr<PK11PasswordPromptRunnable> runnable(
     new PK11PasswordPromptRunnable(slot,
-                                   static_cast<nsIInterfaceRequestor*>(arg));
+                                   static_cast<nsIInterfaceRequestor*>(arg)));
   runnable->DispatchToMainThreadAndWait();
   return runnable->mResult;
 }
 
-void PR_CALLBACK HandshakeCallback(PRFileDesc* fd, void* client_data) {
+void HandshakeCallback(PRFileDesc* fd, void* client_data) {
   nsNSSShutDownPreventionLock locker;
-  PRInt32 sslStatus;
-  char* signer = nsnull;
-  char* cipherName = nsnull;
-  PRInt32 keyLength;
-  nsresult rv;
-  PRInt32 encryptBits;
+  int32_t sslStatus;
+  char* cipherName = nullptr;
+  int32_t keyLength;
+  int32_t encryptBits;
 
   nsNSSSocketInfo* infoObject = (nsNSSSocketInfo*) fd->higher->secret;
 
-  if (infoObject) {
-    // This is the first callback on resumption handshakes
-    infoObject->SetFirstServerHelloReceived();
-  }
+  // certificate validation sets FirstServerHelloReceived, so if that flag
+  // is absent at handshake time we have a resumed session.
+  bool isResumedSession = !(infoObject->GetFirstServerHelloReceived());
+
+  // This is the first callback on resumption handshakes
+  infoObject->SetFirstServerHelloReceived();
 
   // If the handshake completed, then we know the site is TLS tolerant (if this
   // was a TLS connection).
-  nsSSLIOLayerHelpers::rememberTolerantSite(infoObject);
+  nsSSLIOLayerHelpers& ioLayerHelpers = infoObject->SharedState().IOLayerHelpers();
+  ioLayerHelpers.rememberTolerantSite(infoObject);
 
   if (SECSuccess != SSL_SecurityStatus(fd, &sslStatus, &cipherName, &keyLength,
-                                       &encryptBits, &signer, nsnull)) {
+                                       &encryptBits, nullptr, nullptr)) {
     return;
   }
 
-  PRInt32 secStatus;
+  int32_t secStatus;
   if (sslStatus == SSL_SECURITY_STATUS_OFF)
     secStatus = nsIWebProgressListener::STATE_IS_BROKEN;
-  else if (encryptBits >= 90)
-    secStatus = (nsIWebProgressListener::STATE_IS_SECURE |
-                 nsIWebProgressListener::STATE_SECURE_HIGH);
   else
-    secStatus = (nsIWebProgressListener::STATE_IS_SECURE |
-                 nsIWebProgressListener::STATE_SECURE_LOW);
+    secStatus = nsIWebProgressListener::STATE_IS_SECURE
+              | nsIWebProgressListener::STATE_SECURE_HIGH;
 
   PRBool siteSupportsSafeRenego;
   if (SSL_HandshakeNegotiatedExtension(fd, ssl_renegotiation_info_xtn, &siteSupportsSafeRenego) != SECSuccess
       || !siteSupportsSafeRenego) {
 
-    bool wantWarning = (nsSSLIOLayerHelpers::getWarnLevelMissingRFC5746() > 0);
+    bool wantWarning = (ioLayerHelpers.getWarnLevelMissingRFC5746() > 0);
 
     nsCOMPtr<nsIConsoleService> console;
     if (infoObject && wantWarning) {
@@ -838,108 +877,93 @@ void PR_CALLBACK HandshakeCallback(PRFileDesc* fd, void* client_data) {
         console->LogStringMessage(msg.get());
       }
     }
-    if (nsSSLIOLayerHelpers::treatUnsafeNegotiationAsBroken()) {
+    if (ioLayerHelpers.treatUnsafeNegotiationAsBroken()) {
       secStatus = nsIWebProgressListener::STATE_IS_BROKEN;
     }
   }
 
+  ScopedCERTCertificate serverCert(SSL_PeerCertificate(fd));
 
-  CERTCertificate *peerCert = SSL_PeerCertificate(fd);
-  const char* caName = nsnull; // caName is a pointer only, no ownership
-  char* certOrgName = CERT_GetOrgName(&peerCert->issuer);
-  CERT_DestroyCertificate(peerCert);
-  caName = certOrgName ? certOrgName : signer;
+  infoObject->SetSecurityState(secStatus);
 
-  const char* verisignName = "Verisign, Inc.";
-  // If the CA name is RSA Data Security, then change the name to the real
-  // name of the company i.e. VeriSign, Inc.
-  if (nsCRT::strcmp((const char*)caName, "RSA Data Security, Inc.") == 0) {
-    caName = verisignName;
+  /* Set the SSL Status information */
+  RefPtr<nsSSLStatus> status(infoObject->SSLStatus());
+  if (!status) {
+    status = new nsSSLStatus();
+    infoObject->SetSSLStatus(status);
   }
 
-  nsAutoString shortDesc;
-  const PRUnichar* formatStrings[1] = { ToNewUnicode(NS_ConvertUTF8toUTF16(caName)) };
-  nsCOMPtr<nsINSSComponent> nssComponent(do_GetService(kNSSComponentCID, &rv));
-  if (NS_SUCCEEDED(rv)) {
-    rv = nssComponent->PIPBundleFormatStringFromName("SignedBy",
-                                                   formatStrings, 1,
-                                                   shortDesc);
+  RememberCertErrorsTable::GetInstance().LookupCertErrorBits(infoObject,
+                                                             status);
 
-    nsMemory::Free(const_cast<PRUnichar*>(formatStrings[0]));
+  RefPtr<nsNSSCertificate> nssc(nsNSSCertificate::Create(serverCert));
+  nsCOMPtr<nsIX509Cert> prevcert;
+  infoObject->GetPreviousCert(getter_AddRefs(prevcert));
 
-    nsNSSSocketInfo* infoObject = (nsNSSSocketInfo*) fd->higher->secret;
-    infoObject->SetSecurityState(secStatus);
-    infoObject->SetShortSecurityDescription(shortDesc.get());
-
-    /* Set the SSL Status information */
-    nsRefPtr<nsSSLStatus> status = infoObject->SSLStatus();
-    if (!status) {
-      status = new nsSSLStatus();
-      infoObject->SetSSLStatus(status);
+  bool equals_previous = false;
+  if (prevcert && nssc) {
+    nsresult rv = nssc->Equals(prevcert, &equals_previous);
+    if (NS_FAILED(rv)) {
+      equals_previous = false;
     }
+  }
 
-    RememberCertErrorsTable::GetInstance().LookupCertErrorBits(infoObject,
-                                                               status);
-
-    CERTCertificate *serverCert = SSL_PeerCertificate(fd);
-    if (serverCert) {
-      nsRefPtr<nsNSSCertificate> nssc = nsNSSCertificate::Create(serverCert);
-      CERT_DestroyCertificate(serverCert);
-      serverCert = nsnull;
-
-      nsCOMPtr<nsIX509Cert> prevcert;
-      infoObject->GetPreviousCert(getter_AddRefs(prevcert));
-
-      bool equals_previous = false;
-      if (prevcert && nssc) {
-        nsresult rv = nssc->Equals(prevcert, &equals_previous);
-        if (NS_FAILED(rv)) {
-          equals_previous = false;
-        }
-      }
-
-      if (equals_previous) {
-        PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
-               ("HandshakeCallback using PREV cert %p\n", prevcert.get()));
-        status->mServerCert = prevcert;
-      }
-      else {
-        if (status->mServerCert) {
-          PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
-                 ("HandshakeCallback KEEPING cert %p\n", status->mServerCert.get()));
-        }
-        else {
-          PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
-                 ("HandshakeCallback using NEW cert %p\n", nssc.get()));
-          status->mServerCert = nssc;
-        }
-      }
+  if (equals_previous) {
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
+            ("HandshakeCallback using PREV cert %p\n", prevcert.get()));
+    status->mServerCert = prevcert;
+  }
+  else {
+    if (status->mServerCert) {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
+              ("HandshakeCallback KEEPING cert %p\n", status->mServerCert.get()));
     }
+    else {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG,
+              ("HandshakeCallback using NEW cert %p\n", nssc.get()));
+      status->mServerCert = nssc;
+    }
+  }
 
-    status->mHaveKeyLengthAndCipher = true;
-    status->mKeyLength = keyLength;
-    status->mSecretKeyLength = encryptBits;
-    status->mCipherName.Assign(cipherName);
+  status->mHaveKeyLengthAndCipher = true;
+  status->mKeyLength = keyLength;
+  status->mSecretKeyLength = encryptBits;
+  status->mCipherName.Assign(cipherName);
 
-    // Get the NPN value. Do this on the stack and copy it into
-    // a string rather than preallocating the string because right
-    // now we expect NPN to fail more often than it succeeds.
-    SSLNextProtoState state;
-    unsigned char npnbuf[256];
-    unsigned int npnlen;
+  // Get the NPN value.
+  SSLNextProtoState state;
+  unsigned char npnbuf[256];
+  unsigned int npnlen;
     
-    if (SSL_GetNextProto(fd, &state, npnbuf, &npnlen, 256) == SECSuccess &&
-        state == SSL_NEXT_PROTO_NEGOTIATED)
+  if (SSL_GetNextProto(fd, &state, npnbuf, &npnlen, 256) == SECSuccess) {
+    if (state == SSL_NEXT_PROTO_NEGOTIATED)
       infoObject->SetNegotiatedNPN(reinterpret_cast<char *>(npnbuf), npnlen);
     else
-      infoObject->SetNegotiatedNPN(nsnull, 0);
-
-    infoObject->SetHandshakeCompleted();
+      infoObject->SetNegotiatedNPN(nullptr, 0);
+    mozilla::Telemetry::Accumulate(Telemetry::SSL_NPN_TYPE, state);
   }
+  else
+    infoObject->SetNegotiatedNPN(nullptr, 0);
+
+  SSLChannelInfo channelInfo;
+  if (SSL_GetChannelInfo(fd, &channelInfo, sizeof(channelInfo)) == SECSuccess) {
+    // Get the protocol version for telemetry
+    // 0=ssl3, 1=tls1, 2=tls1.1, 3=tls1.2
+    unsigned int versionEnum = channelInfo.protocolVersion & 0xFF;
+    Telemetry::Accumulate(Telemetry::SSL_HANDSHAKE_VERSION, versionEnum);
+
+    SSLCipherSuiteInfo cipherInfo;
+    if (SSL_GetCipherSuiteInfo(channelInfo.cipherSuite, &cipherInfo,
+                                sizeof (cipherInfo)) == SECSuccess) {
+      // keyExchange null=0, rsa=1, dh=2, fortezza=3, ecdh=4
+      Telemetry::Accumulate(Telemetry::SSL_KEY_EXCHANGE_ALGORITHM,
+                            cipherInfo.keaType);
+    }
+      
+  }
+  infoObject->SetHandshakeCompleted(isResumedSession);
 
   PORT_Free(cipherName);
-  PR_FREEIF(certOrgName);
-  PR_Free(signer);
 }
 
 struct OCSPDefaultResponders {
@@ -954,65 +978,65 @@ static struct OCSPDefaultResponders myDefaultOCSPResponders[] = {
   /* COMODO */
   {
     "CN=AddTrust External CA Root,OU=AddTrust External TTP Network,O=AddTrust AB,C=SE",
-    nsnull, "rb2YejS0Jvf6xCZU7wO94CTLVBo=", nsnull,
+    nullptr, "rb2YejS0Jvf6xCZU7wO94CTLVBo=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=COMODO Certification Authority,O=COMODO CA Limited,L=Salford,ST=Greater Manchester,C=GB",
-    nsnull, "C1jli8ZMFTekQKkwqSG+RzZaVv8=", nsnull,
+    nullptr, "C1jli8ZMFTekQKkwqSG+RzZaVv8=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=COMODO EV SGC CA,O=COMODO CA Limited,L=Salford,ST=Greater Manchester,C=GB",
-    nsnull, "f/ZMNigUrs0eN6/eWvJbw6CsK/4=", nsnull,
+    nullptr, "f/ZMNigUrs0eN6/eWvJbw6CsK/4=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=COMODO EV SSL CA,O=COMODO CA Limited,L=Salford,ST=Greater Manchester,C=GB",
-    nsnull, "aRZJ7LZ1ZFrpAyNgL1RipTRcPuI=", nsnull,
+    nullptr, "aRZJ7LZ1ZFrpAyNgL1RipTRcPuI=", nullptr,
     "http://ocsp.comodoca.com"
   },
   {
     "CN=UTN - DATACorp SGC,OU=http://www.usertrust.com,O=The USERTRUST Network,L=Salt Lake City,ST=UT,C=US",
-    nsnull, "UzLRs89/+uDxoF2FTpLSnkUdtE8=", nsnull,
+    nullptr, "UzLRs89/+uDxoF2FTpLSnkUdtE8=", nullptr,
     "http://ocsp.usertrust.com"
   },
   {
     "CN=UTN-USERFirst-Hardware,OU=http://www.usertrust.com,O=The USERTRUST Network,L=Salt Lake City,ST=UT,C=US",
-    nsnull, "oXJfJhsomEOVXQc31YWWnUvSw0U=", nsnull,
+    nullptr, "oXJfJhsomEOVXQc31YWWnUvSw0U=", nullptr,
     "http://ocsp.usertrust.com"
   },
   /* Network Solutions */
   {
     "CN=Network Solutions Certificate Authority,O=Network Solutions L.L.C.,C=US",
-    nsnull, "ITDJ+wDXTpjah6oq0KcusUAxp0w=", nsnull,
+    nullptr, "ITDJ+wDXTpjah6oq0KcusUAxp0w=", nullptr,
     "http://ocsp.netsolssl.com"
   },
   {
     "CN=Network Solutions EV SSL CA,O=Network Solutions L.L.C.,C=US",
-    nsnull, "tk6FnYQfGx3UUolOB5Yt+d7xj8w=", nsnull,
+    nullptr, "tk6FnYQfGx3UUolOB5Yt+d7xj8w=", nullptr,
     "http://ocsp.netsolssl.com"
   },
   /* GlobalSign */
   {
     "CN=GlobalSign Root CA,OU=Root CA,O=GlobalSign nv-sa,C=BE",
-    nsnull, "YHtmGkUNl8qJUC99BM00qP/8/Us=", nsnull,
+    nullptr, "YHtmGkUNl8qJUC99BM00qP/8/Us=", nullptr,
     "http://ocsp.globalsign.com/ExtendedSSLCACross"
   },
   {
     "CN=GlobalSign,O=GlobalSign,OU=GlobalSign Root CA - R2",
-    nsnull, "m+IHV2ccHsBqBt5ZtJot39wZhi4=", nsnull,
+    nullptr, "m+IHV2ccHsBqBt5ZtJot39wZhi4=", nullptr,
     "http://ocsp.globalsign.com/ExtendedSSLCA"
   },
   {
     "CN=GlobalSign Extended Validation CA,O=GlobalSign,OU=Extended Validation CA",
-    nsnull, "NLH5yYxrNUTMCGkK7uOjuVy/FuA=", nsnull,
+    nullptr, "NLH5yYxrNUTMCGkK7uOjuVy/FuA=", nullptr,
     "http://ocsp.globalsign.com/ExtendedSSL"
   },
   /* Trustwave */
   {
     "CN=SecureTrust CA,O=SecureTrust Corporation,C=US",
-    nsnull, "QjK2FvoE/f5dS3rD/fdMQB1aQ68=", nsnull,
+    nullptr, "QjK2FvoE/f5dS3rD/fdMQB1aQ68=", nullptr,
     "http://ocsp.trustwave.com"
   }
 };
@@ -1020,7 +1044,7 @@ static struct OCSPDefaultResponders myDefaultOCSPResponders[] = {
 static const unsigned int numResponders =
     (sizeof myDefaultOCSPResponders) / (sizeof myDefaultOCSPResponders[0]);
 
-static CERT_StringFromCertFcn oldOCSPAIAInfoCallback = nsnull;
+static CERT_StringFromCertFcn oldOCSPAIAInfoCallback = nullptr;
 
 /*
  * See if we have a hard-coded default responder for this certificate's
@@ -1028,7 +1052,7 @@ static CERT_StringFromCertFcn oldOCSPAIAInfoCallback = nsnull;
  *
  * The result needs to be freed (PORT_Free) when no longer in use.
  */
-char* PR_CALLBACK MyAlternateOCSPAIAInfoCallback(CERTCertificate *cert) {
+char* MyAlternateOCSPAIAInfoCallback(CERTCertificate *cert) {
   if (cert && !cert->isRoot) {
     unsigned int i;
     for (i=0; i < numResponders; i++) {
@@ -1049,7 +1073,7 @@ char* PR_CALLBACK MyAlternateOCSPAIAInfoCallback(CERTCertificate *cert) {
   if (oldOCSPAIAInfoCallback)
     return (*oldOCSPAIAInfoCallback)(cert);
 
-  return nsnull;
+  return nullptr;
 }
 
 void cleanUpMyDefaultOCSPResponders() {
@@ -1058,11 +1082,11 @@ void cleanUpMyDefaultOCSPResponders() {
   for (i=0; i < numResponders; i++) {
     if (myDefaultOCSPResponders[i].issuerName) {
       CERT_DestroyName(myDefaultOCSPResponders[i].issuerName);
-      myDefaultOCSPResponders[i].issuerName = nsnull;
+      myDefaultOCSPResponders[i].issuerName = nullptr;
     }
     if (myDefaultOCSPResponders[i].issuerKeyID) {
       SECITEM_FreeItem(myDefaultOCSPResponders[i].issuerKeyID, true);
-      myDefaultOCSPResponders[i].issuerKeyID = nsnull;
+      myDefaultOCSPResponders[i].issuerKeyID = nullptr;
     }
   }
 }
@@ -1082,9 +1106,9 @@ SECStatus RegisterMyOCSPAIAInfoCallback() {
     if (!(myDefaultOCSPResponders[i].issuerName))
       goto loser;
     // Create a SECItem from the Base64 authority key identifier keyID.
-    myDefaultOCSPResponders[i].issuerKeyID = NSSBase64_DecodeBuffer(nsnull,
-          nsnull, myDefaultOCSPResponders[i].issuerKeyID_base64,
-          (PRUint32)PORT_Strlen(myDefaultOCSPResponders[i].issuerKeyID_base64));
+    myDefaultOCSPResponders[i].issuerKeyID = NSSBase64_DecodeBuffer(nullptr,
+          nullptr, myDefaultOCSPResponders[i].issuerKeyID_base64,
+          (uint32_t)PORT_Strlen(myDefaultOCSPResponders[i].issuerKeyID_base64));
     if (!(myDefaultOCSPResponders[i].issuerKeyID))
       goto loser;
   }
@@ -1111,12 +1135,12 @@ SECStatus UnregisterMyOCSPAIAInfoCallback() {
 
   // Unregister our alternate OCSP Responder URL lookup function.
   rv = CERT_RegisterAlternateOCSPAIAInfoCallBack(oldOCSPAIAInfoCallback,
-                                                 nsnull);
+                                                 nullptr);
   if (rv != SECSuccess)
     return rv;
 
   // Tidy up.
-  oldOCSPAIAInfoCallback = nsnull;
+  oldOCSPAIAInfoCallback = nullptr;
   cleanUpMyDefaultOCSPResponders();
   return SECSuccess;
 }
