@@ -13,7 +13,6 @@
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
-#include "prmem.h"
 #include "nsPrintfCString.h"
 #include "nsIDateTimeFormat.h"
 #include "nsDateTimeFormatCID.h"
@@ -25,6 +24,7 @@
 #include "nsAutoPtr.h"
 #include "nsIMutableArray.h"
 #include "nsTArray.h"
+#include "mozilla/Attributes.h"
 
 #include "nsWildCard.h"
 
@@ -34,7 +34,7 @@ class nsIDOMDataTransfer;
                             { 0x91, 0x10, 0x81, 0x46, 0x61, 0x4c, 0xa7, 0xf0 } }
 #define NS_FILECOMPLETE_CONTRACTID "@mozilla.org/autocomplete/search;1?name=file"
 
-class nsFileResult : public nsIAutoCompleteResult
+class nsFileResult MOZ_FINAL : public nsIAutoCompleteResult
 {
 public:
   // aSearchString is the text typed into the autocomplete widget
@@ -46,7 +46,7 @@ public:
 
   nsTArray<nsString> mValues;
   nsAutoString mSearchString;
-  PRUint16 mSearchResult;
+  uint16_t mSearchResult;
 };
 
 NS_IMPL_ISUPPORTS1(nsFileResult, nsIAutoCompleteResult)
@@ -58,7 +58,7 @@ nsFileResult::nsFileResult(const nsAString& aSearchString,
   if (aSearchString.IsEmpty())
     mSearchResult = RESULT_IGNORED;
   else {
-    PRInt32 slashPos = mSearchString.RFindChar('/');
+    int32_t slashPos = mSearchString.RFindChar('/');
     mSearchResult = RESULT_FAILURE;
     nsCOMPtr<nsIFile> directory;
     nsDependentSubstring parent(Substring(mSearchString, 0, slashPos + 1));
@@ -101,14 +101,14 @@ NS_IMETHODIMP nsFileResult::GetSearchString(nsAString & aSearchString)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetSearchResult(PRUint16 *aSearchResult)
+NS_IMETHODIMP nsFileResult::GetSearchResult(uint16_t *aSearchResult)
 {
   NS_ENSURE_ARG_POINTER(aSearchResult);
   *aSearchResult = mSearchResult;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetDefaultIndex(PRInt32 *aDefaultIndex)
+NS_IMETHODIMP nsFileResult::GetDefaultIndex(int32_t *aDefaultIndex)
 {
   NS_ENSURE_ARG_POINTER(aDefaultIndex);
   *aDefaultIndex = -1;
@@ -121,7 +121,7 @@ NS_IMETHODIMP nsFileResult::GetErrorDescription(nsAString & aErrorDescription)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetMatchCount(PRUint32 *aMatchCount)
+NS_IMETHODIMP nsFileResult::GetMatchCount(uint32_t *aMatchCount)
 {
   NS_ENSURE_ARG_POINTER(aMatchCount);
   *aMatchCount = mValues.Length();
@@ -135,41 +135,41 @@ NS_IMETHODIMP nsFileResult::GetTypeAheadResult(bool *aTypeAheadResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetValueAt(PRInt32 index, nsAString & aValue)
+NS_IMETHODIMP nsFileResult::GetValueAt(int32_t index, nsAString & aValue)
 {
   aValue = mValues[index];
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetLabelAt(PRInt32 index, nsAString & aValue)
+NS_IMETHODIMP nsFileResult::GetLabelAt(int32_t index, nsAString & aValue)
 {
   return GetValueAt(index, aValue);
 }
 
-NS_IMETHODIMP nsFileResult::GetCommentAt(PRInt32 index, nsAString & aComment)
+NS_IMETHODIMP nsFileResult::GetCommentAt(int32_t index, nsAString & aComment)
 {
   aComment.Truncate();
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetStyleAt(PRInt32 index, nsAString & aStyle)
+NS_IMETHODIMP nsFileResult::GetStyleAt(int32_t index, nsAString & aStyle)
 {
   aStyle.Truncate();
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::GetImageAt(PRInt32 index, nsAString & aImage)
+NS_IMETHODIMP nsFileResult::GetImageAt(int32_t index, nsAString & aImage)
 {
   aImage.Truncate();
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFileResult::RemoveValueAt(PRInt32 rowIndex, bool removeFromDb)
+NS_IMETHODIMP nsFileResult::RemoveValueAt(int32_t rowIndex, bool removeFromDb)
 {
   return NS_OK;
 }
 
-class nsFileComplete : public nsIAutoCompleteSearch
+class nsFileComplete MOZ_FINAL : public nsIAutoCompleteSearch
 {
 public:
   NS_DECL_ISUPPORTS
@@ -214,23 +214,21 @@ protected:
   virtual ~nsFileView();
   
   void FilterFiles();
-  void ReverseArray(nsISupportsArray* aArray);
-  void SortArray(nsISupportsArray* aArray);
+  void ReverseArray(nsTArray<nsCOMPtr<nsIFile> >& aArray);
+  void SortArray(nsTArray<nsCOMPtr<nsIFile> >& aArray);
   void SortInternal();
 
-  nsCOMPtr<nsISupportsArray> mFileList;
-  nsCOMPtr<nsISupportsArray> mDirList;
-  nsCOMPtr<nsISupportsArray> mFilteredFiles;
+  nsTArray<nsCOMPtr<nsIFile> > mFileList;
+  nsTArray<nsCOMPtr<nsIFile> > mDirList;
+  nsTArray<nsCOMPtr<nsIFile> > mFilteredFiles;
 
   nsCOMPtr<nsIFile> mDirectoryPath;
   nsCOMPtr<nsITreeBoxObject> mTree;
   nsCOMPtr<nsITreeSelection> mSelection;
-  nsCOMPtr<nsIAtom> mDirectoryAtom;
-  nsCOMPtr<nsIAtom> mFileAtom;
   nsCOMPtr<nsIDateTimeFormat> mDateFormatter;
 
-  PRInt16 mSortType;
-  PRInt32 mTotalRows;
+  int16_t mSortType;
+  int32_t mTotalRows;
 
   nsTArray<PRUnichar*> mCurrentFilters;
 
@@ -276,34 +274,14 @@ nsFileView::nsFileView() :
 
 nsFileView::~nsFileView()
 {
-  PRUint32 count = mCurrentFilters.Length();
-  for (PRUint32 i = 0; i < count; ++i)
+  uint32_t count = mCurrentFilters.Length();
+  for (uint32_t i = 0; i < count; ++i)
     NS_Free(mCurrentFilters[i]);
 }
 
 nsresult
 nsFileView::Init()
 {
-  mDirectoryAtom = do_GetAtom("directory");
-  if (!mDirectoryAtom)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  mFileAtom = do_GetAtom("file");
-  if (!mFileAtom)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_NewISupportsArray(getter_AddRefs(mFileList));
-  if (!mFileList)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_NewISupportsArray(getter_AddRefs(mDirList));
-  if (!mDirList)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_NewISupportsArray(getter_AddRefs(mFilteredFiles));
-  if (!mFilteredFiles)
-    return NS_ERROR_OUT_OF_MEMORY;
-
   mDateFormatter = do_CreateInstance(NS_DATETIMEFORMAT_CONTRACTID);
   if (!mDateFormatter)
     return NS_ERROR_OUT_OF_MEMORY;
@@ -345,12 +323,11 @@ nsFileView::SetShowOnlyDirectories(bool aOnlyDirs)
     return NS_OK;
 
   mDirectoryFilter = aOnlyDirs;
-  PRUint32 dirCount;
-  mDirList->Count(&dirCount);
+  uint32_t dirCount = mDirList.Length();
   if (mDirectoryFilter) {
-    PRInt32 rowDiff = mTotalRows - dirCount;
+    int32_t rowDiff = mTotalRows - dirCount;
 
-    mFilteredFiles->Clear();
+    mFilteredFiles.Clear();
     mTotalRows = dirCount;
     if (mTree)
       mTree->RowCountChanged(mTotalRows, -rowDiff);
@@ -377,7 +354,7 @@ nsFileView::GetShowOnlyDirectories(bool* aOnlyDirs)
 }
 
 NS_IMETHODIMP
-nsFileView::GetSortType(PRInt16* aSortType)
+nsFileView::GetSortType(int16_t* aSortType)
 {
   *aSortType = mSortType;
   return NS_OK;
@@ -391,7 +368,7 @@ nsFileView::GetReverseSort(bool* aReverseSort)
 }
 
 NS_IMETHODIMP
-nsFileView::Sort(PRInt16 aSortType, bool aReverseSort)
+nsFileView::Sort(int16_t aSortType, bool aReverseSort)
 {
   if (aSortType == mSortType) {
     if (aReverseSort == mReverseSort)
@@ -427,8 +404,8 @@ nsFileView::SetDirectory(nsIFile* aDirectory)
   }
 
   mDirectoryPath = aDirectory;
-  mFileList->Clear();
-  mDirList->Clear();
+  mFileList.Clear();
+  mDirList.Clear();
 
   bool hasMore = false;
 
@@ -445,11 +422,11 @@ nsFileView::SetDirectory(nsIFile* aDirectory)
         bool isHidden;
         theFile->IsHidden(&isHidden);
         if (mShowHiddenFiles || !isHidden) {
-          mDirList->AppendElement(theFile);
+          mDirList.AppendElement(theFile);
         }
       }
       else {
-        mFileList->AppendElement(theFile);
+        mFileList.AppendElement(theFile);
       }
     }
   }
@@ -473,8 +450,8 @@ nsFileView::SetDirectory(nsIFile* aDirectory)
 NS_IMETHODIMP
 nsFileView::SetFilter(const nsAString& aFilterString)
 {
-  PRUint32 filterCount = mCurrentFilters.Length();
-  for (PRUint32 i = 0; i < filterCount; ++i)
+  uint32_t filterCount = mCurrentFilters.Length();
+  for (uint32_t i = 0; i < filterCount; ++i)
     NS_Free(mCurrentFilters[i]);
   mCurrentFilters.Clear();
 
@@ -516,12 +493,11 @@ nsFileView::SetFilter(const nsAString& aFilterString)
 
   if (mTree) {
     mTree->BeginUpdateBatch();
-    PRUint32 count;
-    mDirList->Count(&count);
+    uint32_t count = mDirList.Length();
     mTree->RowCountChanged(count, count - mTotalRows);
   }
 
-  mFilteredFiles->Clear();
+  mFilteredFiles.Clear();
 
   FilterFiles();
 
@@ -538,32 +514,30 @@ nsFileView::SetFilter(const nsAString& aFilterString)
 NS_IMETHODIMP
 nsFileView::GetSelectedFiles(nsIArray** aFiles)
 {
-  *aFiles = nsnull;
+  *aFiles = nullptr;
   if (!mSelection)
     return NS_OK;
 
-  PRInt32 numRanges;
+  int32_t numRanges;
   mSelection->GetRangeCount(&numRanges);
 
-  PRUint32 dirCount;
-  mDirList->Count(&dirCount);
-
+  uint32_t dirCount = mDirList.Length();
   nsCOMPtr<nsIMutableArray> fileArray =
     do_CreateInstance(NS_ARRAY_CONTRACTID);
   NS_ENSURE_STATE(fileArray);
 
-  for (PRInt32 range = 0; range < numRanges; ++range) {
-    PRInt32 rangeBegin, rangeEnd;
+  for (int32_t range = 0; range < numRanges; ++range) {
+    int32_t rangeBegin, rangeEnd;
     mSelection->GetRangeAt(range, &rangeBegin, &rangeEnd);
 
-    for (PRInt32 itemIndex = rangeBegin; itemIndex <= rangeEnd; ++itemIndex) {
-      nsCOMPtr<nsIFile> curFile;
+    for (int32_t itemIndex = rangeBegin; itemIndex <= rangeEnd; ++itemIndex) {
+      nsIFile* curFile = nullptr;
 
-      if (itemIndex < (PRInt32) dirCount)
-        curFile = do_QueryElementAt(mDirList, itemIndex);
+      if (itemIndex < (int32_t) dirCount)
+        curFile = mDirList[itemIndex];
       else {
         if (itemIndex < mTotalRows)
-          curFile = do_QueryElementAt(mFilteredFiles, itemIndex - dirCount);
+          curFile = mFilteredFiles[itemIndex - dirCount];
       }
 
       if (curFile)
@@ -579,7 +553,7 @@ nsFileView::GetSelectedFiles(nsIArray** aFiles)
 // nsITreeView implementation
 
 NS_IMETHODIMP
-nsFileView::GetRowCount(PRInt32* aRowCount)
+nsFileView::GetRowCount(int32_t* aRowCount)
 {
   *aRowCount = mTotalRows;
   return NS_OK;
@@ -601,57 +575,54 @@ nsFileView::SetSelection(nsITreeSelection* aSelection)
 }
 
 NS_IMETHODIMP
-nsFileView::GetRowProperties(PRInt32 aIndex,
-                             nsISupportsArray* aProperties)
+nsFileView::GetRowProperties(int32_t aIndex, nsAString& aProps)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetCellProperties(PRInt32 aRow, nsITreeColumn* aCol,
-                              nsISupportsArray* aProperties)
+nsFileView::GetCellProperties(int32_t aRow, nsITreeColumn* aCol,
+                              nsAString& aProps)
 {
-  PRUint32 dirCount;
-  mDirList->Count(&dirCount);
+  uint32_t dirCount = mDirList.Length();
 
-  if (aRow < (PRInt32) dirCount)
-    aProperties->AppendElement(mDirectoryAtom);
+  if (aRow < (int32_t) dirCount)
+    aProps.AppendLiteral("directory");
   else if (aRow < mTotalRows)
-    aProperties->AppendElement(mFileAtom);
+    aProps.AppendLiteral("file");
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetColumnProperties(nsITreeColumn* aCol,
-                                nsISupportsArray* aProperties)
+nsFileView::GetColumnProperties(nsITreeColumn* aCol, nsAString& aProps)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::IsContainer(PRInt32 aIndex, bool* aIsContainer)
+nsFileView::IsContainer(int32_t aIndex, bool* aIsContainer)
 {
   *aIsContainer = false;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::IsContainerOpen(PRInt32 aIndex, bool* aIsOpen)
+nsFileView::IsContainerOpen(int32_t aIndex, bool* aIsOpen)
 {
   *aIsOpen = false;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::IsContainerEmpty(PRInt32 aIndex, bool* aIsEmpty)
+nsFileView::IsContainerEmpty(int32_t aIndex, bool* aIsEmpty)
 {
   *aIsEmpty = false;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::IsSeparator(PRInt32 aIndex, bool* aIsSeparator)
+nsFileView::IsSeparator(int32_t aIndex, bool* aIsSeparator)
 {
   *aIsSeparator = false;
   return NS_OK;
@@ -665,7 +636,7 @@ nsFileView::IsSorted(bool* aIsSorted)
 }
 
 NS_IMETHODIMP
-nsFileView::CanDrop(PRInt32 aIndex, PRInt32 aOrientation,
+nsFileView::CanDrop(int32_t aIndex, int32_t aOrientation,
                     nsIDOMDataTransfer* dataTransfer, bool* aCanDrop)
 {
   *aCanDrop = false;
@@ -673,20 +644,20 @@ nsFileView::CanDrop(PRInt32 aIndex, PRInt32 aOrientation,
 }
 
 NS_IMETHODIMP
-nsFileView::Drop(PRInt32 aRow, PRInt32 aOrientation, nsIDOMDataTransfer* dataTransfer)
+nsFileView::Drop(int32_t aRow, int32_t aOrientation, nsIDOMDataTransfer* dataTransfer)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetParentIndex(PRInt32 aRowIndex, PRInt32* aParentIndex)
+nsFileView::GetParentIndex(int32_t aRowIndex, int32_t* aParentIndex)
 {
   *aParentIndex = -1;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex, 
+nsFileView::HasNextSibling(int32_t aRowIndex, int32_t aAfterIndex, 
                            bool* aHasSibling)
 {
   *aHasSibling = (aAfterIndex < (mTotalRows - 1));
@@ -694,50 +665,47 @@ nsFileView::HasNextSibling(PRInt32 aRowIndex, PRInt32 aAfterIndex,
 }
 
 NS_IMETHODIMP
-nsFileView::GetLevel(PRInt32 aIndex, PRInt32* aLevel)
+nsFileView::GetLevel(int32_t aIndex, int32_t* aLevel)
 {
   *aLevel = 0;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetImageSrc(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::GetImageSrc(int32_t aRow, nsITreeColumn* aCol,
                         nsAString& aImageSrc)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetProgressMode(PRInt32 aRow, nsITreeColumn* aCol,
-                            PRInt32* aProgressMode)
+nsFileView::GetProgressMode(int32_t aRow, nsITreeColumn* aCol,
+                            int32_t* aProgressMode)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetCellValue(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::GetCellValue(int32_t aRow, nsITreeColumn* aCol,
                          nsAString& aCellValue)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::GetCellText(int32_t aRow, nsITreeColumn* aCol,
                         nsAString& aCellText)
 {
-  PRUint32 dirCount, fileCount;
-  mDirList->Count(&dirCount);
-  mFilteredFiles->Count(&fileCount);
-
+  uint32_t dirCount = mDirList.Length();
   bool isDirectory;
-  nsCOMPtr<nsIFile> curFile;
+  nsIFile* curFile = nullptr;
 
-  if (aRow < (PRInt32) dirCount) {
+  if (aRow < (int32_t) dirCount) {
     isDirectory = true;
-    curFile = do_QueryElementAt(mDirList, aRow);
+    curFile = mDirList[aRow];
   } else if (aRow < mTotalRows) {
     isDirectory = false;
-    curFile = do_QueryElementAt(mFilteredFiles, aRow - dirCount);
+    curFile = mFilteredFiles[aRow - dirCount];
   } else {
     // invalid row
     aCellText.SetCapacity(0);
@@ -749,11 +717,11 @@ nsFileView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol,
   if (NS_LITERAL_STRING("FilenameColumn").Equals(colID)) {
     curFile->GetLeafName(aCellText);
   } else if (NS_LITERAL_STRING("LastModifiedColumn").Equals(colID)) {
-    PRInt64 lastModTime;
+    PRTime lastModTime;
     curFile->GetLastModifiedTime(&lastModTime);
     // XXX FormatPRTime could take an nsAString&
     nsAutoString temp;
-    mDateFormatter->FormatPRTime(nsnull, kDateFormatShort, kTimeFormatSeconds,
+    mDateFormatter->FormatPRTime(nullptr, kDateFormatShort, kTimeFormatSeconds,
                                  lastModTime * 1000, temp);
     aCellText = temp;
   } else {
@@ -761,7 +729,7 @@ nsFileView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol,
     if (isDirectory)
       aCellText.SetCapacity(0);
     else {
-      PRInt64 fileSize;
+      int64_t fileSize;
       curFile->GetFileSize(&fileSize);
       CopyUTF8toUTF16(nsPrintfCString("%lld", fileSize), aCellText);
     }
@@ -778,7 +746,7 @@ nsFileView::SetTree(nsITreeBoxObject* aTree)
 }
 
 NS_IMETHODIMP
-nsFileView::ToggleOpenState(PRInt32 aIndex)
+nsFileView::ToggleOpenState(int32_t aIndex)
 {
   return NS_OK;
 }
@@ -796,13 +764,13 @@ nsFileView::SelectionChanged()
 }
 
 NS_IMETHODIMP
-nsFileView::CycleCell(PRInt32 aRow, nsITreeColumn* aCol)
+nsFileView::CycleCell(int32_t aRow, nsITreeColumn* aCol)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::IsEditable(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::IsEditable(int32_t aRow, nsITreeColumn* aCol,
                        bool* aIsEditable)
 {
   *aIsEditable = false;
@@ -810,7 +778,7 @@ nsFileView::IsEditable(PRInt32 aRow, nsITreeColumn* aCol,
 }
 
 NS_IMETHODIMP
-nsFileView::IsSelectable(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::IsSelectable(int32_t aRow, nsITreeColumn* aCol,
                          bool* aIsSelectable)
 {
   *aIsSelectable = false;
@@ -818,14 +786,14 @@ nsFileView::IsSelectable(PRInt32 aRow, nsITreeColumn* aCol,
 }
 
 NS_IMETHODIMP
-nsFileView::SetCellValue(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::SetCellValue(int32_t aRow, nsITreeColumn* aCol,
                          const nsAString& aValue)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::SetCellText(PRInt32 aRow, nsITreeColumn* aCol,
+nsFileView::SetCellText(int32_t aRow, nsITreeColumn* aCol,
                         const nsAString& aValue)
 {
   return NS_OK;
@@ -838,13 +806,13 @@ nsFileView::PerformAction(const PRUnichar* aAction)
 }
 
 NS_IMETHODIMP
-nsFileView::PerformActionOnRow(const PRUnichar* aAction, PRInt32 aRow)
+nsFileView::PerformActionOnRow(const PRUnichar* aAction, int32_t aRow)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::PerformActionOnCell(const PRUnichar* aAction, PRInt32 aRow,
+nsFileView::PerformActionOnCell(const PRUnichar* aAction, int32_t aRow,
                                 nsITreeColumn* aCol)
 {
   return NS_OK;
@@ -855,16 +823,14 @@ nsFileView::PerformActionOnCell(const PRUnichar* aAction, PRInt32 aRow,
 void
 nsFileView::FilterFiles()
 {
-  PRUint32 count = 0;
-  mDirList->Count(&count);
+  uint32_t count = mDirList.Length();
   mTotalRows = count;
-  mFileList->Count(&count);
-  mFilteredFiles->Clear();
-  PRUint32 filterCount = mCurrentFilters.Length();
+  count = mFileList.Length();
+  mFilteredFiles.Clear();
+  uint32_t filterCount = mCurrentFilters.Length();
 
-  nsCOMPtr<nsIFile> file;
-  for (PRUint32 i = 0; i < count; ++i) {
-    file = do_QueryElementAt(mFileList, i);
+  for (uint32_t i = 0; i < count; ++i) {
+    nsIFile* file = mFileList[i];
     bool isHidden = false;
     if (!mShowHiddenFiles)
       file->IsHidden(&isHidden);
@@ -876,7 +842,7 @@ nsFileView::FilterFiles()
     }
     
     if (!isHidden) {
-      for (PRUint32 j = 0; j < filterCount; ++j) {
+      for (uint32_t j = 0; j < filterCount; ++j) {
         bool matched = false;
         if (!nsCRT::strcmp(mCurrentFilters.ElementAt(j),
                            NS_LITERAL_STRING("..apps").get()))
@@ -888,7 +854,7 @@ nsFileView::FilterFiles()
                                       true) == MATCH);
 
         if (matched) {
-          mFilteredFiles->AppendElement(file);
+          mFilteredFiles.AppendElement(file);
           ++mTotalRows;
           break;
         }
@@ -898,15 +864,15 @@ nsFileView::FilterFiles()
 }
 
 void
-nsFileView::ReverseArray(nsISupportsArray* aArray)
+nsFileView::ReverseArray(nsTArray<nsCOMPtr<nsIFile> >& aArray)
 {
-  PRUint32 count;
-  aArray->Count(&count);
-  for (PRUint32 i = 0; i < count/2; ++i) {
-    nsCOMPtr<nsISupports> element = dont_AddRef(aArray->ElementAt(i));
-    nsCOMPtr<nsISupports> element2 = dont_AddRef(aArray->ElementAt(count-i-1));
-    aArray->ReplaceElementAt(element2, i);
-    aArray->ReplaceElementAt(element, count-i-1);
+  uint32_t count = aArray.Length();
+  for (uint32_t i = 0; i < count/2; ++i) {
+    // If we get references to the COMPtrs in the array, and then .swap() them
+    // we avoid AdRef() / Release() calls.
+    nsCOMPtr<nsIFile>& element = aArray[i];
+    nsCOMPtr<nsIFile>& element2 = aArray[count - i - 1];
+    element.swap(element2);
   }
 }
 
@@ -929,14 +895,14 @@ SortSizeCallback(const void* aElement1, const void* aElement2, void* aContext)
   nsIFile* file1 = *static_cast<nsIFile* const *>(aElement1);
   nsIFile* file2 = *static_cast<nsIFile* const *>(aElement2);
 
-  PRInt64 size1, size2;
+  int64_t size1, size2;
   file1->GetFileSize(&size1);
   file2->GetFileSize(&size2);
 
-  if (LL_EQ(size1, size2))
+  if (size1 == size2)
     return 0;
 
-  return (LL_CMP(size1, <, size2) ? -1 : 1);
+  return size1 < size2 ? -1 : 1;
 }
 
 static int
@@ -945,18 +911,18 @@ SortDateCallback(const void* aElement1, const void* aElement2, void* aContext)
   nsIFile* file1 = *static_cast<nsIFile* const *>(aElement1);
   nsIFile* file2 = *static_cast<nsIFile* const *>(aElement2);
 
-  PRInt64 time1, time2;
+  PRTime time1, time2;
   file1->GetLastModifiedTime(&time1);
   file2->GetLastModifiedTime(&time2);
 
-  if (LL_EQ(time1, time2))
+  if (time1 == time2)
     return 0;
 
-  return (LL_CMP(time1, <, time2) ? -1 : 1);
+  return time1 < time2 ? -1 : 1;
 }
 
 void
-nsFileView::SortArray(nsISupportsArray* aArray)
+nsFileView::SortArray(nsTArray<nsCOMPtr<nsIFile> >& aArray)
 {
   // We assume the array to be in filesystem order, which
   // for our purposes, is completely unordered.
@@ -977,21 +943,18 @@ nsFileView::SortArray(nsISupportsArray* aArray)
     return;
   }
 
-  PRUint32 count;
-  aArray->Count(&count);
+  uint32_t count = aArray.Length();
 
-  // each item will have an additional refcount while
-  // the array is alive.
   nsIFile** array = new nsIFile*[count];
-  PRUint32 i;
-  for (i = 0; i < count; ++i)
-    aArray->QueryElementAt(i, NS_GET_IID(nsIFile), (void**)&(array[i]));
+  for (uint32_t i = 0; i < count; ++i) {
+    array[i] = aArray[i];
+  }
 
-  NS_QuickSort(array, count, sizeof(nsIFile*), compareFunc, nsnull);
+  NS_QuickSort(array, count, sizeof(nsIFile*), compareFunc, nullptr);
 
-  for (i = 0; i < count; ++i) {
-    aArray->ReplaceElementAt(array[i], i);
-    NS_RELEASE(array[i]);
+  for (uint32_t i = 0; i < count; ++i) {
+    // Use swap() to avoid refcounting.
+    aArray[i].swap(array[i]);
   }
 
   delete[] array;
