@@ -5,13 +5,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <gtk/gtk.h>
+
 #include "nsIdleServiceGTK.h"
 #include "nsIServiceManager.h"
 #include "nsDebug.h"
 #include "prlink.h"
 #include "prlog.h"
 
-
 #ifdef PR_LOGGING
 static PRLogModuleInfo* sIdleLog = nullptr;
 #endif
@@ -102,7 +103,7 @@ nsIdleServiceGTK::PollIdleTime(uint32_t *aIdleTime)
     *aIdleTime = 0;
 
     // We might not have a display (cf. in xpcshell)
-    Display *dplay = GDK_DISPLAY();
+    Display *dplay = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
     if (!dplay) {
 #ifdef PR_LOGGING
         PR_LOG(sIdleLog, PR_LOG_WARNING, ("No display found!\n"));
