@@ -6,6 +6,7 @@
 #include "mozilla/Mutex.h"
 #include "mozilla/Attributes.h"
 #include "nsStreamUtils.h"
+#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "nsIPipe.h"
 #include "nsIEventTarget.h"
@@ -45,12 +46,11 @@ private:
         bool val;
         nsresult rv = mTarget->IsOnCurrentThread(&val);
         if (NS_FAILED(rv) || !val) {
-            nsCOMPtr<nsIInputStreamCallback> event;
-            NS_NewInputStreamReadyEvent(getter_AddRefs(event), mCallback,
-                                        mTarget);
-            mCallback = 0;
+            nsCOMPtr<nsIInputStreamCallback> event =
+                NS_NewInputStreamReadyEvent(mCallback, mTarget);
+            mCallback = nullptr;
             if (event) {
-                rv = event->OnInputStreamReady(nsnull);
+                rv = event->OnInputStreamReady(nullptr);
                 if (NS_FAILED(rv)) {
                     NS_NOTREACHED("leaking stream event");
                     nsISupports *sup = event;
@@ -80,7 +80,7 @@ public:
         if (mCallback) {
             if (mStream)
                 mCallback->OnInputStreamReady(mStream);
-            mCallback = nsnull;
+            mCallback = nullptr;
         }
         return NS_OK;
     }
@@ -124,12 +124,11 @@ private:
         bool val;
         nsresult rv = mTarget->IsOnCurrentThread(&val);
         if (NS_FAILED(rv) || !val) {
-            nsCOMPtr<nsIOutputStreamCallback> event;
-            NS_NewOutputStreamReadyEvent(getter_AddRefs(event), mCallback,
-                                         mTarget);
-            mCallback = 0;
+            nsCOMPtr<nsIOutputStreamCallback> event =
+                NS_NewOutputStreamReadyEvent(mCallback, mTarget);
+            mCallback = nullptr;
             if (event) {
-                rv = event->OnOutputStreamReady(nsnull);
+                rv = event->OnOutputStreamReady(nullptr);
                 if (NS_FAILED(rv)) {
                     NS_NOTREACHED("leaking stream event");
                     nsISupports *sup = event;
@@ -159,7 +158,7 @@ public:
         if (mCallback) {
             if (mStream)
                 mCallback->OnOutputStreamReady(mStream);
-            mCallback = nsnull;
+            mCallback = nullptr;
         }
         return NS_OK;
     }
@@ -175,32 +174,26 @@ NS_IMPL_THREADSAFE_ISUPPORTS2(nsOutputStreamReadyEvent, nsIRunnable,
 
 //-----------------------------------------------------------------------------
 
-nsresult
-NS_NewInputStreamReadyEvent(nsIInputStreamCallback **event,
-                            nsIInputStreamCallback *callback,
+already_AddRefed<nsIInputStreamCallback>
+NS_NewInputStreamReadyEvent(nsIInputStreamCallback *callback,
                             nsIEventTarget *target)
 {
     NS_ASSERTION(callback, "null callback");
     NS_ASSERTION(target, "null target");
-    nsInputStreamReadyEvent *ev = new nsInputStreamReadyEvent(callback, target);
-    if (!ev)
-        return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(*event = ev);
-    return NS_OK;
+    nsRefPtr<nsInputStreamReadyEvent> ev =
+        new nsInputStreamReadyEvent(callback, target);
+    return ev.forget();
 }
 
-nsresult
-NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback **event,
-                             nsIOutputStreamCallback *callback,
+already_AddRefed<nsIOutputStreamCallback>
+NS_NewOutputStreamReadyEvent(nsIOutputStreamCallback *callback,
                              nsIEventTarget *target)
 {
     NS_ASSERTION(callback, "null callback");
     NS_ASSERTION(target, "null target");
-    nsOutputStreamReadyEvent *ev = new nsOutputStreamReadyEvent(callback, target);
-    if (!ev)
-        return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(*event = ev);
-    return NS_OK;
+    nsRefPtr<nsOutputStreamReadyEvent> ev =
+        new nsOutputStreamReadyEvent(callback, target);
+    return ev.forget();
 }
 
 //-----------------------------------------------------------------------------
@@ -216,8 +209,9 @@ public:
 
     nsAStreamCopier()
         : mLock("nsAStreamCopier.mLock")
-        , mCallback(nsnull)
-        , mClosure(nsnull)
+        , mCallback(nullptr)
+        , mProgressCallback(nullptr)
+        , mClosure(nullptr)
         , mChunkSize(0)
         , mEventInProcess(false)
         , mEventIsPending(false)
@@ -239,9 +233,10 @@ public:
                    nsIEventTarget *target,
                    nsAsyncCopyCallbackFun callback,
                    void *closure,
-                   PRUint32 chunksize,
+                   uint32_t chunksize,
                    bool closeSource,
-                   bool closeSink)
+                   bool closeSink,
+                   nsAsyncCopyProgressFun progressCallback)
     {
         mSource = source;
         mSink = sink;
@@ -251,6 +246,7 @@ public:
         mChunkSize = chunksize;
         mCloseSource = closeSource;
         mCloseSink = closeSink;
+        mProgressCallback = progressCallback;
 
         mAsyncSource = do_QueryInterface(mSource);
         mAsyncSink = do_QueryInterface(mSink);
@@ -260,7 +256,7 @@ public:
 
     // implemented by subclasses, returns number of bytes copied and
     // sets source and sink condition before returning.
-    virtual PRUint32 DoCopy(nsresult *sourceCondition, nsresult *sinkCondition) = 0;
+    virtual uint32_t DoCopy(nsresult *sourceCondition, nsresult *sinkCondition) = 0;
 
     void Process()
     {
@@ -284,7 +280,10 @@ public:
             //       because we have consumed all of our data.
             bool copyFailed = false;
             if (!canceled) {
-                PRUint32 n = DoCopy(&sourceCondition, &sinkCondition);
+                uint32_t n = DoCopy(&sourceCondition, &sinkCondition);
+                if (n > 0 && mProgressCallback) {
+                    mProgressCallback(mClosure, n);
+                }
                 copyFailed = NS_FAILED(sourceCondition) ||
                              NS_FAILED(sinkCondition) || n == 0;
 
@@ -296,24 +295,24 @@ public:
                 if (sourceCondition == NS_BASE_STREAM_WOULD_BLOCK && mAsyncSource) {
                     // need to wait for more data from source.  while waiting for
                     // more source data, be sure to observe failures on output end.
-                    mAsyncSource->AsyncWait(this, 0, 0, nsnull);
+                    mAsyncSource->AsyncWait(this, 0, 0, nullptr);
 
                     if (mAsyncSink)
                         mAsyncSink->AsyncWait(this,
                                               nsIAsyncOutputStream::WAIT_CLOSURE_ONLY,
-                                              0, nsnull);
+                                              0, nullptr);
                     break;
                 }
                 else if (sinkCondition == NS_BASE_STREAM_WOULD_BLOCK && mAsyncSink) {
                     // need to wait for more room in the sink.  while waiting for
                     // more room in the sink, be sure to observer failures on the
                     // input end.
-                    mAsyncSink->AsyncWait(this, 0, 0, nsnull);
+                    mAsyncSink->AsyncWait(this, 0, 0, nullptr);
 
                     if (mAsyncSource)
                         mAsyncSource->AsyncWait(this,
                                                 nsIAsyncInputStream::WAIT_CLOSURE_ONLY,
-                                                0, nsnull);
+                                                0, nullptr);
                     break;
                 }
             }
@@ -326,8 +325,8 @@ public:
                     else
                         mSource->Close();
                 }
-                mAsyncSource = nsnull;
-                mSource = nsnull;
+                mAsyncSource = nullptr;
+                mSource = nullptr;
 
                 if (mCloseSink) {
                     // close sink
@@ -347,8 +346,8 @@ public:
                             mSink->Close();
                     }
                 }
-                mAsyncSink = nsnull;
-                mSink = nsnull;
+                mAsyncSink = nullptr;
+                mSink = nullptr;
 
                 // notify state complete...
                 if (mCallback) {
@@ -449,8 +448,9 @@ protected:
     nsCOMPtr<nsIEventTarget>       mTarget;
     Mutex                          mLock;
     nsAsyncCopyCallbackFun         mCallback;
+    nsAsyncCopyProgressFun         mProgressCallback;
     void                          *mClosure;
-    PRUint32                       mChunkSize;
+    uint32_t                       mChunkSize;
     bool                           mEventInProcess;
     bool                           mEventIsPending;
     bool                           mCloseSource;
@@ -478,9 +478,9 @@ public:
     static NS_METHOD ConsumeInputBuffer(nsIInputStream *inStr,
                                         void *closure,
                                         const char *buffer,
-                                        PRUint32 offset,
-                                        PRUint32 count,
-                                        PRUint32 *countWritten)
+                                        uint32_t offset,
+                                        uint32_t count,
+                                        uint32_t *countWritten)
     {
         ReadSegmentsState *state = (ReadSegmentsState *) closure;
 
@@ -493,13 +493,13 @@ public:
         return state->mSinkCondition;
     }
 
-    PRUint32 DoCopy(nsresult *sourceCondition, nsresult *sinkCondition)
+    uint32_t DoCopy(nsresult *sourceCondition, nsresult *sinkCondition)
     {
         ReadSegmentsState state;
         state.mSink = mSink;
         state.mSinkCondition = NS_OK;
 
-        PRUint32 n;
+        uint32_t n;
         *sourceCondition =
             mSource->ReadSegments(ConsumeInputBuffer, &state, mChunkSize, &n);
         *sinkCondition = state.mSinkCondition;
@@ -521,9 +521,9 @@ public:
     static NS_METHOD FillOutputBuffer(nsIOutputStream *outStr,
                                       void *closure,
                                       char *buffer,
-                                      PRUint32 offset,
-                                      PRUint32 count,
-                                      PRUint32 *countRead)
+                                      uint32_t offset,
+                                      uint32_t count,
+                                      uint32_t *countRead)
     {
         WriteSegmentsState *state = (WriteSegmentsState *) closure;
 
@@ -536,13 +536,13 @@ public:
         return state->mSourceCondition;
     }
 
-    PRUint32 DoCopy(nsresult *sourceCondition, nsresult *sinkCondition)
+    uint32_t DoCopy(nsresult *sourceCondition, nsresult *sinkCondition)
     {
         WriteSegmentsState state;
         state.mSource = mSource;
         state.mSourceCondition = NS_OK;
 
-        PRUint32 n;
+        uint32_t n;
         *sinkCondition =
             mSink->WriteSegments(FillOutputBuffer, &state, mChunkSize, &n);
         *sourceCondition = state.mSourceCondition;
@@ -557,12 +557,13 @@ NS_AsyncCopy(nsIInputStream         *source,
              nsIOutputStream        *sink,
              nsIEventTarget         *target,
              nsAsyncCopyMode         mode,
-             PRUint32                chunkSize,
+             uint32_t                chunkSize,
              nsAsyncCopyCallbackFun  callback,
              void                   *closure,
              bool                    closeSource,
              bool                    closeSink,
-             nsISupports           **aCopierCtx)
+             nsISupports           **aCopierCtx,
+             nsAsyncCopyProgressFun  progressCallback)
 {
     NS_ASSERTION(target, "non-null target required");
 
@@ -580,7 +581,7 @@ NS_AsyncCopy(nsIInputStream         *source,
     // Start() takes an owning ref to the copier...
     NS_ADDREF(copier);
     rv = copier->Start(source, sink, target, callback, closure, chunkSize,
-                       closeSource, closeSink);
+                       closeSource, closeSink, progressCallback);
 
     if (aCopierCtx) {
         *aCopierCtx = static_cast<nsISupports*>(
@@ -605,32 +606,35 @@ NS_CancelAsyncCopy(nsISupports *aCopierCtx, nsresult aReason)
 //-----------------------------------------------------------------------------
 
 nsresult
-NS_ConsumeStream(nsIInputStream *stream, PRUint32 maxCount, nsACString &result)
+NS_ConsumeStream(nsIInputStream *stream, uint32_t maxCount, nsACString &result)
 {
     nsresult rv = NS_OK;
     result.Truncate();
 
     while (maxCount) {
-        PRUint32 avail;
-        rv = stream->Available(&avail);
+        uint64_t avail64;
+        rv = stream->Available(&avail64);
         if (NS_FAILED(rv)) {
             if (rv == NS_BASE_STREAM_CLOSED)
                 rv = NS_OK;
             break;
         }
-        if (avail == 0)
+        if (avail64 == 0)
             break;
-        if (avail > maxCount)
-            avail = maxCount;
+
+        uint32_t avail = (uint32_t)XPCOM_MIN<uint64_t>(avail64, maxCount);
 
         // resize result buffer
-        PRUint32 length = result.Length();
+        uint32_t length = result.Length();
+        if (avail > UINT32_MAX - length)
+            return NS_ERROR_FILE_TOO_BIG;
+        
         result.SetLength(length + avail);
         if (result.Length() != (length + avail))
             return NS_ERROR_OUT_OF_MEMORY;
         char *buf = result.BeginWriting() + length;
         
-        PRUint32 n;
+        uint32_t n;
         rv = stream->Read(buf, avail, &n);
         if (NS_FAILED(rv))
             break;
@@ -650,9 +654,9 @@ static NS_METHOD
 TestInputStream(nsIInputStream *inStr,
                 void *closure,
                 const char *buffer,
-                PRUint32 offset,
-                PRUint32 count,
-                PRUint32 *countWritten)
+                uint32_t offset,
+                uint32_t count,
+                uint32_t *countWritten)
 {
     bool *result = static_cast<bool *>(closure);
     *result = true;
@@ -663,7 +667,7 @@ bool
 NS_InputStreamIsBuffered(nsIInputStream *stream)
 {
     bool result = false;
-    PRUint32 n;
+    uint32_t n;
     nsresult rv = stream->ReadSegments(TestInputStream,
                                        &result, 1, &n);
     return result || NS_SUCCEEDED(rv);
@@ -673,9 +677,9 @@ static NS_METHOD
 TestOutputStream(nsIOutputStream *outStr,
                  void *closure,
                  char *buffer,
-                 PRUint32 offset,
-                 PRUint32 count,
-                 PRUint32 *countRead)
+                 uint32_t offset,
+                 uint32_t count,
+                 uint32_t *countRead)
 {
     bool *result = static_cast<bool *>(closure);
     *result = true;
@@ -686,7 +690,7 @@ bool
 NS_OutputStreamIsBuffered(nsIOutputStream *stream)
 {
     bool result = false;
-    PRUint32 n;
+    uint32_t n;
     stream->WriteSegments(TestOutputStream, &result, 1, &n);
     return result;
 }
@@ -697,14 +701,14 @@ NS_METHOD
 NS_CopySegmentToStream(nsIInputStream *inStr,
                        void *closure,
                        const char *buffer,
-                       PRUint32 offset,
-                       PRUint32 count,
-                       PRUint32 *countWritten)
+                       uint32_t offset,
+                       uint32_t count,
+                       uint32_t *countWritten)
 {
     nsIOutputStream *outStr = static_cast<nsIOutputStream *>(closure);
     *countWritten = 0;
     while (count) {
-        PRUint32 n;
+        uint32_t n;
         nsresult rv = outStr->Write(buffer, count, &n);
         if (NS_FAILED(rv))
             return rv;
@@ -719,9 +723,9 @@ NS_METHOD
 NS_CopySegmentToBuffer(nsIInputStream *inStr,
                        void *closure,
                        const char *buffer,
-                       PRUint32 offset,
-                       PRUint32 count,
-                       PRUint32 *countWritten)
+                       uint32_t offset,
+                       uint32_t count,
+                       uint32_t *countWritten)
 {
     char *toBuf = static_cast<char *>(closure);
     memcpy(&toBuf[offset], buffer, count);
@@ -733,9 +737,9 @@ NS_METHOD
 NS_CopySegmentToBuffer(nsIOutputStream *outStr,
                        void *closure,
                        char *buffer,
-                       PRUint32 offset,
-                       PRUint32 count,
-                       PRUint32 *countRead)
+                       uint32_t offset,
+                       uint32_t count,
+                       uint32_t *countRead)
 {
     const char* fromBuf = static_cast<const char*>(closure);
     memcpy(buffer, &fromBuf[offset], count);
@@ -747,9 +751,9 @@ NS_METHOD
 NS_DiscardSegment(nsIInputStream *inStr,
                   void *closure,
                   const char *buffer,
-                  PRUint32 offset,
-                  PRUint32 count,
-                  PRUint32 *countWritten)
+                  uint32_t offset,
+                  uint32_t count,
+                  uint32_t *countWritten)
 {
     *countWritten = count;
     return NS_OK;
@@ -761,9 +765,9 @@ NS_METHOD
 NS_WriteSegmentThunk(nsIInputStream *inStr,
                      void *closure,
                      const char *buffer,
-                     PRUint32 offset,
-                     PRUint32 count,
-                     PRUint32 *countWritten)
+                     uint32_t offset,
+                     uint32_t count,
+                     uint32_t *countWritten)
 {
     nsWriteSegmentThunk *thunk = static_cast<nsWriteSegmentThunk *>(closure);
     return thunk->mFun(thunk->mStream, thunk->mClosure, buffer, offset, count,
