@@ -120,9 +120,9 @@ pageInfoTreeView.prototype = {
     this.tree.ensureRowIsVisible(0);
   },
 
-  getRowProperties: function(row, prop) { },
-  getCellProperties: function(row, column, prop) { },
-  getColumnProperties: function(column, prop) { },
+  getRowProperties: function(row) { return ""; },
+  getCellProperties: function(row, column) { return ""; },
+  getColumnProperties: function(column) { return ""; },
   isContainer: function(index) { return false; },
   isContainerOpen: function(index) { return false; },
   isSeparator: function(index) { return false; },
@@ -157,6 +157,8 @@ const COL_IMAGE_COUNT   = 4;
 const COL_IMAGE_NODE    = 5;
 const COL_IMAGE_BG      = 6;
 const COL_IMAGE_SIZENUM = 7;
+const COL_IMAGE_DEVICE  = 8;
+const COL_IMAGE_MIME    = 9;
 
 // column number to copy from, second argument to pageInfoTreeView's constructor
 const COPYCOL_NONE = -1;
@@ -173,28 +175,22 @@ var gFieldView = new pageInfoTreeView(COPYCOL_FIELD_VALUE);
 var gLinkView = new pageInfoTreeView(COPYCOL_LINK_ADDRESS);
 var gImageView = new pageInfoTreeView(COPYCOL_IMAGE);
 
-var gAtomSvc = Components.classes["@mozilla.org/atom-service;1"]
-                         .getService(Components.interfaces.nsIAtomService);
-var gBrokenAtom = gAtomSvc.getAtom("broken");
-var gLtrAtom = gAtomSvc.getAtom("ltr");
+gImageView.getCellProperties = function(row, col) {
+  var properties = col.id == "image-address" ? "ltr" : "";
 
-gImageView.getCellProperties = function(row, col, props) {
   if (gImageView.data[row][COL_IMAGE_SIZE] == gStrings.unknown &&
       !/^https:/.test(gImageView.data[row][COL_IMAGE_ADDRESS]))
-    props.AppendElement(gBrokenAtom);
+    properties += " broken";
 
-  if (col.id == "image-address")
-    props.AppendElement(gLtrAtom);
+  return properties;
 };
 
-gFormView.getCellProperties = function(row, col, props) {
-  if (col.id == "form-action")
-    props.AppendElement(gLtrAtom);
+gFormView.getCellProperties = function(row, col) {
+  return col.id == "form-action" ? "ltr" : "";
 };
 
-gLinkView.getCellProperties = function(row, col, props) {
-  if (col.id == "link-address")
-    props.AppendElement(gLtrAtom);
+gLinkView.getCellProperties = function(row, col) {
+  return col.id == "link-address" ? "ltr" : "";
 };
 
 gImageView.cycleHeader = function(col)
@@ -454,6 +450,19 @@ function onClickMore()
   showTab("securityTab");
 }
 
+var cacheListener = {
+  onCacheEntryAvailable: function onCacheEntryAvailable(descriptor) {
+    if (descriptor) {
+      var pageSize = descriptor.dataSize;
+      var kbSize = Math.round(pageSize / 1024 * 100) / 100;
+      var sizeText = gBundle.getFormattedString("generalSize",
+                                                [formatNumber(kbSize),
+                                                 formatNumber(pageSize)]);
+      setItemValue("sizetext", sizeText);
+    }
+  }
+};
+
 function makeGeneralTab()
 {
   var title = (gDocument.title) ? gBundle.getFormattedString("pageTitle", [gDocument.title]) : gBundle.getString("noPageTitle");
@@ -496,26 +505,16 @@ function makeGeneralTab()
   document.getElementById("modifiedtext").value = modifiedText;
 
   // get cache info
+  setItemValue("sizetext", null);
   var cacheKey = url.replace(/#.*$/, "");
   try {
-    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
+    httpCacheSession.asyncOpenCacheEntry(cacheKey, ACCESS_READ, cacheListener);
   }
-  catch(ex) {
-    try {
-      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
-    }
-    catch(ex2) { }
+  catch(ex) { }
+  try {
+    ftpCacheSession.asyncOpenCacheEntry(cacheKey, ACCESS_READ, cacheListener);
   }
-
-  var sizeText;
-  if (cacheEntryDescriptor) {
-    var pageSize = cacheEntryDescriptor.dataSize;
-    var kbSize = Math.round(pageSize / 1024 * 100) / 100;
-    sizeText = gBundle.getFormattedString("generalSize",
-                                          [formatNumber(kbSize),
-                                           formatNumber(pageSize)]);
-  }
-  setItemValue("sizetext", sizeText);
+  catch(ex) { }
 
   securityOnLoad();
 }
@@ -576,10 +575,22 @@ function ensureSelection(view)
     view.selection.select(0);
 }
 
-function addImage(url, type, alt, elem, isBg)
+function imgCacheListener(url, type, alt, elem, isBg)
 {
-  if (!url)
-    return;
+  this.url = url;
+  this.type = type;
+  this.alt = alt;
+  this.elem = elem;
+  this.isBg = isBg;
+}
+
+imgCacheListener.prototype.onCacheEntryAvailable =
+function onCacheEntryAvailable(cacheEntryDescriptor) {
+  var url = this.url;
+  var type = this.type;
+  var alt = this.alt;
+  var elem = this.elem;
+  var isBg = this.isBg;
 
   if (!gImageHash.hasOwnProperty(url))
     gImageHash[url] = { };
@@ -587,28 +598,21 @@ function addImage(url, type, alt, elem, isBg)
     gImageHash[url][type] = { };
   if (!gImageHash[url][type].hasOwnProperty(alt)) {
     gImageHash[url][type][alt] = gImageView.data.length;
-    try {
-      // open for READ, in non-blocking mode
-      var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, ACCESS_READ, false);
-    }
-    catch(ex) {
-      try {
-        // open for READ, in non-blocking mode
-        cacheEntryDescriptor = ftpCacheSession.openCacheEntry(url, ACCESS_READ, false);
-      }
-      catch(ex2) { }
-    }
 
     var sizeText;
     var pageSize;
+    var deviceID;
+    var mimeType;
     if (cacheEntryDescriptor) {
+      mimeType = getContentTypeFromHeaders(cacheEntryDescriptor);
+      deviceID = cacheEntryDescriptor.deviceID;
       pageSize = cacheEntryDescriptor.dataSize;
       var kbSize = Math.round(pageSize / 1024 * 100) / 100;
       sizeText = gBundle.getFormattedString("mediaFileSize", [formatNumber(kbSize)]);
     }
     else
       sizeText = gStrings.unknown;
-    gImageView.addRow([url, type, sizeText, alt, 1, elem, isBg, pageSize]);
+    gImageView.addRow([url, type, sizeText, alt, 1, elem, isBg, pageSize, deviceID, mimeType]);
 
     // Add the observer, only once.
     if (gImageView.data.length == 1) {
@@ -621,6 +625,15 @@ function addImage(url, type, alt, elem, isBg)
   }
 }
 
+function addImage(url, type, alt, elem, isBg)
+{
+  if (url) try {
+    var listener = new imgCacheListener(url, type, alt, elem, isBg);
+    httpCacheSession.asyncOpenCacheEntry(url, ACCESS_READ, listener);
+  }
+  catch (ex) { }
+}
+
 function grabAll(elem)
 {
   // check for background images, any node may have multiple
@@ -872,8 +885,17 @@ function saveMedia()
     var item = getSelectedImage(tree);
     var url = gImageView.data[tree.currentIndex][COL_IMAGE_ADDRESS];
 
-    if (url)
-      saveURL(url, null, "SaveImageTitle", false, true, makeURI(item.baseURI));
+    if (url) {
+      let titleKey = "SaveImageTitle";
+
+      if (item instanceof HTMLVideoElement)
+        titleKey = "SaveVideoTitle";
+      else if (item instanceof HTMLAudioElement)
+        titleKey = "SaveAudioTitle";
+
+      saveURL(url, null, titleKey, false, true, makeURI(item.baseURI),
+              gDocument);
+    }
   }
   else {
     var odir  = selectSaveFolder();
@@ -890,7 +912,7 @@ function saveMedia()
 
     var saveAnImage = function(aURIString, aChosenData, aBaseURI) {
       internalSave(aURIString, null, null, null, null, false, "SaveImageTitle",
-                   aChosenData, aBaseURI);
+                   aChosenData, aBaseURI, gDocument);
     }
 
     for (var i = 0; i < rowArray.length; i++) {
@@ -965,12 +987,7 @@ function onImageSelect()
 
 function makePreview(row)
 {
-  var imageTree = document.getElementById("imagetree");
-  var item = getSelectedImage(imageTree);
-  var col = imageTree.columns["image-address"];
-  var url = gImageView.getCellText(row, col);
-  // image-bg
-  var isBG = gImageView.data[row][COL_IMAGE_BG];
+  var [url, type, sizeText, alt, count, item, isBG, pageSize, deviceID, cachedType] = gImageView.data[row];
   var isAudio = false;
 
   setItemValue("imageurltext", url);
@@ -998,48 +1015,23 @@ function makePreview(row)
 
   // get cache info
   var sourceText = gBundle.getString("generalNotCached");
-  var cacheKey = url.replace(/#.*$/, "");
-  try {
-    // open for READ, in non-blocking mode
-    var cacheEntryDescriptor = httpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
-    if (cacheEntryDescriptor)
-      switch (cacheEntryDescriptor.deviceID) {
-        case "disk":
-          sourceText = gBundle.getString("generalDiskCache");
-          break;
-        case "memory":
-          sourceText = gBundle.getString("generalMemoryCache");
-          break;
-        default:
-          sourceText = cacheEntryDescriptor.deviceID;
-          break;
-      }
-  }
-  catch(ex) {
-    try {
-      // open for READ, in non-blocking mode
-      cacheEntryDescriptor = ftpCacheSession.openCacheEntry(cacheKey, ACCESS_READ, false);
-      if (cacheEntryDescriptor)
-        switch (cacheEntryDescriptor.deviceID) {
-          case "disk":
-            sourceText = gBundle.getString("generalDiskCache");
-            break;
-          case "memory":
-            sourceText = gBundle.getString("generalMemoryCache");
-            break;
-          default:
-            sourceText = cacheEntryDescriptor.deviceID;
-            break;
-        }
+  if (deviceID) {
+    switch (deviceID) {
+      case "disk":
+        sourceText = gBundle.getString("generalDiskCache");
+        break;
+      case "memory":
+        sourceText = gBundle.getString("generalMemoryCache");
+        break;
+      default:
+        sourceText = cacheEntryDescriptor.deviceID;
+        break;
     }
-    catch(ex2) { }
   }
   setItemValue("imagesourcetext", sourceText);
 
   // find out the file size
-  var sizeText;
-  if (cacheEntryDescriptor) {
-    var pageSize = cacheEntryDescriptor.dataSize;
+  if (pageSize) {
     var kbSize = Math.round(pageSize / 1024 * 100) / 100;
     sizeText = gBundle.getFormattedString("generalSize",
                                           [formatNumber(kbSize),
@@ -1048,15 +1040,25 @@ function makePreview(row)
   setItemValue("imagesizetext", sizeText);
 
   var mimeType;
-  var numFrames = 0;
-  if (item instanceof HTMLObjectElement ||
-      item instanceof HTMLEmbedElement ||
-      item instanceof HTMLLinkElement)
-    mimeType = item.type;
-  if (!mimeType && item instanceof nsIImageLoadingContent)
-    [mimeType, numFrames] = getContentTypeFromImgRequest(item);
+  var typeString = "mediaImageType";
+  if (!isBG) {
+    if (item instanceof nsIImageLoadingContent) {
+      var imageRequest = item.getRequest(nsIImageLoadingContent.CURRENT_REQUEST);
+      if (imageRequest) {
+        mimeType = imageRequest.mimeType;
+        if (imageRequest.imageStatus & imageRequest.STATUS_DECODE_COMPLETE &&
+            imageRequest.image.animated)
+          typeString = "mediaAnimatedType";
+      }
+    }
+    if (!mimeType &&
+        (item instanceof HTMLObjectElement ||
+         item instanceof HTMLEmbedElement ||
+         item instanceof HTMLLinkElement))
+      mimeType = item.type;
+  }
   if (!mimeType)
-    mimeType = getContentTypeFromHeaders(cacheEntryDescriptor);
+    mimeType = cachedType;
 
   // if we have a data url, get the MIME type from the url
   if (!mimeType) {
@@ -1071,11 +1073,7 @@ function makePreview(row)
     let imageMimeType = /^image\/(.*)/.exec(mimeType);
     if (imageMimeType) {
       imageType = imageMimeType[1].toUpperCase();
-      if (numFrames > 1)
-        imageType = gBundle.getFormattedString("mediaAnimatedImageType",
-                                               [imageType, numFrames]);
-      else
-        imageType = gBundle.getFormattedString("mediaImageType", [imageType]);
+      imageType = gBundle.getFormattedString(typeString, [imageType]);
     }
     else {
       // the MIME type doesn't begin with image/, display the raw type
@@ -1141,7 +1139,7 @@ function makePreview(row)
   else if (item instanceof HTMLVideoElement && isProtocolAllowed) {
     newImage = document.createElementNS("http://www.w3.org/1999/xhtml", "video");
     newImage.id = "thepreviewimage";
-    newImage.mozLoadFrom(item);
+    newImage.src = url;
     newImage.controls = true;
     width = physWidth = item.videoWidth;
     height = physHeight = item.videoHeight;
@@ -1237,21 +1235,6 @@ function getContentTypeFromHeaders(cacheEntryDescriptor)
           .exec(cacheEntryDescriptor.getMetaDataElement("response-head")))[1];
 }
 
-function getContentTypeFromImgRequest(item)
-{
-  var httpRequest;
-  var numFrames = 0;
-
-  var imageRequest = item.getRequest(nsIImageLoadingContent.CURRENT_REQUEST);
-  if (imageRequest) {
-    httpRequest = imageRequest.mimeType;
-    let image = imageRequest.image;
-    if (image)
-      numFrames = image.numFrames;
-  }
-  return [httpRequest, numFrames];
-}
-
 //******** Other Misc Stuff
 // Modified from the Links Panel v2.3, http://segment7.net/mozilla/links/links.html
 // parse a node to extract the contents of the node
