@@ -427,10 +427,8 @@ function BrowseMorePanels()
   var browser_url = "chrome://navigator/content/navigator.xul";
   var locale;
   try {
-    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-                          .getService(Components.interfaces.nsIPrefBranch);
-    url = prefs.getCharPref("sidebar.customize.more_panels.url");
-    var temp = prefs.getCharPref("browser.chromeURL");
+    url = Services.prefs.getCharPref("sidebar.customize.more_panels.url");
+    var temp = Services.prefs.getCharPref("browser.chromeURL");
     if (temp) browser_url = temp;
   } catch(ex) {
     debug("Unable to get prefs: "+ex);
