@@ -55,10 +55,10 @@ var gSyncSetup = {
 
   init: function () {
     let obs = [
-      ["weave:service:changepph:finish", "onResetPassphrase"],
-      ["weave:service:login:start",  "onLoginStart"],
-      ["weave:service:login:error",  "onLoginEnd"],
-      ["weave:service:login:finish", "onLoginEnd"]];
+      ["weave:service:changepassphrase", "onResetPassphrase"],
+      ["weave:service:login:start",      "onLoginStart"],
+      ["weave:service:login:error",      "onLoginEnd"],
+      ["weave:service:login:finish",     "onLoginEnd"]];
 
     // Add the observers now and remove them on unload
     let self = this;
@@ -124,13 +124,13 @@ var gSyncSetup = {
   resetPassphrase: function resetPassphrase() {
     // Apply the existing form fields so that
     // Weave.Service.changePassphrase() has the necessary credentials.
-    Weave.Identity.account = document.getElementById("existingAccountName").value;
-    Weave.Identity.basicPassword = document.getElementById("existingPassword").value;
+    Weave.Service.identity.account = document.getElementById("existingAccountName").value;
+    Weave.Service.identity.basicPassword = document.getElementById("existingPassword").value;
 
     // Generate a new passphrase so that Weave.Service.login() will
     // actually do something.
     let passphrase = Weave.Utils.generatePassphrase();
-    Weave.Identity.syncKey = passphrase;
+    Weave.Service.identity.syncKey = passphrase;
 
     // Only open the dialog if username + password are actually correct.
     Weave.Service.login();
@@ -154,7 +154,7 @@ var gSyncSetup = {
 
   onResetPassphrase: function () {
     document.getElementById("existingPassphrase").value =
-      Weave.Utils.hyphenatePassphrase(Weave.Identity.syncKey);
+      Weave.Utils.hyphenatePassphrase(Weave.Service.identity.syncKey);
     this.checkFields();
     this.wizard.advance();
   },
@@ -287,7 +287,7 @@ var gSyncSetup = {
     this._setFeedbackMessage(feedback, valid, str);
     this.status.email = valid;
     if (valid)
-      Weave.Identity.account = value;
+      Weave.Service.identity.account = value;
     this.checkFields();
   },
 
@@ -313,7 +313,7 @@ var gSyncSetup = {
 
   onPassphraseGenerate: function () {
     let passphrase = Weave.Utils.generatePassphrase();
-    Weave.Identity.syncKey = passphrase;
+    Weave.Service.identity.syncKey = passphrase;
     let el = document.getElementById("weavePassphrase");
     el.value = Weave.Utils.hyphenatePassphrase(passphrase);
   },
@@ -372,7 +372,7 @@ var gSyncSetup = {
         this.wizard.getButton("back").hidden = true;
         this.wizard.getButton("cancel").hidden = !this._resettingSync;
         this.wizard.getButton("extra1").hidden = true;
-        document.getElementById("syncComputerName").value = Weave.Clients.localName;
+        document.getElementById("syncComputerName").value = Weave.Service.clientsEngine.localName;
         document.getElementById("syncOptions").collapsed = this._resettingSync;
         document.getElementById("mergeOptions").collapsed = this._settingUpNew;
         break;
@@ -432,8 +432,8 @@ var gSyncSetup = {
                                                 challenge, response);
 
         if (error == null) {
-          Weave.Identity.account = email;
-          Weave.Identity.basicPassword = password;
+          Weave.Service.identity.account = email;
+          Weave.Service.identity.basicPassword = password;
           this._handleNoScript(false);
           this.wizard.pageIndex = SETUP_SUCCESS_PAGE;
           return false;
@@ -449,11 +449,11 @@ var gSyncSetup = {
         this.captchaBrowser.loadURI(Weave.Service.miscAPI + "captcha_html");
         break;
       case EXISTING_ACCOUNT_LOGIN_PAGE:
-        Weave.Identity.account = Weave.Utils.normalizeAccount(
+        Weave.Service.identity.account = Weave.Utils.normalizeAccount(
           document.getElementById("existingAccountName").value);
-        Weave.Identity.basicPassword = document.getElementById("existingPassword").value;
+        Weave.Service.identity.basicPassword = document.getElementById("existingPassword").value;
         let pp = document.getElementById("existingPassphrase").value;
-        Weave.Identity.syncKey = Weave.Utils.normalizePassphrase(pp);
+        Weave.Service.identity.syncKey = Weave.Utils.normalizePassphrase(pp);
         if (Weave.Service.login())
           this.wizard.pageIndex = SETUP_SUCCESS_PAGE;
         return false;
@@ -577,9 +577,9 @@ var gSyncSetup = {
       onPaired: function onPaired() {},
 
       onComplete: function onComplete(credentials) {
-        Weave.Identity.account = credentials.account;
-        Weave.Identity.basicPassword = credentials.password;
-        Weave.Identity.syncKey = credentials.synckey;
+        Weave.Service.identity.account = credentials.account;
+        Weave.Service.identity.basicPassword = credentials.password;
+        Weave.Service.identity.syncKey = credentials.synckey;
         Weave.Service.serverURL = credentials.serverURL;
         self.wizard.pageIndex = SETUP_SUCCESS_PAGE;
       },
@@ -799,7 +799,7 @@ var gSyncSetup = {
         let places_db = PlacesUtils.history
                                    .QueryInterface(Ci.nsPIPlacesDatabase)
                                    .DBConnection;
-        if (Weave.Engines.get("history").enabled) {
+        if (Weave.Service.engineManager.get("history").enabled) {
           let daysOfHistory = 0;
           let stm = places_db.createStatement(
             "SELECT ROUND(( " +
@@ -822,7 +822,7 @@ var gSyncSetup = {
           document.getElementById("historyCount").hidden = true;
         }
 
-        if (Weave.Engines.get("bookmarks").enabled) {
+        if (Weave.Service.engineManager.get("bookmarks").enabled) {
           let bookmarks = 0;
           let stm = places_db.createStatement(
             "SELECT count(*) AS bookmarks " +
@@ -842,7 +842,7 @@ var gSyncSetup = {
           document.getElementById("bookmarkCount").hidden = true;
         }
 
-        if (Weave.Engines.get("passwords").enabled) {
+        if (Weave.Service.engineManager.get("passwords").enabled) {
           let logins = Services.logins.getAllLogins({});
           // Support %S for historical reasons (see bug 600141)
           document.getElementById("passwordCount").value =
@@ -854,7 +854,24 @@ var gSyncSetup = {
           document.getElementById("passwordCount").hidden = true;
         }
 
-        if (!Weave.Engines.get("prefs").enabled) {
+        let addonsEngine = Weave.Service.engineManager.get("addons");
+        if (addonsEngine.enabled) {
+          let ids = addonsEngine._store.getAllIDs();
+          let blessedcount = 0;
+          for each (let i in ids) {
+            if (i)
+              blessedcount++;
+          }
+          // Bug 600141 does not apply as this does not have to support existing strings.
+          document.getElementById("addonCount").value =
+            PluralForm.get(blessedcount,
+                           this._stringBundle.GetStringFromName("addonsCount.label"))
+                      .replace("#1", blessedcount);
+        } else {
+          document.getElementById("addonCount").hidden = true;
+        }
+
+        if (!Weave.Service.engineManager.get("prefs").enabled) {
           document.getElementById("prefsWipe").hidden = true;
         }
 
@@ -872,9 +889,9 @@ var gSyncSetup = {
           box.appendChild(node);
         }
 
-        for each (let name in Weave.Clients.stats.names) {
+        for each (let name in Weave.Service.clientsEngine.stats.names) {
           // Don't list the current client
-          if (name == Weave.Clients.localName)
+          if (name == Weave.Service.clientsEngine.localName)
             continue;
 
           // Only show the first several client names
